package generated.eGKmitServices.terminal.arztPC;

import swt.util.ByteArray;
import java.nio.charset.Charset;

/**
 * Class for T(L)V encoding of messages and other classes
 * 
 */
public class Coding {

	/**
	 * offset into encodeDestination for writing/reading
	 */
	private short encoding_length;

	/**
	 * array used for encoding/decoding
	 */
	private byte[] encodeDestination;

	/**
	 * Singleton instance
	 */
	private static Coding the_instance;

	private static Charset charset = Charset.forName("UTF-8");

	/**
	 * Singleton accessor
	 * 
	 * @return Singleton instance
	 */

	public static Coding getInstance() {
		if (the_instance == null)
			the_instance = new Coding();
		return the_instance;
	}
	/**
	 * return offset into encodeDestination
	 * 
	 * @return value of encoding_length
	 */
	public short getEncodingLength() {
		return encoding_length;
	}

	/**
	 * initializes encoding. This method must be called before encoding an object.
	 * The offset (encoding_length) is reset to 0 and encodeDestination is set to dest.
	 * 
	 * @param dest The (empty) destination array for encoding
	 */
	public void encodeInit(byte[] dest) {
		encodeDestination = dest;
		encoding_length = 0;
	}
	/**
	 * initializes decoding. The offset (encoding_length) is reset to 0.
	 */
	public void decodeInit() {
		encoding_length = 0;
	}

	/**
	 * initializes encoding and encodes an object.
	 * 
	 * @param o The object to encode
	 * @param dest destination array for the encoding
	 * @return length of the encoded object
	 */

	/**
	 * decodes an object beginning at given offset
	 * 
	 * @param in array containing encoded object
	 * @param offset where the encoded object begins
	 * @return the decoded object
	 */

	private byte decodeByte(byte[] in) throws TerminalException {
		if (in[encoding_length++] != Code.BYTE) {

			stop();
		}

		byte res = in[encoding_length++];
		return res;
	}

	private boolean decodeBoolean(byte[] in) throws TerminalException {
		if (in[encoding_length++] != Code.BOOLEAN) {
			stop();

		}
		boolean res = (in[encoding_length++] == 1 ? true : false);
		return res;
	}

	public Codeable decode(byte[] in, short offset) throws Exception {
		encoding_length = offset;
		return decode(in);
	}
	public byte[] stringToByte(String s) {
		byte[] b = s.getBytes(charset);
		byte[] ret = new byte[LengthConstants.LENGTHOFSTRING];
		int len = (b.length < LengthConstants.LENGTHOFSTRING)
				? b.length
				: LengthConstants.LENGTHOFSTRING;
		ByteArray.copy(b, 0, ret, 0, len);
		return ret;
	}

	public boolean isValidCardString(String s) {
		return (s.getBytes(charset).length <= LengthConstants.LENGTHOFSTRING);
	}

	public void encodeString(String i) {
		byte[] o = stringToByte(i);
		encodeDestination[encoding_length++] = Code.BYTEARRAY;
		ByteArray.setShort(encodeDestination, encoding_length,
				(short) (o.length));
		encoding_length += 2;
		ByteArray.copy(o, 0, encodeDestination, encoding_length, o.length);
		encoding_length += o.length;
	}

	private void compSizeString(String i) {
		byte[] o = stringToByte(i);
		encoding_length += 3;
		encoding_length += o.length;
	}

	public short encode(Codeable o, byte[] dest) throws Exception {
		encodeInit(dest);

		encode(o);
		return computeSize(o);

	}

	/**
	 * computes the encoding length of an object without actually encoding it.
	 * 
	 * @param o The object of which the encoding length should be computed
	 * @return encoding length
	 */

	public short computeSize(Codeable o) {
		encoding_length = 0;
		try {
			compSize(o);
			return encoding_length;
		} catch (Exception e) {
			return -1;
		}
	}

	public void encodeByte(byte o) {
		encodeDestination[encoding_length++] = Code.BYTE;
		encodeDestination[encoding_length++] = o;
	}

	private void encodeBoolean(boolean o) {
		encodeDestination[encoding_length++] = Code.BOOLEAN;
		encodeDestination[encoding_length++] = (byte) (o ? 1 : 0);
	}

	private void compSizeByte(byte o) {
		encoding_length += 2;
	}

	public void encodeInt(int i) {
		short o = (short) i;
		if (i != (int) o)
			System.err.println("int too big for cast into short.");
		encodeDestination[encoding_length++] = Code.SHORT;
		ByteArray.setShort(encodeDestination, encoding_length, o);
		encoding_length += 2;
	}

	private void compSizeInt(int i) {
		short o = (short) i;
		encoding_length += 3;
	}
	public int decodeInt(byte[] in) {
		short res = ByteArray.getShort(in, ++encoding_length);
		encoding_length += 2;
		return (int) res;
	}

	public byte[] decodeByteArray(byte[] in) {
		short len = ByteArray.getShort(in, ++encoding_length);
		byte[] dest = new byte[len];
		encoding_length += 2;
		ByteArray.copy(in, encoding_length, dest, 0, len);
		encoding_length += len;
		return dest;
	}

	public String byteToString(byte[] b) {
		int len = b.length;
		while (len > 0 && b[len - 1] == 0)
			len--;
		return new String(ByteArray.subarray(b, 0, len), charset);
	}

	public String decodeString(byte[] in) {
		short len = ByteArray.getShort(in, ++encoding_length);
		byte[] dest = new byte[len];
		encoding_length += 2;
		ByteArray.copy(in, encoding_length, dest, 0, len);
		encoding_length += len;
		String str = byteToString(dest);
		return str;
	}

	private void compSizeBoolean(boolean o) {
		encoding_length += 2;
	}

	public byte[] encodeSingleString(String s) {
		byte[] o = stringToByte(s);
		byte[] dest = new byte[3 + o.length];
		int i = 0;
		dest[i++] = Code.BYTEARRAY;
		ByteArray.setShort(dest, i, (short) (o.length));
		i += 2;
		ByteArray.copy(o, 0, dest, i, o.length);
		i += o.length;
		return dest;
	}

	public void encodeByteArray(byte[] o) {

		//byte[] o = stringToByte(i);

		encodeDestination[encoding_length++] = Code.BYTEARRAY;

		ByteArray.setShort(encodeDestination, encoding_length,
				(short) (o.length));
		encoding_length += 2;

		ByteArray.copy(o, 0, encodeDestination, encoding_length, o.length);

		encoding_length += o.length;
	}

	private void compSizeByteArray(byte[] o) {
		encoding_length += 3;
		encoding_length += o.length;
	}

	/**
	 * encodes an object without initializing destination and offset.
	 * 
	 * @param c object to encode
	 */

	public void encode(Codeable c) throws Exception {

		switch (c.getCode()) {
			case Code.AUTHENTISIEREHBA :
				encodeAuthentisiereHBA((AuthentisiereHBA) c);
				break;
			case Code.AUTHENTISIEREEGK :
				encodeAuthentisiereEGK((AuthentisiereEGK) c);
				break;
			case Code.USTARTAUTH :
				encodeUStartAuth((UStartAuth) c);
				break;
			case Code.RETAUTHENTISIEREHBA :
				encodeRetAuthentisiereHBA((RetAuthentisiereHBA) c);
				break;
			case Code.RETAUTHENTISIEREEGK :
				encodeRetAuthentisiereEGK((RetAuthentisiereEGK) c);
				break;
			case Code.SENDECHALLENGEHBA :
				encodeSendeChallengeHBA((SendeChallengeHBA) c);
				break;
			case Code.SENDECHALLENGEEGK :
				encodeSendeChallengeEGK((SendeChallengeEGK) c);
				break;
			case Code.RETSENDECHALLENGEHBA :
				encodeRetSendeChallengeHBA((RetSendeChallengeHBA) c);
				break;
			case Code.RETSENDECHALLENGEEGK :
				encodeRetSendeChallengeEGK((RetSendeChallengeEGK) c);
				break;
			case Code.UENDAUTH :
				encodeUEndAuth((UEndAuth) c);
				break;
			case Code.USTARTSCHLUESSELAUSTAUSCH :
				encodeUStartSchluesselaustausch((UStartSchluesselaustausch) c);
				break;
			case Code.ERFRAGENONCEEGK :
				encodeErfrageNonceEGK((ErfrageNonceEGK) c);
				break;
			case Code.SENDENONCEEGK :
				encodeSendeNonceEGK((SendeNonceEGK) c);
				break;
			case Code.BESTAETIGESCHLUESSELEGK :
				encodeBestaetigeSchluesselEGK((BestaetigeSchluesselEGK) c);
				break;
			case Code.ERFRAGENONCEHBA :
				encodeErfrageNonceHBA((ErfrageNonceHBA) c);
				break;
			case Code.SENDENONCEHBA :
				encodeSendeNonceHBA((SendeNonceHBA) c);
				break;
			case Code.BESTAETIGESCHLUESSELHBA :
				encodeBestaetigeSchluesselHBA((BestaetigeSchluesselHBA) c);
				break;
			case Code.USCHLUESSELAUSTAUSCHOK :
				encodeUSchluesselaustauschOK((USchluesselaustauschOK) c);
				break;
			case Code.UVALIDATEPINEGK :
				encodeUValidatePINEGK((UValidatePINEGK) c);
				break;
			case Code.UBESTAETIGEPINEGK :
				encodeUBestaetigePINEGK((UBestaetigePINEGK) c);
				break;
			case Code.ERFRAGENONCEEGKPIN :
				encodeErfrageNonceEGKpin((ErfrageNonceEGKpin) c);
				break;
			case Code.SENDENONCEEGKPIN :
				encodeSendeNonceEGKpin((SendeNonceEGKpin) c);
				break;
			case Code.SENDEPINEGK :
				encodeSendePINEGK((SendePINEGK) c);
				break;
			case Code.BESTAETIGEPINEGK :
				encodeBestaetigePINEGK((BestaetigePINEGK) c);
				break;
			case Code.UVALIDATEPINHBA :
				encodeUValidatePINHBA((UValidatePINHBA) c);
				break;
			case Code.UBESTAETIGEPINHBA :
				encodeUBestaetigePINHBA((UBestaetigePINHBA) c);
				break;
			case Code.ERFRAGENONCEHBAPIN :
				encodeErfrageNonceHBApin((ErfrageNonceHBApin) c);
				break;
			case Code.SENDENONCEHBAPIN :
				encodeSendeNonceHBApin((SendeNonceHBApin) c);
				break;
			case Code.SENDEPINHBA :
				encodeSendePINHBA((SendePINHBA) c);
				break;
			case Code.BESTAETIGEPINHBA :
				encodeBestaetigePINHBA((BestaetigePINHBA) c);
				break;
			case Code.SENDENONCEREZEPT :
				encodeSendeNonceRezept((SendeNonceRezept) c);
				break;
			case Code.USENDEREZEPTDATEN :
				encodeUSendeRezeptdaten((USendeRezeptdaten) c);
				break;
			case Code.SCHICKEREZEPTDATEN :
				encodeSchickeRezeptdaten((SchickeRezeptdaten) c);
				break;
			case Code.SENDEREZEPTDATENSIGNATUR :
				encodeSendeRezeptdatensignatur((SendeRezeptdatensignatur) c);
				break;
			case Code.SENDEREZEPT :
				encodeSendeRezept((SendeRezept) c);
				break;
			case Code.UBESTAETIGEREZEPTAUSSTELLEN :
				encodeUBestaetigeRezeptAusstellen((UBestaetigeRezeptAusstellen) c);
				break;
			case Code.ERFRAGENONCEREZEPT :
				encodeErfrageNonceRezept((ErfrageNonceRezept) c);
				break;
			case Code.BESTAETIGEREZEPTAUSSTELLEN :
				encodeBestaetigeRezeptAusstellen((BestaetigeRezeptAusstellen) c);
				break;
			case Code.ENDAUTH :
				encodeEndAuth((EndAuth) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTHBA :
				encodeSendeNonceVerschluesseltHBA((SendeNonceVerschluesseltHBA) c);
				break;
			case Code.SENDESCHLUESSEL :
				encodeSendeSchluessel((SendeSchluessel) c);
				break;
			case Code.ERFRAGENONCEEGKWEITER :
				encodeErfrageNonceEGKWeiter((ErfrageNonceEGKWeiter) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTEGK :
				encodeSendeNonceVerschluesseltEGK((SendeNonceVerschluesseltEGK) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTEGKWEITER :
				encodeSendeNonceVerschluesseltEGKWeiter((SendeNonceVerschluesseltEGKWeiter) c);
				break;
			case Code.SENDESESSIONKEY :
				encodeSendeSessionkey((SendeSessionkey) c);
				break;
			case Code.UBESTAETIGESCHLUESSELAUSTAUSCH :
				encodeUBestaetigeSchluesselaustausch((UBestaetigeSchluesselaustausch) c);
				break;
			case Code.BESTAETIGESCHLUESSELAUSTAUSCH :
				encodeBestaetigeSchluesselaustausch((BestaetigeSchluesselaustausch) c);
				break;
			case Code.SENDECHALLENGEHBAWEITER :
				encodeSendeChallengeHBAWeiter((SendeChallengeHBAWeiter) c);
				break;
			case Code.SENDECHALLENGEEGKWEITER :
				encodeSendeChallengeEGKWeiter((SendeChallengeEGKWeiter) c);
				break;
			case Code.RETAUTHENTISIEREEGKWEITER :
				encodeRetAuthentisiereEGKWeiter((RetAuthentisiereEGKWeiter) c);
				break;
			case Code.RETSENDECHALLENGEEGKWEITER :
				encodeRetSendeChallengeEGKWeiter((RetSendeChallengeEGKWeiter) c);
				break;
			case Code.SENDEZERTIFIKATHBA :
				encodeSendeZertifikatHBA((SendeZertifikatHBA) c);
				break;
			case Code.SENDESESSIONKEYWEITER :
				encodeSendeSessionkeyWeiter((SendeSessionkeyWeiter) c);
				break;
			case Code.UREZEPTEINLOESEN :
				encodeURezeptEinloesen((URezeptEinloesen) c);
				break;
			case Code.UREZEPTAUSGABE :
				encodeURezeptausgabe((URezeptausgabe) c);
				break;
			case Code.ERFRAGENONCEFUEREINLOESEN :
				encodeErfrageNonceFuerEinloesen((ErfrageNonceFuerEinloesen) c);
				break;
			case Code.SENDENONCEFUEREINLOESEN :
				encodeSendeNonceFuerEinloesen((SendeNonceFuerEinloesen) c);
				break;
			case Code.EINLOESEN :
				encodeEinloesen((Einloesen) c);
				break;
			case Code.SENDEREZEPTZUMEINLOESEN :
				encodeSendeRezeptZumEinloesen((SendeRezeptZumEinloesen) c);
				break;
			case Code.UERSTELLENOTFALLDATEN :
				encodeUErstelleNotfalldaten((UErstelleNotfalldaten) c);
				break;
			case Code.ERFRAGENONCENOTFALLDATENERSTELLEN :
				encodeErfrageNonceNotfalldatenErstellen((ErfrageNonceNotfalldatenErstellen) c);
				break;
			case Code.SENDENONCENOTFALLDATENERSTELLEN :
				encodeSendeNonceNotfalldatenErstellen((SendeNonceNotfalldatenErstellen) c);
				break;
			case Code.SENDENOTFALLDATEN :
				encodeSendeNotfalldaten((SendeNotfalldaten) c);
				break;
			case Code.SENDENOTFALLDATENSIGNATUR :
				encodeSendeNotfalldatensignatur((SendeNotfalldatensignatur) c);
				break;
			case Code.SENDENOTFALLDATENUNDSIG :
				encodeSendeNotfalldatenUndSig((SendeNotfalldatenUndSig) c);
				break;
			case Code.BESTAETIGENOTFALLDATENERSTELLEN :
				encodeBestaetigeNotfalldatenErstellen((BestaetigeNotfalldatenErstellen) c);
				break;
			case Code.UBESTAETIGENOTFALLDATENERSTELLEN :
				encodeUBestaetigeNotfalldatenErstellen((UBestaetigeNotfalldatenErstellen) c);
				break;
			case Code.ULESENOTFALLDATEN :
				encodeULeseNotfalldaten((ULeseNotfalldaten) c);
				break;
			case Code.PINARZTEINGEGEBEN :
				encodePINArztEingegeben((PINArztEingegeben) c);
				break;
			case Code.PINOK :
				encodePINOK((PINOK) c);
				break;
			case Code.ERFRAGENOTFALLDATEN :
				encodeErfrageNotfalldaten((ErfrageNotfalldaten) c);
				break;
			case Code.SCHICKENOTFALLDATEN :
				encodeSchickeNotfalldaten((SchickeNotfalldaten) c);
				break;
			case Code.USENDENOTFALLDATEN :
				encodeUSendeNotfalldaten((USendeNotfalldaten) c);
				break;
			case Code.UAUTHEGKUNDKIOSK :
				encodeUAuthEGKUndKiosk((UAuthEGKUndKiosk) c);
				break;
			case Code.SENDEZERTKIOSK :
				encodeSendeZertKiosk((SendeZertKiosk) c);
				break;
			case Code.SENDEZERTEGK :
				encodeSendeZertEGK((SendeZertEGK) c);
				break;
			case Code.SENDENONCES :
				encodeSendeNonces((SendeNonces) c);
				break;
			case Code.BESTAETIGENONCE :
				encodeBestaetigeNonce((BestaetigeNonce) c);
				break;
			case Code.UAUTHOK :
				encodeUAuthOK((UAuthOK) c);
				break;
			case Code.UPINEINGABEKIOSK :
				encodeUPINEingabeKiosk((UPINEingabeKiosk) c);
				break;
			case Code.ERFRAGENONCEPIN :
				encodeErfrageNoncePIN((ErfrageNoncePIN) c);
				break;
			case Code.SENDENONCEPIN :
				encodeSendeNoncePIN((SendeNoncePIN) c);
				break;
			case Code.SENDEPIN :
				encodeSendePIN((SendePIN) c);
				break;
			case Code.BESTAETIGEPIN :
				encodeBestaetigePIN((BestaetigePIN) c);
				break;
			case Code.UBESTAETIGEPIN :
				encodeUBestaetigePIN((UBestaetigePIN) c);
				break;
			case Code.USCHLUESSELAUSTAUSCH :
				encodeUSchluesselaustausch((USchluesselaustausch) c);
				break;
			case Code.ERFRAGENONCESCHLUESSELAUSTAUSCH :
				encodeErfrageNonceSchluesselaustausch((ErfrageNonceSchluesselaustausch) c);
				break;
			case Code.SENDENONCESCHLUESSELAUSTAUSCH :
				encodeSendeNonceSchluesselaustausch((SendeNonceSchluesselaustausch) c);
				break;
			case Code.SENDESESSIONKEYKIOSK :
				encodeSendeSessionkeyKiosk((SendeSessionkeyKiosk) c);
				break;
			case Code.SCHLUESSELKIOSKOK :
				encodeSchluesselKioskOK((SchluesselKioskOK) c);
				break;
			case Code.USCHLUESSELKIOSKOK :
				encodeUSchluesselKioskOK((USchluesselKioskOK) c);
				break;
			case Code.ULESESTAMMUNDNOTFALLDATEN :
				encodeULeseStammUndNotfalldaten((ULeseStammUndNotfalldaten) c);
				break;
			case Code.LESESTAMMUNDNOTFALLDATEN :
				encodeLeseStammUndNotfalldaten((LeseStammUndNotfalldaten) c);
				break;
			case Code.SENDESTAMMUNDNOTFALLDATEN :
				encodeSendeStammUndNotfalldaten((SendeStammUndNotfalldaten) c);
				break;
			case Code.USENDESTAMMUNDNOTFALLDATEN :
				encodeUSendeStammUndNotfalldaten((USendeStammUndNotfalldaten) c);
				break;
			case Code.ULESEREZEPT :
				encodeULeseRezept((ULeseRezept) c);
				break;
			case Code.LESEREZEPT :
				encodeLeseRezept((LeseRezept) c);
				break;
			case Code.SENDEREZEPTDATEN :
				encodeSendeRezeptdaten((SendeRezeptdaten) c);
				break;
			case Code.USENDEREZEPT :
				encodeUSendeRezept((USendeRezept) c);
				break;
			case Code.UNOTFALLDATENLOESCHEN :
				encodeUNotfalldatenLoeschen((UNotfalldatenLoeschen) c);
				break;
			case Code.ERFRAGENONCENOTFALLDATENLOESCHEN :
				encodeErfrageNonceNotfalldatenLoeschen((ErfrageNonceNotfalldatenLoeschen) c);
				break;
			case Code.LOESCHENOTFALLDATEN :
				encodeLoescheNotfalldaten((LoescheNotfalldaten) c);
				break;
			case Code.BESTAETIGELOESCHENNOTFALLDATEN :
				encodeBestaetigeLoeschenNotfalldaten((BestaetigeLoeschenNotfalldaten) c);
				break;
			case Code.UBESTAETIGELOESCHENNOTFALLDATEN :
				encodeUBestaetigeLoeschenNotfalldaten((UBestaetigeLoeschenNotfalldaten) c);
				break;
			case Code.SENDENONCE :
				encodeSendeNonce((SendeNonce) c);
				break;
			case Code.UUSEROK :
				encodeUUserOK((UUserOK) c);
				break;
			case Code.UUSERUPDATE :
				encodeUUserUpdate((UUserUpdate) c);
				break;
			case Code.GETDATA :
				encodeGetData((GetData) c);
				break;
			case Code.OK :
				encodeOK((OK) c);
				break;
			case Code.CARDDATA :
				encodeCardData((CardData) c);
				break;
			case Code.UPDATE :
				encodeUpdate((Update) c);
				break;
			case Code.NONCE :
				encodeNonce((Nonce) c);
				break;
			case Code.ZERTIFIKATLEISTUNGSERBRINGER :
				encodeZertifikatLeistungserbringer((ZertifikatLeistungserbringer) c);
				break;
			case Code.ZERTIFIKATPATIENT :
				encodeZertifikatPatient((ZertifikatPatient) c);
				break;
			case Code.ENCDATAASYMM :
				encodeEncDataAsymm((EncDataAsymm) c);
				break;
			case Code.NONCEDATA :
				encodeNonceData((NonceData) c);
				break;
			case Code.SECRET :
				encodeSecret((Secret) c);
				break;
			case Code.PINDATA :
				encodePINData((PINData) c);
				break;
			case Code.ENCDATASYMM :
				encodeEncDataSymm((EncDataSymm) c);
				break;
			case Code.EGKDATEN :
				encodeEGKDaten((EGKDaten) c);
				break;
			case Code.ARZNEIMITTEL :
				encodeArzneimittel((Arzneimittel) c);
				break;
			case Code.EREZEPTDATEN :
				encodeERezeptdaten((ERezeptdaten) c);
				break;
			case Code.SIGNEDDATA :
				encodeSignedData((SignedData) c);
				break;
			case Code.SECRETNONCEDATA :
				encodeSecretNonceData((SecretNonceData) c);
				break;
			case Code.KEYDATA :
				encodeKeyData((KeyData) c);
				break;
			case Code.INDEXNONCE :
				encodeIndexNonce((IndexNonce) c);
				break;
			case Code.REZEPTDATENUNDNONCE :
				encodeRezeptdatenUndNonce((RezeptdatenUndNonce) c);
				break;
			case Code.NOTFALLDATEN :
				encodeNotfalldaten((Notfalldaten) c);
				break;
			case Code.NOTFALLDATENNONCE :
				encodeNotfalldatenNonce((NotfalldatenNonce) c);
				break;
			case Code.ZERTIFIKATKIOSK :
				encodeZertifikatKiosk((ZertifikatKiosk) c);
				break;
			case Code.STAMMUNDNOTFALLDATEN :
				encodeStammUndNotfalldaten((StammUndNotfalldaten) c);
				break;
			case Code.VERSICHERTENSTAMMDATEN :
				encodeVersichertenstammdaten((Versichertenstammdaten) c);
				break;
			case Code.NONCEDATAANDINDEX :
				encodeNonceDataAndIndex((NonceDataAndIndex) c);
				break;
			case Code.NONCEUNDREZEPT :
				encodeNonceUndRezept((NonceUndRezept) c);
				break;
			case Code.NONCESTRINGDATA :
				encodeNonceStringData((NonceStringData) c);
				break;
			case Code.UPDATEAUTH :
				encodeUpdateAuth((UpdateAuth) c);
				break;
			case Code.LEISTUNGSERBRINGERINFO :
				encodeLeistungserbringerInfo((LeistungserbringerInfo) c);
				break;
			case Code.ZERTIFIKATPATIENTDATA :
				encodeZertifikatPatientData((ZertifikatPatientData) c);
				break;
			case Code.SYMMKEY :
				encodeSymmKey((SymmKey) c);
				break;
			case Code.ZERTIFIKATKIOSKDATA :
				encodeZertifikatKioskData((ZertifikatKioskData) c);
				break;
			case Code.PUBLICKEY :
				encodePublicKey((PublicKey) c);
				break;
			case Code.ZERTIFIKATTRUSTEDTHIRDPARTY :
				encodeZertifikatTrustedThirdParty((ZertifikatTrustedThirdParty) c);
				break;
			case Code.PRIVATEKEY :
				encodePrivateKey((PrivateKey) c);
				break;
			case Code.ZERTIFIKATTTPDATA :
				encodeZertifikatTTPData((ZertifikatTTPData) c);
				break;
			// never reached
			default :

				stop();

		}
	}

	public Codeable decode(byte[] in) throws Exception {
		switch (in[encoding_length]) {
			case Code.AUTHENTISIEREHBA :
				return decodeAuthentisiereHBA(in);
			case Code.AUTHENTISIEREEGK :
				return decodeAuthentisiereEGK(in);
			case Code.USTARTAUTH :
				return decodeUStartAuth(in);
			case Code.RETAUTHENTISIEREHBA :
				return decodeRetAuthentisiereHBA(in);
			case Code.RETAUTHENTISIEREEGK :
				return decodeRetAuthentisiereEGK(in);
			case Code.SENDECHALLENGEHBA :
				return decodeSendeChallengeHBA(in);
			case Code.SENDECHALLENGEEGK :
				return decodeSendeChallengeEGK(in);
			case Code.RETSENDECHALLENGEHBA :
				return decodeRetSendeChallengeHBA(in);
			case Code.RETSENDECHALLENGEEGK :
				return decodeRetSendeChallengeEGK(in);
			case Code.UENDAUTH :
				return decodeUEndAuth(in);
			case Code.USTARTSCHLUESSELAUSTAUSCH :
				return decodeUStartSchluesselaustausch(in);
			case Code.ERFRAGENONCEEGK :
				return decodeErfrageNonceEGK(in);
			case Code.SENDENONCEEGK :
				return decodeSendeNonceEGK(in);
			case Code.BESTAETIGESCHLUESSELEGK :
				return decodeBestaetigeSchluesselEGK(in);
			case Code.ERFRAGENONCEHBA :
				return decodeErfrageNonceHBA(in);
			case Code.SENDENONCEHBA :
				return decodeSendeNonceHBA(in);
			case Code.BESTAETIGESCHLUESSELHBA :
				return decodeBestaetigeSchluesselHBA(in);
			case Code.USCHLUESSELAUSTAUSCHOK :
				return decodeUSchluesselaustauschOK(in);
			case Code.UVALIDATEPINEGK :
				return decodeUValidatePINEGK(in);
			case Code.UBESTAETIGEPINEGK :
				return decodeUBestaetigePINEGK(in);
			case Code.ERFRAGENONCEEGKPIN :
				return decodeErfrageNonceEGKpin(in);
			case Code.SENDENONCEEGKPIN :
				return decodeSendeNonceEGKpin(in);
			case Code.SENDEPINEGK :
				return decodeSendePINEGK(in);
			case Code.BESTAETIGEPINEGK :
				return decodeBestaetigePINEGK(in);
			case Code.UVALIDATEPINHBA :
				return decodeUValidatePINHBA(in);
			case Code.UBESTAETIGEPINHBA :
				return decodeUBestaetigePINHBA(in);
			case Code.ERFRAGENONCEHBAPIN :
				return decodeErfrageNonceHBApin(in);
			case Code.SENDENONCEHBAPIN :
				return decodeSendeNonceHBApin(in);
			case Code.SENDEPINHBA :
				return decodeSendePINHBA(in);
			case Code.BESTAETIGEPINHBA :
				return decodeBestaetigePINHBA(in);
			case Code.SENDENONCEREZEPT :
				return decodeSendeNonceRezept(in);
			case Code.USENDEREZEPTDATEN :
				return decodeUSendeRezeptdaten(in);
			case Code.SCHICKEREZEPTDATEN :
				return decodeSchickeRezeptdaten(in);
			case Code.SENDEREZEPTDATENSIGNATUR :
				return decodeSendeRezeptdatensignatur(in);
			case Code.SENDEREZEPT :
				return decodeSendeRezept(in);
			case Code.UBESTAETIGEREZEPTAUSSTELLEN :
				return decodeUBestaetigeRezeptAusstellen(in);
			case Code.ERFRAGENONCEREZEPT :
				return decodeErfrageNonceRezept(in);
			case Code.BESTAETIGEREZEPTAUSSTELLEN :
				return decodeBestaetigeRezeptAusstellen(in);
			case Code.ENDAUTH :
				return decodeEndAuth(in);
			case Code.SENDENONCEVERSCHLUESSELTHBA :
				return decodeSendeNonceVerschluesseltHBA(in);
			case Code.SENDESCHLUESSEL :
				return decodeSendeSchluessel(in);
			case Code.ERFRAGENONCEEGKWEITER :
				return decodeErfrageNonceEGKWeiter(in);
			case Code.SENDENONCEVERSCHLUESSELTEGK :
				return decodeSendeNonceVerschluesseltEGK(in);
			case Code.SENDENONCEVERSCHLUESSELTEGKWEITER :
				return decodeSendeNonceVerschluesseltEGKWeiter(in);
			case Code.SENDESESSIONKEY :
				return decodeSendeSessionkey(in);
			case Code.UBESTAETIGESCHLUESSELAUSTAUSCH :
				return decodeUBestaetigeSchluesselaustausch(in);
			case Code.BESTAETIGESCHLUESSELAUSTAUSCH :
				return decodeBestaetigeSchluesselaustausch(in);
			case Code.SENDECHALLENGEHBAWEITER :
				return decodeSendeChallengeHBAWeiter(in);
			case Code.SENDECHALLENGEEGKWEITER :
				return decodeSendeChallengeEGKWeiter(in);
			case Code.RETAUTHENTISIEREEGKWEITER :
				return decodeRetAuthentisiereEGKWeiter(in);
			case Code.RETSENDECHALLENGEEGKWEITER :
				return decodeRetSendeChallengeEGKWeiter(in);
			case Code.SENDEZERTIFIKATHBA :
				return decodeSendeZertifikatHBA(in);
			case Code.SENDESESSIONKEYWEITER :
				return decodeSendeSessionkeyWeiter(in);
			case Code.UREZEPTEINLOESEN :
				return decodeURezeptEinloesen(in);
			case Code.UREZEPTAUSGABE :
				return decodeURezeptausgabe(in);
			case Code.ERFRAGENONCEFUEREINLOESEN :
				return decodeErfrageNonceFuerEinloesen(in);
			case Code.SENDENONCEFUEREINLOESEN :
				return decodeSendeNonceFuerEinloesen(in);
			case Code.EINLOESEN :
				return decodeEinloesen(in);
			case Code.SENDEREZEPTZUMEINLOESEN :
				return decodeSendeRezeptZumEinloesen(in);
			case Code.UERSTELLENOTFALLDATEN :
				return decodeUErstelleNotfalldaten(in);
			case Code.ERFRAGENONCENOTFALLDATENERSTELLEN :
				return decodeErfrageNonceNotfalldatenErstellen(in);
			case Code.SENDENONCENOTFALLDATENERSTELLEN :
				return decodeSendeNonceNotfalldatenErstellen(in);
			case Code.SENDENOTFALLDATEN :
				return decodeSendeNotfalldaten(in);
			case Code.SENDENOTFALLDATENSIGNATUR :
				return decodeSendeNotfalldatensignatur(in);
			case Code.SENDENOTFALLDATENUNDSIG :
				return decodeSendeNotfalldatenUndSig(in);
			case Code.BESTAETIGENOTFALLDATENERSTELLEN :
				return decodeBestaetigeNotfalldatenErstellen(in);
			case Code.UBESTAETIGENOTFALLDATENERSTELLEN :
				return decodeUBestaetigeNotfalldatenErstellen(in);
			case Code.ULESENOTFALLDATEN :
				return decodeULeseNotfalldaten(in);
			case Code.PINARZTEINGEGEBEN :
				return decodePINArztEingegeben(in);
			case Code.PINOK :
				return decodePINOK(in);
			case Code.ERFRAGENOTFALLDATEN :
				return decodeErfrageNotfalldaten(in);
			case Code.SCHICKENOTFALLDATEN :
				return decodeSchickeNotfalldaten(in);
			case Code.USENDENOTFALLDATEN :
				return decodeUSendeNotfalldaten(in);
			case Code.UAUTHEGKUNDKIOSK :
				return decodeUAuthEGKUndKiosk(in);
			case Code.SENDEZERTKIOSK :
				return decodeSendeZertKiosk(in);
			case Code.SENDEZERTEGK :
				return decodeSendeZertEGK(in);
			case Code.SENDENONCES :
				return decodeSendeNonces(in);
			case Code.BESTAETIGENONCE :
				return decodeBestaetigeNonce(in);
			case Code.UAUTHOK :
				return decodeUAuthOK(in);
			case Code.UPINEINGABEKIOSK :
				return decodeUPINEingabeKiosk(in);
			case Code.ERFRAGENONCEPIN :
				return decodeErfrageNoncePIN(in);
			case Code.SENDENONCEPIN :
				return decodeSendeNoncePIN(in);
			case Code.SENDEPIN :
				return decodeSendePIN(in);
			case Code.BESTAETIGEPIN :
				return decodeBestaetigePIN(in);
			case Code.UBESTAETIGEPIN :
				return decodeUBestaetigePIN(in);
			case Code.USCHLUESSELAUSTAUSCH :
				return decodeUSchluesselaustausch(in);
			case Code.ERFRAGENONCESCHLUESSELAUSTAUSCH :
				return decodeErfrageNonceSchluesselaustausch(in);
			case Code.SENDENONCESCHLUESSELAUSTAUSCH :
				return decodeSendeNonceSchluesselaustausch(in);
			case Code.SENDESESSIONKEYKIOSK :
				return decodeSendeSessionkeyKiosk(in);
			case Code.SCHLUESSELKIOSKOK :
				return decodeSchluesselKioskOK(in);
			case Code.USCHLUESSELKIOSKOK :
				return decodeUSchluesselKioskOK(in);
			case Code.ULESESTAMMUNDNOTFALLDATEN :
				return decodeULeseStammUndNotfalldaten(in);
			case Code.LESESTAMMUNDNOTFALLDATEN :
				return decodeLeseStammUndNotfalldaten(in);
			case Code.SENDESTAMMUNDNOTFALLDATEN :
				return decodeSendeStammUndNotfalldaten(in);
			case Code.USENDESTAMMUNDNOTFALLDATEN :
				return decodeUSendeStammUndNotfalldaten(in);
			case Code.ULESEREZEPT :
				return decodeULeseRezept(in);
			case Code.LESEREZEPT :
				return decodeLeseRezept(in);
			case Code.SENDEREZEPTDATEN :
				return decodeSendeRezeptdaten(in);
			case Code.USENDEREZEPT :
				return decodeUSendeRezept(in);
			case Code.UNOTFALLDATENLOESCHEN :
				return decodeUNotfalldatenLoeschen(in);
			case Code.ERFRAGENONCENOTFALLDATENLOESCHEN :
				return decodeErfrageNonceNotfalldatenLoeschen(in);
			case Code.LOESCHENOTFALLDATEN :
				return decodeLoescheNotfalldaten(in);
			case Code.BESTAETIGELOESCHENNOTFALLDATEN :
				return decodeBestaetigeLoeschenNotfalldaten(in);
			case Code.UBESTAETIGELOESCHENNOTFALLDATEN :
				return decodeUBestaetigeLoeschenNotfalldaten(in);
			case Code.SENDENONCE :
				return decodeSendeNonce(in);
			case Code.UUSEROK :
				return decodeUUserOK(in);
			case Code.UUSERUPDATE :
				return decodeUUserUpdate(in);
			case Code.GETDATA :
				return decodeGetData(in);
			case Code.OK :
				return decodeOK(in);
			case Code.CARDDATA :
				return decodeCardData(in);
			case Code.UPDATE :
				return decodeUpdate(in);
			case Code.NONCE :
				return decodeNonce(in);
			case Code.ZERTIFIKATLEISTUNGSERBRINGER :
				return decodeZertifikatLeistungserbringer(in);
			case Code.ZERTIFIKATPATIENT :
				return decodeZertifikatPatient(in);
			case Code.ENCDATAASYMM :
				return decodeEncDataAsymm(in);
			case Code.NONCEDATA :
				return decodeNonceData(in);
			case Code.SECRET :
				return decodeSecret(in);
			case Code.PINDATA :
				return decodePINData(in);
			case Code.ENCDATASYMM :
				return decodeEncDataSymm(in);
			case Code.EGKDATEN :
				return decodeEGKDaten(in);
			case Code.ARZNEIMITTEL :
				return decodeArzneimittel(in);
			case Code.EREZEPTDATEN :
				return decodeERezeptdaten(in);
			case Code.SIGNEDDATA :
				return decodeSignedData(in);
			case Code.SECRETNONCEDATA :
				return decodeSecretNonceData(in);
			case Code.KEYDATA :
				return decodeKeyData(in);
			case Code.INDEXNONCE :
				return decodeIndexNonce(in);
			case Code.REZEPTDATENUNDNONCE :
				return decodeRezeptdatenUndNonce(in);
			case Code.NOTFALLDATEN :
				return decodeNotfalldaten(in);
			case Code.NOTFALLDATENNONCE :
				return decodeNotfalldatenNonce(in);
			case Code.ZERTIFIKATKIOSK :
				return decodeZertifikatKiosk(in);
			case Code.STAMMUNDNOTFALLDATEN :
				return decodeStammUndNotfalldaten(in);
			case Code.VERSICHERTENSTAMMDATEN :
				return decodeVersichertenstammdaten(in);
			case Code.NONCEDATAANDINDEX :
				return decodeNonceDataAndIndex(in);
			case Code.NONCEUNDREZEPT :
				return decodeNonceUndRezept(in);
			case Code.NONCESTRINGDATA :
				return decodeNonceStringData(in);
			case Code.UPDATEAUTH :
				return decodeUpdateAuth(in);
			case Code.LEISTUNGSERBRINGERINFO :
				return decodeLeistungserbringerInfo(in);
			case Code.ZERTIFIKATPATIENTDATA :
				return decodeZertifikatPatientData(in);
			case Code.SYMMKEY :
				return decodeSymmKey(in);
			case Code.ZERTIFIKATKIOSKDATA :
				return decodeZertifikatKioskData(in);
			case Code.PUBLICKEY :
				return decodePublicKey(in);
			case Code.ZERTIFIKATTRUSTEDTHIRDPARTY :
				return decodeZertifikatTrustedThirdParty(in);
			case Code.PRIVATEKEY :
				return decodePrivateKey(in);
			case Code.ZERTIFIKATTTPDATA :
				return decodeZertifikatTTPData(in);
				// never reached
			default :
				stop();
		}
		return null;
	}

	public void compSize(Codeable c) throws Exception {
		switch (c.getCode()) {
			case Code.AUTHENTISIEREHBA :

				compSizeAuthentisiereHBA((AuthentisiereHBA) c);

				break;
			case Code.AUTHENTISIEREEGK :

				compSizeAuthentisiereEGK((AuthentisiereEGK) c);

				break;
			case Code.USTARTAUTH :

				compSizeUStartAuth((UStartAuth) c);

				break;
			case Code.RETAUTHENTISIEREHBA :

				compSizeRetAuthentisiereHBA((RetAuthentisiereHBA) c);

				break;
			case Code.RETAUTHENTISIEREEGK :

				compSizeRetAuthentisiereEGK((RetAuthentisiereEGK) c);

				break;
			case Code.SENDECHALLENGEHBA :

				compSizeSendeChallengeHBA((SendeChallengeHBA) c);

				break;
			case Code.SENDECHALLENGEEGK :

				compSizeSendeChallengeEGK((SendeChallengeEGK) c);

				break;
			case Code.RETSENDECHALLENGEHBA :

				compSizeRetSendeChallengeHBA((RetSendeChallengeHBA) c);

				break;
			case Code.RETSENDECHALLENGEEGK :

				compSizeRetSendeChallengeEGK((RetSendeChallengeEGK) c);

				break;
			case Code.UENDAUTH :

				compSizeUEndAuth((UEndAuth) c);

				break;
			case Code.USTARTSCHLUESSELAUSTAUSCH :

				compSizeUStartSchluesselaustausch((UStartSchluesselaustausch) c);

				break;
			case Code.ERFRAGENONCEEGK :

				compSizeErfrageNonceEGK((ErfrageNonceEGK) c);

				break;
			case Code.SENDENONCEEGK :

				compSizeSendeNonceEGK((SendeNonceEGK) c);

				break;
			case Code.BESTAETIGESCHLUESSELEGK :

				compSizeBestaetigeSchluesselEGK((BestaetigeSchluesselEGK) c);

				break;
			case Code.ERFRAGENONCEHBA :

				compSizeErfrageNonceHBA((ErfrageNonceHBA) c);

				break;
			case Code.SENDENONCEHBA :

				compSizeSendeNonceHBA((SendeNonceHBA) c);

				break;
			case Code.BESTAETIGESCHLUESSELHBA :

				compSizeBestaetigeSchluesselHBA((BestaetigeSchluesselHBA) c);

				break;
			case Code.USCHLUESSELAUSTAUSCHOK :

				compSizeUSchluesselaustauschOK((USchluesselaustauschOK) c);

				break;
			case Code.UVALIDATEPINEGK :

				compSizeUValidatePINEGK((UValidatePINEGK) c);

				break;
			case Code.UBESTAETIGEPINEGK :

				compSizeUBestaetigePINEGK((UBestaetigePINEGK) c);

				break;
			case Code.ERFRAGENONCEEGKPIN :

				compSizeErfrageNonceEGKpin((ErfrageNonceEGKpin) c);

				break;
			case Code.SENDENONCEEGKPIN :

				compSizeSendeNonceEGKpin((SendeNonceEGKpin) c);

				break;
			case Code.SENDEPINEGK :

				compSizeSendePINEGK((SendePINEGK) c);

				break;
			case Code.BESTAETIGEPINEGK :

				compSizeBestaetigePINEGK((BestaetigePINEGK) c);

				break;
			case Code.UVALIDATEPINHBA :

				compSizeUValidatePINHBA((UValidatePINHBA) c);

				break;
			case Code.UBESTAETIGEPINHBA :

				compSizeUBestaetigePINHBA((UBestaetigePINHBA) c);

				break;
			case Code.ERFRAGENONCEHBAPIN :

				compSizeErfrageNonceHBApin((ErfrageNonceHBApin) c);

				break;
			case Code.SENDENONCEHBAPIN :

				compSizeSendeNonceHBApin((SendeNonceHBApin) c);

				break;
			case Code.SENDEPINHBA :

				compSizeSendePINHBA((SendePINHBA) c);

				break;
			case Code.BESTAETIGEPINHBA :

				compSizeBestaetigePINHBA((BestaetigePINHBA) c);

				break;
			case Code.SENDENONCEREZEPT :

				compSizeSendeNonceRezept((SendeNonceRezept) c);

				break;
			case Code.USENDEREZEPTDATEN :

				compSizeUSendeRezeptdaten((USendeRezeptdaten) c);

				break;
			case Code.SCHICKEREZEPTDATEN :

				compSizeSchickeRezeptdaten((SchickeRezeptdaten) c);

				break;
			case Code.SENDEREZEPTDATENSIGNATUR :

				compSizeSendeRezeptdatensignatur((SendeRezeptdatensignatur) c);

				break;
			case Code.SENDEREZEPT :

				compSizeSendeRezept((SendeRezept) c);

				break;
			case Code.UBESTAETIGEREZEPTAUSSTELLEN :

				compSizeUBestaetigeRezeptAusstellen((UBestaetigeRezeptAusstellen) c);

				break;
			case Code.ERFRAGENONCEREZEPT :

				compSizeErfrageNonceRezept((ErfrageNonceRezept) c);

				break;
			case Code.BESTAETIGEREZEPTAUSSTELLEN :

				compSizeBestaetigeRezeptAusstellen((BestaetigeRezeptAusstellen) c);

				break;
			case Code.ENDAUTH :

				compSizeEndAuth((EndAuth) c);

				break;
			case Code.SENDENONCEVERSCHLUESSELTHBA :

				compSizeSendeNonceVerschluesseltHBA((SendeNonceVerschluesseltHBA) c);

				break;
			case Code.SENDESCHLUESSEL :

				compSizeSendeSchluessel((SendeSchluessel) c);

				break;
			case Code.ERFRAGENONCEEGKWEITER :

				compSizeErfrageNonceEGKWeiter((ErfrageNonceEGKWeiter) c);

				break;
			case Code.SENDENONCEVERSCHLUESSELTEGK :

				compSizeSendeNonceVerschluesseltEGK((SendeNonceVerschluesseltEGK) c);

				break;
			case Code.SENDENONCEVERSCHLUESSELTEGKWEITER :

				compSizeSendeNonceVerschluesseltEGKWeiter((SendeNonceVerschluesseltEGKWeiter) c);

				break;
			case Code.SENDESESSIONKEY :

				compSizeSendeSessionkey((SendeSessionkey) c);

				break;
			case Code.UBESTAETIGESCHLUESSELAUSTAUSCH :

				compSizeUBestaetigeSchluesselaustausch((UBestaetigeSchluesselaustausch) c);

				break;
			case Code.BESTAETIGESCHLUESSELAUSTAUSCH :

				compSizeBestaetigeSchluesselaustausch((BestaetigeSchluesselaustausch) c);

				break;
			case Code.SENDECHALLENGEHBAWEITER :

				compSizeSendeChallengeHBAWeiter((SendeChallengeHBAWeiter) c);

				break;
			case Code.SENDECHALLENGEEGKWEITER :

				compSizeSendeChallengeEGKWeiter((SendeChallengeEGKWeiter) c);

				break;
			case Code.RETAUTHENTISIEREEGKWEITER :

				compSizeRetAuthentisiereEGKWeiter((RetAuthentisiereEGKWeiter) c);

				break;
			case Code.RETSENDECHALLENGEEGKWEITER :

				compSizeRetSendeChallengeEGKWeiter((RetSendeChallengeEGKWeiter) c);

				break;
			case Code.SENDEZERTIFIKATHBA :

				compSizeSendeZertifikatHBA((SendeZertifikatHBA) c);

				break;
			case Code.SENDESESSIONKEYWEITER :

				compSizeSendeSessionkeyWeiter((SendeSessionkeyWeiter) c);

				break;
			case Code.UREZEPTEINLOESEN :

				compSizeURezeptEinloesen((URezeptEinloesen) c);

				break;
			case Code.UREZEPTAUSGABE :

				compSizeURezeptausgabe((URezeptausgabe) c);

				break;
			case Code.ERFRAGENONCEFUEREINLOESEN :

				compSizeErfrageNonceFuerEinloesen((ErfrageNonceFuerEinloesen) c);

				break;
			case Code.SENDENONCEFUEREINLOESEN :

				compSizeSendeNonceFuerEinloesen((SendeNonceFuerEinloesen) c);

				break;
			case Code.EINLOESEN :

				compSizeEinloesen((Einloesen) c);

				break;
			case Code.SENDEREZEPTZUMEINLOESEN :

				compSizeSendeRezeptZumEinloesen((SendeRezeptZumEinloesen) c);

				break;
			case Code.UERSTELLENOTFALLDATEN :

				compSizeUErstelleNotfalldaten((UErstelleNotfalldaten) c);

				break;
			case Code.ERFRAGENONCENOTFALLDATENERSTELLEN :

				compSizeErfrageNonceNotfalldatenErstellen((ErfrageNonceNotfalldatenErstellen) c);

				break;
			case Code.SENDENONCENOTFALLDATENERSTELLEN :

				compSizeSendeNonceNotfalldatenErstellen((SendeNonceNotfalldatenErstellen) c);

				break;
			case Code.SENDENOTFALLDATEN :

				compSizeSendeNotfalldaten((SendeNotfalldaten) c);

				break;
			case Code.SENDENOTFALLDATENSIGNATUR :

				compSizeSendeNotfalldatensignatur((SendeNotfalldatensignatur) c);

				break;
			case Code.SENDENOTFALLDATENUNDSIG :

				compSizeSendeNotfalldatenUndSig((SendeNotfalldatenUndSig) c);

				break;
			case Code.BESTAETIGENOTFALLDATENERSTELLEN :

				compSizeBestaetigeNotfalldatenErstellen((BestaetigeNotfalldatenErstellen) c);

				break;
			case Code.UBESTAETIGENOTFALLDATENERSTELLEN :

				compSizeUBestaetigeNotfalldatenErstellen((UBestaetigeNotfalldatenErstellen) c);

				break;
			case Code.ULESENOTFALLDATEN :

				compSizeULeseNotfalldaten((ULeseNotfalldaten) c);

				break;
			case Code.PINARZTEINGEGEBEN :

				compSizePINArztEingegeben((PINArztEingegeben) c);

				break;
			case Code.PINOK :

				compSizePINOK((PINOK) c);

				break;
			case Code.ERFRAGENOTFALLDATEN :

				compSizeErfrageNotfalldaten((ErfrageNotfalldaten) c);

				break;
			case Code.SCHICKENOTFALLDATEN :

				compSizeSchickeNotfalldaten((SchickeNotfalldaten) c);

				break;
			case Code.USENDENOTFALLDATEN :

				compSizeUSendeNotfalldaten((USendeNotfalldaten) c);

				break;
			case Code.UAUTHEGKUNDKIOSK :

				compSizeUAuthEGKUndKiosk((UAuthEGKUndKiosk) c);

				break;
			case Code.SENDEZERTKIOSK :

				compSizeSendeZertKiosk((SendeZertKiosk) c);

				break;
			case Code.SENDEZERTEGK :

				compSizeSendeZertEGK((SendeZertEGK) c);

				break;
			case Code.SENDENONCES :

				compSizeSendeNonces((SendeNonces) c);

				break;
			case Code.BESTAETIGENONCE :

				compSizeBestaetigeNonce((BestaetigeNonce) c);

				break;
			case Code.UAUTHOK :

				compSizeUAuthOK((UAuthOK) c);

				break;
			case Code.UPINEINGABEKIOSK :

				compSizeUPINEingabeKiosk((UPINEingabeKiosk) c);

				break;
			case Code.ERFRAGENONCEPIN :

				compSizeErfrageNoncePIN((ErfrageNoncePIN) c);

				break;
			case Code.SENDENONCEPIN :

				compSizeSendeNoncePIN((SendeNoncePIN) c);

				break;
			case Code.SENDEPIN :

				compSizeSendePIN((SendePIN) c);

				break;
			case Code.BESTAETIGEPIN :

				compSizeBestaetigePIN((BestaetigePIN) c);

				break;
			case Code.UBESTAETIGEPIN :

				compSizeUBestaetigePIN((UBestaetigePIN) c);

				break;
			case Code.USCHLUESSELAUSTAUSCH :

				compSizeUSchluesselaustausch((USchluesselaustausch) c);

				break;
			case Code.ERFRAGENONCESCHLUESSELAUSTAUSCH :

				compSizeErfrageNonceSchluesselaustausch((ErfrageNonceSchluesselaustausch) c);

				break;
			case Code.SENDENONCESCHLUESSELAUSTAUSCH :

				compSizeSendeNonceSchluesselaustausch((SendeNonceSchluesselaustausch) c);

				break;
			case Code.SENDESESSIONKEYKIOSK :

				compSizeSendeSessionkeyKiosk((SendeSessionkeyKiosk) c);

				break;
			case Code.SCHLUESSELKIOSKOK :

				compSizeSchluesselKioskOK((SchluesselKioskOK) c);

				break;
			case Code.USCHLUESSELKIOSKOK :

				compSizeUSchluesselKioskOK((USchluesselKioskOK) c);

				break;
			case Code.ULESESTAMMUNDNOTFALLDATEN :

				compSizeULeseStammUndNotfalldaten((ULeseStammUndNotfalldaten) c);

				break;
			case Code.LESESTAMMUNDNOTFALLDATEN :

				compSizeLeseStammUndNotfalldaten((LeseStammUndNotfalldaten) c);

				break;
			case Code.SENDESTAMMUNDNOTFALLDATEN :

				compSizeSendeStammUndNotfalldaten((SendeStammUndNotfalldaten) c);

				break;
			case Code.USENDESTAMMUNDNOTFALLDATEN :

				compSizeUSendeStammUndNotfalldaten((USendeStammUndNotfalldaten) c);

				break;
			case Code.ULESEREZEPT :

				compSizeULeseRezept((ULeseRezept) c);

				break;
			case Code.LESEREZEPT :

				compSizeLeseRezept((LeseRezept) c);

				break;
			case Code.SENDEREZEPTDATEN :

				compSizeSendeRezeptdaten((SendeRezeptdaten) c);

				break;
			case Code.USENDEREZEPT :

				compSizeUSendeRezept((USendeRezept) c);

				break;
			case Code.UNOTFALLDATENLOESCHEN :

				compSizeUNotfalldatenLoeschen((UNotfalldatenLoeschen) c);

				break;
			case Code.ERFRAGENONCENOTFALLDATENLOESCHEN :

				compSizeErfrageNonceNotfalldatenLoeschen((ErfrageNonceNotfalldatenLoeschen) c);

				break;
			case Code.LOESCHENOTFALLDATEN :

				compSizeLoescheNotfalldaten((LoescheNotfalldaten) c);

				break;
			case Code.BESTAETIGELOESCHENNOTFALLDATEN :

				compSizeBestaetigeLoeschenNotfalldaten((BestaetigeLoeschenNotfalldaten) c);

				break;
			case Code.UBESTAETIGELOESCHENNOTFALLDATEN :

				compSizeUBestaetigeLoeschenNotfalldaten((UBestaetigeLoeschenNotfalldaten) c);

				break;
			case Code.SENDENONCE :

				compSizeSendeNonce((SendeNonce) c);

				break;
			case Code.UUSEROK :

				compSizeUUserOK((UUserOK) c);

				break;
			case Code.UUSERUPDATE :

				compSizeUUserUpdate((UUserUpdate) c);

				break;
			case Code.GETDATA :

				compSizeGetData((GetData) c);

				break;
			case Code.OK :

				compSizeOK((OK) c);

				break;
			case Code.CARDDATA :

				compSizeCardData((CardData) c);

				break;
			case Code.UPDATE :

				compSizeUpdate((Update) c);

				break;
			case Code.NONCE :

				compSizeNonce((Nonce) c);

				break;
			case Code.ZERTIFIKATLEISTUNGSERBRINGER :

				compSizeZertifikatLeistungserbringer((ZertifikatLeistungserbringer) c);

				break;
			case Code.ZERTIFIKATPATIENT :

				compSizeZertifikatPatient((ZertifikatPatient) c);

				break;
			case Code.ENCDATAASYMM :

				compSizeEncDataAsymm((EncDataAsymm) c);

				break;
			case Code.NONCEDATA :

				compSizeNonceData((NonceData) c);

				break;
			case Code.SECRET :

				compSizeSecret((Secret) c);

				break;
			case Code.PINDATA :

				compSizePINData((PINData) c);

				break;
			case Code.ENCDATASYMM :

				compSizeEncDataSymm((EncDataSymm) c);

				break;
			case Code.EGKDATEN :

				compSizeEGKDaten((EGKDaten) c);

				break;
			case Code.ARZNEIMITTEL :

				compSizeArzneimittel((Arzneimittel) c);

				break;
			case Code.EREZEPTDATEN :

				compSizeERezeptdaten((ERezeptdaten) c);

				break;
			case Code.SIGNEDDATA :

				compSizeSignedData((SignedData) c);

				break;
			case Code.SECRETNONCEDATA :

				compSizeSecretNonceData((SecretNonceData) c);

				break;
			case Code.KEYDATA :

				compSizeKeyData((KeyData) c);

				break;
			case Code.INDEXNONCE :

				compSizeIndexNonce((IndexNonce) c);

				break;
			case Code.REZEPTDATENUNDNONCE :

				compSizeRezeptdatenUndNonce((RezeptdatenUndNonce) c);

				break;
			case Code.NOTFALLDATEN :

				compSizeNotfalldaten((Notfalldaten) c);

				break;
			case Code.NOTFALLDATENNONCE :

				compSizeNotfalldatenNonce((NotfalldatenNonce) c);

				break;
			case Code.ZERTIFIKATKIOSK :

				compSizeZertifikatKiosk((ZertifikatKiosk) c);

				break;
			case Code.STAMMUNDNOTFALLDATEN :

				compSizeStammUndNotfalldaten((StammUndNotfalldaten) c);

				break;
			case Code.VERSICHERTENSTAMMDATEN :

				compSizeVersichertenstammdaten((Versichertenstammdaten) c);

				break;
			case Code.NONCEDATAANDINDEX :

				compSizeNonceDataAndIndex((NonceDataAndIndex) c);

				break;
			case Code.NONCEUNDREZEPT :

				compSizeNonceUndRezept((NonceUndRezept) c);

				break;
			case Code.NONCESTRINGDATA :

				compSizeNonceStringData((NonceStringData) c);

				break;
			case Code.UPDATEAUTH :

				compSizeUpdateAuth((UpdateAuth) c);

				break;
			case Code.LEISTUNGSERBRINGERINFO :

				compSizeLeistungserbringerInfo((LeistungserbringerInfo) c);

				break;
			case Code.ZERTIFIKATPATIENTDATA :

				compSizeZertifikatPatientData((ZertifikatPatientData) c);

				break;
			case Code.SYMMKEY :

				compSizeSymmKey((SymmKey) c);

				break;
			case Code.ZERTIFIKATKIOSKDATA :

				compSizeZertifikatKioskData((ZertifikatKioskData) c);

				break;
			case Code.PUBLICKEY :

				compSizePublicKey((PublicKey) c);

				break;
			case Code.ZERTIFIKATTRUSTEDTHIRDPARTY :

				compSizeZertifikatTrustedThirdParty((ZertifikatTrustedThirdParty) c);

				break;
			case Code.PRIVATEKEY :

				compSizePrivateKey((PrivateKey) c);

				break;
			case Code.ZERTIFIKATTTPDATA :

				compSizeZertifikatTTPData((ZertifikatTTPData) c);

				break;
			// never reached
			default :

				stop();

		}
	}

	public PlainData decodePlainData(byte[] in, byte expectedType)
			throws Exception {
		encoding_length = 0;
		PlainData p = null;
		try {
			if (in[encoding_length] != expectedType) {
				stop();

			}
			switch (in[encoding_length]) {

				case Code.NONCEDATA :
					p = decodeNonceData(in);
					break;
				case Code.PINDATA :
					p = decodePINData(in);
					break;
				case Code.EGKDATEN :
					p = decodeEGKDaten(in);
					break;
				case Code.EREZEPTDATEN :
					p = decodeERezeptdaten(in);
					break;
				case Code.SECRETNONCEDATA :
					p = decodeSecretNonceData(in);
					break;
				case Code.KEYDATA :
					p = decodeKeyData(in);
					break;
				case Code.INDEXNONCE :
					p = decodeIndexNonce(in);
					break;
				case Code.REZEPTDATENUNDNONCE :
					p = decodeRezeptdatenUndNonce(in);
					break;
				case Code.NOTFALLDATEN :
					p = decodeNotfalldaten(in);
					break;
				case Code.NOTFALLDATENNONCE :
					p = decodeNotfalldatenNonce(in);
					break;
				case Code.STAMMUNDNOTFALLDATEN :
					p = decodeStammUndNotfalldaten(in);
					break;
				case Code.VERSICHERTENSTAMMDATEN :
					p = decodeVersichertenstammdaten(in);
					break;
				case Code.NONCEDATAANDINDEX :
					p = decodeNonceDataAndIndex(in);
					break;
				case Code.NONCEUNDREZEPT :
					p = decodeNonceUndRezept(in);
					break;
				case Code.NONCESTRINGDATA :
					p = decodeNonceStringData(in);
					break;

			}
		} catch (ArrayIndexOutOfBoundsException e) {
			stop();

		}

		if (p == null) {
			stop();

		}
		return p;
	}

	public void encodeAuthentisiereHBA(AuthentisiereHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.AUTHENTISIEREHBA;
	}

	public AuthentisiereHBA decodeAuthentisiereHBA(byte[] in) throws Exception {
		AuthentisiereHBA c = new AuthentisiereHBA();
		encoding_length++;
		return c;
	}

	private void compSizeAuthentisiereHBA(AuthentisiereHBA c) throws Exception {
		encoding_length++;
	}

	public void encodeAuthentisiereEGK(AuthentisiereEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.AUTHENTISIEREEGK;

		encode(c.getNonce());
	}

	public AuthentisiereEGK decodeAuthentisiereEGK(byte[] in) throws Exception {
		AuthentisiereEGK c = new AuthentisiereEGK();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeAuthentisiereEGK(AuthentisiereEGK c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeUStartAuth(UStartAuth c) throws Exception {
		encodeDestination[encoding_length++] = Code.USTARTAUTH;
	}

	public UStartAuth decodeUStartAuth(byte[] in) throws Exception {
		UStartAuth c = new UStartAuth();
		encoding_length++;
		return c;
	}

	private void compSizeUStartAuth(UStartAuth c) throws Exception {
		encoding_length++;
	}

	public void encodeRetAuthentisiereHBA(RetAuthentisiereHBA c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.RETAUTHENTISIEREHBA;

		encode(c.getZertArzt());
	}

	public RetAuthentisiereHBA decodeRetAuthentisiereHBA(byte[] in)
			throws Exception {
		RetAuthentisiereHBA c = new RetAuthentisiereHBA();
		encoding_length++;
		c.setZertArzt(((ZertifikatLeistungserbringer) decode(in)));
		return c;
	}

	private void compSizeRetAuthentisiereHBA(RetAuthentisiereHBA c)
			throws Exception {
		encoding_length++;
		compSize(c.getZertArzt());
	}

	public void encodeRetAuthentisiereEGK(RetAuthentisiereEGK c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.RETAUTHENTISIEREEGK;

		encode(c.getZertPatient());
	}

	public RetAuthentisiereEGK decodeRetAuthentisiereEGK(byte[] in)
			throws Exception {
		RetAuthentisiereEGK c = new RetAuthentisiereEGK();
		encoding_length++;
		c.setZertPatient(((ZertifikatPatient) decode(in)));
		return c;
	}

	private void compSizeRetAuthentisiereEGK(RetAuthentisiereEGK c)
			throws Exception {
		encoding_length++;
		compSize(c.getZertPatient());
	}

	public void encodeSendeChallengeHBA(SendeChallengeHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEHBA;

		encode(c.getEncryptedNonce());
	}

	public SendeChallengeHBA decodeSendeChallengeHBA(byte[] in)
			throws Exception {
		SendeChallengeHBA c = new SendeChallengeHBA();
		encoding_length++;
		c.setEncryptedNonce(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeChallengeHBA(SendeChallengeHBA c)
			throws Exception {
		encoding_length++;
		compSize(c.getEncryptedNonce());
	}

	public void encodeSendeChallengeEGK(SendeChallengeEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEEGK;

		encode(c.getEncryptedNonce());
	}

	public SendeChallengeEGK decodeSendeChallengeEGK(byte[] in)
			throws Exception {
		SendeChallengeEGK c = new SendeChallengeEGK();
		encoding_length++;
		c.setEncryptedNonce(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeChallengeEGK(SendeChallengeEGK c)
			throws Exception {
		encoding_length++;
		compSize(c.getEncryptedNonce());
	}

	public void encodeRetSendeChallengeHBA(RetSendeChallengeHBA c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.RETSENDECHALLENGEHBA;

		encode(c.getNonceCard());

		encode(c.getNonceTerm());
	}

	public RetSendeChallengeHBA decodeRetSendeChallengeHBA(byte[] in)
			throws Exception {
		RetSendeChallengeHBA c = new RetSendeChallengeHBA();
		encoding_length++;
		c.setNonceCard(((Nonce) decode(in)));
		c.setNonceTerm(((Nonce) decode(in)));
		return c;
	}

	private void compSizeRetSendeChallengeHBA(RetSendeChallengeHBA c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonceCard());
		compSize(c.getNonceTerm());
	}

	public void encodeRetSendeChallengeEGK(RetSendeChallengeEGK c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.RETSENDECHALLENGEEGK;

		encode(c.getNonceCard());

		encode(c.getNonceTerm());
	}

	public RetSendeChallengeEGK decodeRetSendeChallengeEGK(byte[] in)
			throws Exception {
		RetSendeChallengeEGK c = new RetSendeChallengeEGK();
		encoding_length++;
		c.setNonceCard(((Nonce) decode(in)));
		c.setNonceTerm(((Nonce) decode(in)));
		return c;
	}

	private void compSizeRetSendeChallengeEGK(RetSendeChallengeEGK c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonceCard());
		compSize(c.getNonceTerm());
	}

	public void encodeUEndAuth(UEndAuth c) throws Exception {
		encodeDestination[encoding_length++] = Code.UENDAUTH;
	}

	public UEndAuth decodeUEndAuth(byte[] in) throws Exception {
		UEndAuth c = new UEndAuth();
		encoding_length++;
		return c;
	}

	private void compSizeUEndAuth(UEndAuth c) throws Exception {
		encoding_length++;
	}

	public void encodeUStartSchluesselaustausch(UStartSchluesselaustausch c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.USTARTSCHLUESSELAUSTAUSCH;
	}

	public UStartSchluesselaustausch decodeUStartSchluesselaustausch(byte[] in)
			throws Exception {
		UStartSchluesselaustausch c = new UStartSchluesselaustausch();
		encoding_length++;
		return c;
	}

	private void compSizeUStartSchluesselaustausch(UStartSchluesselaustausch c)
			throws Exception {
		encoding_length++;
	}

	public void encodeErfrageNonceEGK(ErfrageNonceEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEEGK;
	}

	public ErfrageNonceEGK decodeErfrageNonceEGK(byte[] in) throws Exception {
		ErfrageNonceEGK c = new ErfrageNonceEGK();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceEGK(ErfrageNonceEGK c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceEGK(SendeNonceEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEEGK;

		encode(c.getNonce());

		encode(c.getZert());
	}

	public SendeNonceEGK decodeSendeNonceEGK(byte[] in) throws Exception {
		SendeNonceEGK c = new SendeNonceEGK();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setZert(((ZertifikatPatient) decode(in)));
		return c;
	}

	private void compSizeSendeNonceEGK(SendeNonceEGK c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getZert());
	}

	public void encodeBestaetigeSchluesselEGK(BestaetigeSchluesselEGK c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGESCHLUESSELEGK;
	}

	public BestaetigeSchluesselEGK decodeBestaetigeSchluesselEGK(byte[] in)
			throws Exception {
		BestaetigeSchluesselEGK c = new BestaetigeSchluesselEGK();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigeSchluesselEGK(BestaetigeSchluesselEGK c)
			throws Exception {
		encoding_length++;
	}

	public void encodeErfrageNonceHBA(ErfrageNonceHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEHBA;
	}

	public ErfrageNonceHBA decodeErfrageNonceHBA(byte[] in) throws Exception {
		ErfrageNonceHBA c = new ErfrageNonceHBA();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceHBA(ErfrageNonceHBA c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceHBA(SendeNonceHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEHBA;

		encode(c.getNonce());

		encode(c.getZert());
	}

	public SendeNonceHBA decodeSendeNonceHBA(byte[] in) throws Exception {
		SendeNonceHBA c = new SendeNonceHBA();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setZert(((ZertifikatLeistungserbringer) decode(in)));
		return c;
	}

	private void compSizeSendeNonceHBA(SendeNonceHBA c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getZert());
	}

	public void encodeBestaetigeSchluesselHBA(BestaetigeSchluesselHBA c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGESCHLUESSELHBA;
	}

	public BestaetigeSchluesselHBA decodeBestaetigeSchluesselHBA(byte[] in)
			throws Exception {
		BestaetigeSchluesselHBA c = new BestaetigeSchluesselHBA();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigeSchluesselHBA(BestaetigeSchluesselHBA c)
			throws Exception {
		encoding_length++;
	}

	public void encodeUSchluesselaustauschOK(USchluesselaustauschOK c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.USCHLUESSELAUSTAUSCHOK;
	}

	public USchluesselaustauschOK decodeUSchluesselaustauschOK(byte[] in)
			throws Exception {
		USchluesselaustauschOK c = new USchluesselaustauschOK();
		encoding_length++;
		return c;
	}

	private void compSizeUSchluesselaustauschOK(USchluesselaustauschOK c)
			throws Exception {
		encoding_length++;
	}

	public void encodeUValidatePINEGK(UValidatePINEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.UVALIDATEPINEGK;

		encode(c.getPin());
	}

	public UValidatePINEGK decodeUValidatePINEGK(byte[] in) throws Exception {
		UValidatePINEGK c = new UValidatePINEGK();
		encoding_length++;
		c.setPin(((Secret) decode(in)));
		return c;
	}

	private void compSizeUValidatePINEGK(UValidatePINEGK c) throws Exception {
		encoding_length++;
		compSize(c.getPin());
	}

	public void encodeUBestaetigePINEGK(UBestaetigePINEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.UBESTAETIGEPINEGK;
	}

	public UBestaetigePINEGK decodeUBestaetigePINEGK(byte[] in)
			throws Exception {
		UBestaetigePINEGK c = new UBestaetigePINEGK();
		encoding_length++;
		return c;
	}

	private void compSizeUBestaetigePINEGK(UBestaetigePINEGK c)
			throws Exception {
		encoding_length++;
	}

	public void encodeErfrageNonceEGKpin(ErfrageNonceEGKpin c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEEGKPIN;
	}

	public ErfrageNonceEGKpin decodeErfrageNonceEGKpin(byte[] in)
			throws Exception {
		ErfrageNonceEGKpin c = new ErfrageNonceEGKpin();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceEGKpin(ErfrageNonceEGKpin c)
			throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceEGKpin(SendeNonceEGKpin c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEEGKPIN;

		encode(c.getN());
	}

	public SendeNonceEGKpin decodeSendeNonceEGKpin(byte[] in) throws Exception {
		SendeNonceEGKpin c = new SendeNonceEGKpin();
		encoding_length++;
		c.setN(((Nonce) decode(in)));
		return c;
	}

	private void compSizeSendeNonceEGKpin(SendeNonceEGKpin c) throws Exception {
		encoding_length++;
		compSize(c.getN());
	}

	public void encodeSendePINEGK(SendePINEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEPINEGK;

		encode(c.getEd());
	}

	public SendePINEGK decodeSendePINEGK(byte[] in) throws Exception {
		SendePINEGK c = new SendePINEGK();
		encoding_length++;
		c.setEd(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendePINEGK(SendePINEGK c) throws Exception {
		encoding_length++;
		compSize(c.getEd());
	}

	public void encodeBestaetigePINEGK(BestaetigePINEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGEPINEGK;
	}

	public BestaetigePINEGK decodeBestaetigePINEGK(byte[] in) throws Exception {
		BestaetigePINEGK c = new BestaetigePINEGK();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigePINEGK(BestaetigePINEGK c) throws Exception {
		encoding_length++;
	}

	public void encodeUValidatePINHBA(UValidatePINHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.UVALIDATEPINHBA;

		encode(c.getPin());
	}

	public UValidatePINHBA decodeUValidatePINHBA(byte[] in) throws Exception {
		UValidatePINHBA c = new UValidatePINHBA();
		encoding_length++;
		c.setPin(((Secret) decode(in)));
		return c;
	}

	private void compSizeUValidatePINHBA(UValidatePINHBA c) throws Exception {
		encoding_length++;
		compSize(c.getPin());
	}

	public void encodeUBestaetigePINHBA(UBestaetigePINHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.UBESTAETIGEPINHBA;
	}

	public UBestaetigePINHBA decodeUBestaetigePINHBA(byte[] in)
			throws Exception {
		UBestaetigePINHBA c = new UBestaetigePINHBA();
		encoding_length++;
		return c;
	}

	private void compSizeUBestaetigePINHBA(UBestaetigePINHBA c)
			throws Exception {
		encoding_length++;
	}

	public void encodeErfrageNonceHBApin(ErfrageNonceHBApin c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEHBAPIN;
	}

	public ErfrageNonceHBApin decodeErfrageNonceHBApin(byte[] in)
			throws Exception {
		ErfrageNonceHBApin c = new ErfrageNonceHBApin();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceHBApin(ErfrageNonceHBApin c)
			throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceHBApin(SendeNonceHBApin c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEHBAPIN;

		encode(c.getN());
	}

	public SendeNonceHBApin decodeSendeNonceHBApin(byte[] in) throws Exception {
		SendeNonceHBApin c = new SendeNonceHBApin();
		encoding_length++;
		c.setN(((Nonce) decode(in)));
		return c;
	}

	private void compSizeSendeNonceHBApin(SendeNonceHBApin c) throws Exception {
		encoding_length++;
		compSize(c.getN());
	}

	public void encodeSendePINHBA(SendePINHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEPINHBA;

		encode(c.getEd());
	}

	public SendePINHBA decodeSendePINHBA(byte[] in) throws Exception {
		SendePINHBA c = new SendePINHBA();
		encoding_length++;
		c.setEd(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendePINHBA(SendePINHBA c) throws Exception {
		encoding_length++;
		compSize(c.getEd());
	}

	public void encodeBestaetigePINHBA(BestaetigePINHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGEPINHBA;
	}

	public BestaetigePINHBA decodeBestaetigePINHBA(byte[] in) throws Exception {
		BestaetigePINHBA c = new BestaetigePINHBA();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigePINHBA(BestaetigePINHBA c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceRezept(SendeNonceRezept c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEREZEPT;

		encode(c.getEd());
	}

	public SendeNonceRezept decodeSendeNonceRezept(byte[] in) throws Exception {
		SendeNonceRezept c = new SendeNonceRezept();
		encoding_length++;
		c.setEd(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSendeNonceRezept(SendeNonceRezept c) throws Exception {
		encoding_length++;
		compSize(c.getEd());
	}

	public void encodeUSendeRezeptdaten(USendeRezeptdaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.USENDEREZEPTDATEN;

		encodeString((String) c.getAusstellungsdatum());

		encodeString((String) c.getAngabezureinnahme());

		encode(c.getArzneimittel());
	}

	public USendeRezeptdaten decodeUSendeRezeptdaten(byte[] in)
			throws Exception {
		USendeRezeptdaten c = new USendeRezeptdaten();
		encoding_length++;
		c.setAusstellungsdatum(decodeString(in));
		c.setAngabezureinnahme(decodeString(in));
		c.setArzneimittel(((Arzneimittel) decode(in)));
		return c;
	}

	private void compSizeUSendeRezeptdaten(USendeRezeptdaten c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getAusstellungsdatum());

		compSizeString(c.getAngabezureinnahme());
		compSize(c.getArzneimittel());
	}

	public void encodeSchickeRezeptdaten(SchickeRezeptdaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.SCHICKEREZEPTDATEN;

		encode(c.getEd());
	}

	public SchickeRezeptdaten decodeSchickeRezeptdaten(byte[] in)
			throws Exception {
		SchickeRezeptdaten c = new SchickeRezeptdaten();
		encoding_length++;
		c.setEd(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSchickeRezeptdaten(SchickeRezeptdaten c)
			throws Exception {
		encoding_length++;
		compSize(c.getEd());
	}

	public void encodeSendeRezeptdatensignatur(SendeRezeptdatensignatur c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEREZEPTDATENSIGNATUR;

		encode(c.getSig());
	}

	public SendeRezeptdatensignatur decodeSendeRezeptdatensignatur(byte[] in)
			throws Exception {
		SendeRezeptdatensignatur c = new SendeRezeptdatensignatur();
		encoding_length++;
		c.setSig(((SignedData) decode(in)));
		return c;
	}

	private void compSizeSendeRezeptdatensignatur(SendeRezeptdatensignatur c)
			throws Exception {
		encoding_length++;
		compSize(c.getSig());
	}

	public void encodeSendeRezept(SendeRezept c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEREZEPT;

		encode(c.getEd());

		encode(c.getSig());
	}

	public SendeRezept decodeSendeRezept(byte[] in) throws Exception {
		SendeRezept c = new SendeRezept();
		encoding_length++;
		c.setEd(((EncDataSymm) decode(in)));
		c.setSig(((SignedData) decode(in)));
		return c;
	}

	private void compSizeSendeRezept(SendeRezept c) throws Exception {
		encoding_length++;
		compSize(c.getEd());
		compSize(c.getSig());
	}

	public void encodeUBestaetigeRezeptAusstellen(UBestaetigeRezeptAusstellen c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.UBESTAETIGEREZEPTAUSSTELLEN;
	}

	public UBestaetigeRezeptAusstellen decodeUBestaetigeRezeptAusstellen(
			byte[] in) throws Exception {
		UBestaetigeRezeptAusstellen c = new UBestaetigeRezeptAusstellen();
		encoding_length++;
		return c;
	}

	private void compSizeUBestaetigeRezeptAusstellen(
			UBestaetigeRezeptAusstellen c) throws Exception {
		encoding_length++;
	}

	public void encodeErfrageNonceRezept(ErfrageNonceRezept c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEREZEPT;
	}

	public ErfrageNonceRezept decodeErfrageNonceRezept(byte[] in)
			throws Exception {
		ErfrageNonceRezept c = new ErfrageNonceRezept();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceRezept(ErfrageNonceRezept c)
			throws Exception {
		encoding_length++;
	}

	public void encodeBestaetigeRezeptAusstellen(BestaetigeRezeptAusstellen c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGEREZEPTAUSSTELLEN;
	}

	public BestaetigeRezeptAusstellen decodeBestaetigeRezeptAusstellen(byte[] in)
			throws Exception {
		BestaetigeRezeptAusstellen c = new BestaetigeRezeptAusstellen();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigeRezeptAusstellen(BestaetigeRezeptAusstellen c)
			throws Exception {
		encoding_length++;
	}

	public void encodeEndAuth(EndAuth c) throws Exception {
		encodeDestination[encoding_length++] = Code.ENDAUTH;
	}

	public EndAuth decodeEndAuth(byte[] in) throws Exception {
		EndAuth c = new EndAuth();
		encoding_length++;
		return c;
	}

	private void compSizeEndAuth(EndAuth c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceVerschluesseltHBA(SendeNonceVerschluesseltHBA c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEVERSCHLUESSELTHBA;

		encode(c.getEnc());
	}

	public SendeNonceVerschluesseltHBA decodeSendeNonceVerschluesseltHBA(
			byte[] in) throws Exception {
		SendeNonceVerschluesseltHBA c = new SendeNonceVerschluesseltHBA();
		encoding_length++;
		c.setEnc(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSendeNonceVerschluesseltHBA(
			SendeNonceVerschluesseltHBA c) throws Exception {
		encoding_length++;
		compSize(c.getEnc());
	}

	public void encodeSendeSchluessel(SendeSchluessel c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDESCHLUESSEL;

		encode(c.getEnc());
	}

	public SendeSchluessel decodeSendeSchluessel(byte[] in) throws Exception {
		SendeSchluessel c = new SendeSchluessel();
		encoding_length++;
		c.setEnc(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeSchluessel(SendeSchluessel c) throws Exception {
		encoding_length++;
		compSize(c.getEnc());
	}

	public void encodeErfrageNonceEGKWeiter(ErfrageNonceEGKWeiter c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEEGKWEITER;
	}

	public ErfrageNonceEGKWeiter decodeErfrageNonceEGKWeiter(byte[] in)
			throws Exception {
		ErfrageNonceEGKWeiter c = new ErfrageNonceEGKWeiter();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceEGKWeiter(ErfrageNonceEGKWeiter c)
			throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceVerschluesseltEGK(SendeNonceVerschluesseltEGK c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEVERSCHLUESSELTEGK;

		encode(c.getEnc());
	}

	public SendeNonceVerschluesseltEGK decodeSendeNonceVerschluesseltEGK(
			byte[] in) throws Exception {
		SendeNonceVerschluesseltEGK c = new SendeNonceVerschluesseltEGK();
		encoding_length++;
		c.setEnc(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeNonceVerschluesseltEGK(
			SendeNonceVerschluesseltEGK c) throws Exception {
		encoding_length++;
		compSize(c.getEnc());
	}

	public void encodeSendeNonceVerschluesseltEGKWeiter(
			SendeNonceVerschluesseltEGKWeiter c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEVERSCHLUESSELTEGKWEITER;

		encode(c.getEnc());
	}

	public SendeNonceVerschluesseltEGKWeiter decodeSendeNonceVerschluesseltEGKWeiter(
			byte[] in) throws Exception {
		SendeNonceVerschluesseltEGKWeiter c = new SendeNonceVerschluesseltEGKWeiter();
		encoding_length++;
		c.setEnc(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeNonceVerschluesseltEGKWeiter(
			SendeNonceVerschluesseltEGKWeiter c) throws Exception {
		encoding_length++;
		compSize(c.getEnc());
	}

	public void encodeSendeSessionkey(SendeSessionkey c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDESESSIONKEY;

		encode(c.getEnc());
	}

	public SendeSessionkey decodeSendeSessionkey(byte[] in) throws Exception {
		SendeSessionkey c = new SendeSessionkey();
		encoding_length++;
		c.setEnc(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeSessionkey(SendeSessionkey c) throws Exception {
		encoding_length++;
		compSize(c.getEnc());
	}

	public void encodeUBestaetigeSchluesselaustausch(
			UBestaetigeSchluesselaustausch c) throws Exception {
		encodeDestination[encoding_length++] = Code.UBESTAETIGESCHLUESSELAUSTAUSCH;
	}

	public UBestaetigeSchluesselaustausch decodeUBestaetigeSchluesselaustausch(
			byte[] in) throws Exception {
		UBestaetigeSchluesselaustausch c = new UBestaetigeSchluesselaustausch();
		encoding_length++;
		return c;
	}

	private void compSizeUBestaetigeSchluesselaustausch(
			UBestaetigeSchluesselaustausch c) throws Exception {
		encoding_length++;
	}

	public void encodeBestaetigeSchluesselaustausch(
			BestaetigeSchluesselaustausch c) throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGESCHLUESSELAUSTAUSCH;
	}

	public BestaetigeSchluesselaustausch decodeBestaetigeSchluesselaustausch(
			byte[] in) throws Exception {
		BestaetigeSchluesselaustausch c = new BestaetigeSchluesselaustausch();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigeSchluesselaustausch(
			BestaetigeSchluesselaustausch c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeChallengeHBAWeiter(SendeChallengeHBAWeiter c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEHBAWEITER;

		encode(c.getEncCard());

		encode(c.getEnc());
	}

	public SendeChallengeHBAWeiter decodeSendeChallengeHBAWeiter(byte[] in)
			throws Exception {
		SendeChallengeHBAWeiter c = new SendeChallengeHBAWeiter();
		encoding_length++;
		c.setEncCard(((EncDataAsymm) decode(in)));
		c.setEnc(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeChallengeHBAWeiter(SendeChallengeHBAWeiter c)
			throws Exception {
		encoding_length++;
		compSize(c.getEncCard());
		compSize(c.getEnc());
	}

	public void encodeSendeChallengeEGKWeiter(SendeChallengeEGKWeiter c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEEGKWEITER;

		encode(c.getEncCard());

		encode(c.getEnc());
	}

	public SendeChallengeEGKWeiter decodeSendeChallengeEGKWeiter(byte[] in)
			throws Exception {
		SendeChallengeEGKWeiter c = new SendeChallengeEGKWeiter();
		encoding_length++;
		c.setEncCard(((EncDataAsymm) decode(in)));
		c.setEnc(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeChallengeEGKWeiter(SendeChallengeEGKWeiter c)
			throws Exception {
		encoding_length++;
		compSize(c.getEncCard());
		compSize(c.getEnc());
	}

	public void encodeRetAuthentisiereEGKWeiter(RetAuthentisiereEGKWeiter c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.RETAUTHENTISIEREEGKWEITER;

		encode(c.getZertPatient());
	}

	public RetAuthentisiereEGKWeiter decodeRetAuthentisiereEGKWeiter(byte[] in)
			throws Exception {
		RetAuthentisiereEGKWeiter c = new RetAuthentisiereEGKWeiter();
		encoding_length++;
		c.setZertPatient(((ZertifikatPatient) decode(in)));
		return c;
	}

	private void compSizeRetAuthentisiereEGKWeiter(RetAuthentisiereEGKWeiter c)
			throws Exception {
		encoding_length++;
		compSize(c.getZertPatient());
	}

	public void encodeRetSendeChallengeEGKWeiter(RetSendeChallengeEGKWeiter c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.RETSENDECHALLENGEEGKWEITER;

		encode(c.getNonce());
	}

	public RetSendeChallengeEGKWeiter decodeRetSendeChallengeEGKWeiter(byte[] in)
			throws Exception {
		RetSendeChallengeEGKWeiter c = new RetSendeChallengeEGKWeiter();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeRetSendeChallengeEGKWeiter(RetSendeChallengeEGKWeiter c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeSendeZertifikatHBA(SendeZertifikatHBA c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEZERTIFIKATHBA;

		encode(c.getZertLeistungserbringer());
	}

	public SendeZertifikatHBA decodeSendeZertifikatHBA(byte[] in)
			throws Exception {
		SendeZertifikatHBA c = new SendeZertifikatHBA();
		encoding_length++;
		c.setZertLeistungserbringer(((ZertifikatLeistungserbringer) decode(in)));
		return c;
	}

	private void compSizeSendeZertifikatHBA(SendeZertifikatHBA c)
			throws Exception {
		encoding_length++;
		compSize(c.getZertLeistungserbringer());
	}

	public void encodeSendeSessionkeyWeiter(SendeSessionkeyWeiter c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDESESSIONKEYWEITER;

		encode(c.getEnc());
	}

	public SendeSessionkeyWeiter decodeSendeSessionkeyWeiter(byte[] in)
			throws Exception {
		SendeSessionkeyWeiter c = new SendeSessionkeyWeiter();
		encoding_length++;
		c.setEnc(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeSessionkeyWeiter(SendeSessionkeyWeiter c)
			throws Exception {
		encoding_length++;
		compSize(c.getEnc());
	}

	public void encodeURezeptEinloesen(URezeptEinloesen c) throws Exception {
		encodeDestination[encoding_length++] = Code.UREZEPTEINLOESEN;

		encodeInt((int) c.getIndex());
	}

	public URezeptEinloesen decodeURezeptEinloesen(byte[] in) throws Exception {
		URezeptEinloesen c = new URezeptEinloesen();
		encoding_length++;
		c.setIndex(decodeInt(in));
		return c;
	}

	private void compSizeURezeptEinloesen(URezeptEinloesen c) throws Exception {
		encoding_length++;

		compSizeInt(c.getIndex());
	}

	public void encodeURezeptausgabe(URezeptausgabe c) throws Exception {
		encodeDestination[encoding_length++] = Code.UREZEPTAUSGABE;

		encode(c.getRezeptdaten());
	}

	public URezeptausgabe decodeURezeptausgabe(byte[] in) throws Exception {
		URezeptausgabe c = new URezeptausgabe();
		encoding_length++;
		c.setRezeptdaten(((ERezeptdaten) decode(in)));
		return c;
	}

	private void compSizeURezeptausgabe(URezeptausgabe c) throws Exception {
		encoding_length++;
		compSize(c.getRezeptdaten());
	}

	public void encodeErfrageNonceFuerEinloesen(ErfrageNonceFuerEinloesen c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEFUEREINLOESEN;
	}

	public ErfrageNonceFuerEinloesen decodeErfrageNonceFuerEinloesen(byte[] in)
			throws Exception {
		ErfrageNonceFuerEinloesen c = new ErfrageNonceFuerEinloesen();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceFuerEinloesen(ErfrageNonceFuerEinloesen c)
			throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceFuerEinloesen(SendeNonceFuerEinloesen c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEFUEREINLOESEN;

		encode(c.getNonce());
	}

	public SendeNonceFuerEinloesen decodeSendeNonceFuerEinloesen(byte[] in)
			throws Exception {
		SendeNonceFuerEinloesen c = new SendeNonceFuerEinloesen();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeSendeNonceFuerEinloesen(SendeNonceFuerEinloesen c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeEinloesen(Einloesen c) throws Exception {
		encodeDestination[encoding_length++] = Code.EINLOESEN;

		encode(c.getIn());
	}

	public Einloesen decodeEinloesen(byte[] in) throws Exception {
		Einloesen c = new Einloesen();
		encoding_length++;
		c.setIn(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeEinloesen(Einloesen c) throws Exception {
		encoding_length++;
		compSize(c.getIn());
	}

	public void encodeSendeRezeptZumEinloesen(SendeRezeptZumEinloesen c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEREZEPTZUMEINLOESEN;

		encode(c.getRezeptdatenUNonce());

		encode(c.getSig());

		encode(c.getZertArzt());
	}

	public SendeRezeptZumEinloesen decodeSendeRezeptZumEinloesen(byte[] in)
			throws Exception {
		SendeRezeptZumEinloesen c = new SendeRezeptZumEinloesen();
		encoding_length++;
		c.setRezeptdatenUNonce(((EncDataSymm) decode(in)));
		c.setSig(((SignedData) decode(in)));
		c.setZertArzt(((ZertifikatLeistungserbringer) decode(in)));
		return c;
	}

	private void compSizeSendeRezeptZumEinloesen(SendeRezeptZumEinloesen c)
			throws Exception {
		encoding_length++;
		compSize(c.getRezeptdatenUNonce());
		compSize(c.getSig());
		compSize(c.getZertArzt());
	}

	public void encodeUErstelleNotfalldaten(UErstelleNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.UERSTELLENOTFALLDATEN;

		encodeString((String) c.getNotfalldaten());
	}

	public UErstelleNotfalldaten decodeUErstelleNotfalldaten(byte[] in)
			throws Exception {
		UErstelleNotfalldaten c = new UErstelleNotfalldaten();
		encoding_length++;
		c.setNotfalldaten(decodeString(in));
		return c;
	}

	private void compSizeUErstelleNotfalldaten(UErstelleNotfalldaten c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getNotfalldaten());
	}

	public void encodeErfrageNonceNotfalldatenErstellen(
			ErfrageNonceNotfalldatenErstellen c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCENOTFALLDATENERSTELLEN;
	}

	public ErfrageNonceNotfalldatenErstellen decodeErfrageNonceNotfalldatenErstellen(
			byte[] in) throws Exception {
		ErfrageNonceNotfalldatenErstellen c = new ErfrageNonceNotfalldatenErstellen();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceNotfalldatenErstellen(
			ErfrageNonceNotfalldatenErstellen c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceNotfalldatenErstellen(
			SendeNonceNotfalldatenErstellen c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCENOTFALLDATENERSTELLEN;

		encode(c.getNoncedata());
	}

	public SendeNonceNotfalldatenErstellen decodeSendeNonceNotfalldatenErstellen(
			byte[] in) throws Exception {
		SendeNonceNotfalldatenErstellen c = new SendeNonceNotfalldatenErstellen();
		encoding_length++;
		c.setNoncedata(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSendeNonceNotfalldatenErstellen(
			SendeNonceNotfalldatenErstellen c) throws Exception {
		encoding_length++;
		compSize(c.getNoncedata());
	}

	public void encodeSendeNotfalldaten(SendeNotfalldaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENOTFALLDATEN;

		encode(c.getNotfalldaten());
	}

	public SendeNotfalldaten decodeSendeNotfalldaten(byte[] in)
			throws Exception {
		SendeNotfalldaten c = new SendeNotfalldaten();
		encoding_length++;
		c.setNotfalldaten(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSendeNotfalldaten(SendeNotfalldaten c)
			throws Exception {
		encoding_length++;
		compSize(c.getNotfalldaten());
	}

	public void encodeSendeNotfalldatensignatur(SendeNotfalldatensignatur c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENOTFALLDATENSIGNATUR;

		encode(c.getSig());
	}

	public SendeNotfalldatensignatur decodeSendeNotfalldatensignatur(byte[] in)
			throws Exception {
		SendeNotfalldatensignatur c = new SendeNotfalldatensignatur();
		encoding_length++;
		c.setSig(((SignedData) decode(in)));
		return c;
	}

	private void compSizeSendeNotfalldatensignatur(SendeNotfalldatensignatur c)
			throws Exception {
		encoding_length++;
		compSize(c.getSig());
	}

	public void encodeSendeNotfalldatenUndSig(SendeNotfalldatenUndSig c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENOTFALLDATENUNDSIG;

		encode(c.getNfd());

		encode(c.getSig());
	}

	public SendeNotfalldatenUndSig decodeSendeNotfalldatenUndSig(byte[] in)
			throws Exception {
		SendeNotfalldatenUndSig c = new SendeNotfalldatenUndSig();
		encoding_length++;
		c.setNfd(((EncDataSymm) decode(in)));
		c.setSig(((SignedData) decode(in)));
		return c;
	}

	private void compSizeSendeNotfalldatenUndSig(SendeNotfalldatenUndSig c)
			throws Exception {
		encoding_length++;
		compSize(c.getNfd());
		compSize(c.getSig());
	}

	public void encodeBestaetigeNotfalldatenErstellen(
			BestaetigeNotfalldatenErstellen c) throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGENOTFALLDATENERSTELLEN;
	}

	public BestaetigeNotfalldatenErstellen decodeBestaetigeNotfalldatenErstellen(
			byte[] in) throws Exception {
		BestaetigeNotfalldatenErstellen c = new BestaetigeNotfalldatenErstellen();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigeNotfalldatenErstellen(
			BestaetigeNotfalldatenErstellen c) throws Exception {
		encoding_length++;
	}

	public void encodeUBestaetigeNotfalldatenErstellen(
			UBestaetigeNotfalldatenErstellen c) throws Exception {
		encodeDestination[encoding_length++] = Code.UBESTAETIGENOTFALLDATENERSTELLEN;
	}

	public UBestaetigeNotfalldatenErstellen decodeUBestaetigeNotfalldatenErstellen(
			byte[] in) throws Exception {
		UBestaetigeNotfalldatenErstellen c = new UBestaetigeNotfalldatenErstellen();
		encoding_length++;
		return c;
	}

	private void compSizeUBestaetigeNotfalldatenErstellen(
			UBestaetigeNotfalldatenErstellen c) throws Exception {
		encoding_length++;
	}

	public void encodeULeseNotfalldaten(ULeseNotfalldaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.ULESENOTFALLDATEN;
	}

	public ULeseNotfalldaten decodeULeseNotfalldaten(byte[] in)
			throws Exception {
		ULeseNotfalldaten c = new ULeseNotfalldaten();
		encoding_length++;
		return c;
	}

	private void compSizeULeseNotfalldaten(ULeseNotfalldaten c)
			throws Exception {
		encoding_length++;
	}

	public void encodePINArztEingegeben(PINArztEingegeben c) throws Exception {
		encodeDestination[encoding_length++] = Code.PINARZTEINGEGEBEN;

		encode(c.getNonce());
	}

	public PINArztEingegeben decodePINArztEingegeben(byte[] in)
			throws Exception {
		PINArztEingegeben c = new PINArztEingegeben();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizePINArztEingegeben(PINArztEingegeben c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodePINOK(PINOK c) throws Exception {
		encodeDestination[encoding_length++] = Code.PINOK;

		encode(c.getNoncedata());
	}

	public PINOK decodePINOK(byte[] in) throws Exception {
		PINOK c = new PINOK();
		encoding_length++;
		c.setNoncedata(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizePINOK(PINOK c) throws Exception {
		encoding_length++;
		compSize(c.getNoncedata());
	}

	public void encodeErfrageNotfalldaten(ErfrageNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENOTFALLDATEN;

		encode(c.getNonce());
	}

	public ErfrageNotfalldaten decodeErfrageNotfalldaten(byte[] in)
			throws Exception {
		ErfrageNotfalldaten c = new ErfrageNotfalldaten();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeErfrageNotfalldaten(ErfrageNotfalldaten c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeSchickeNotfalldaten(SchickeNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SCHICKENOTFALLDATEN;

		encode(c.getNfd());
	}

	public SchickeNotfalldaten decodeSchickeNotfalldaten(byte[] in)
			throws Exception {
		SchickeNotfalldaten c = new SchickeNotfalldaten();
		encoding_length++;
		c.setNfd(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSchickeNotfalldaten(SchickeNotfalldaten c)
			throws Exception {
		encoding_length++;
		compSize(c.getNfd());
	}

	public void encodeUSendeNotfalldaten(USendeNotfalldaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.USENDENOTFALLDATEN;

		encodeString((String) c.getDaten());
	}

	public USendeNotfalldaten decodeUSendeNotfalldaten(byte[] in)
			throws Exception {
		USendeNotfalldaten c = new USendeNotfalldaten();
		encoding_length++;
		c.setDaten(decodeString(in));
		return c;
	}

	private void compSizeUSendeNotfalldaten(USendeNotfalldaten c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getDaten());
	}

	public void encodeUAuthEGKUndKiosk(UAuthEGKUndKiosk c) throws Exception {
		encodeDestination[encoding_length++] = Code.UAUTHEGKUNDKIOSK;
	}

	public UAuthEGKUndKiosk decodeUAuthEGKUndKiosk(byte[] in) throws Exception {
		UAuthEGKUndKiosk c = new UAuthEGKUndKiosk();
		encoding_length++;
		return c;
	}

	private void compSizeUAuthEGKUndKiosk(UAuthEGKUndKiosk c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeZertKiosk(SendeZertKiosk c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEZERTKIOSK;

		encode(c.getZertKiosk());
	}

	public SendeZertKiosk decodeSendeZertKiosk(byte[] in) throws Exception {
		SendeZertKiosk c = new SendeZertKiosk();
		encoding_length++;
		c.setZertKiosk(((ZertifikatKiosk) decode(in)));
		return c;
	}

	private void compSizeSendeZertKiosk(SendeZertKiosk c) throws Exception {
		encoding_length++;
		compSize(c.getZertKiosk());
	}

	public void encodeSendeZertEGK(SendeZertEGK c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEZERTEGK;

		encode(c.getNd());

		encode(c.getZ());
	}

	public SendeZertEGK decodeSendeZertEGK(byte[] in) throws Exception {
		SendeZertEGK c = new SendeZertEGK();
		encoding_length++;
		c.setNd(((EncDataAsymm) decode(in)));
		c.setZ(((ZertifikatPatient) decode(in)));
		return c;
	}

	private void compSizeSendeZertEGK(SendeZertEGK c) throws Exception {
		encoding_length++;
		compSize(c.getNd());
		compSize(c.getZ());
	}

	public void encodeSendeNonces(SendeNonces c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCES;

		encode(c.getNonce());

		encode(c.getNd());
	}

	public SendeNonces decodeSendeNonces(byte[] in) throws Exception {
		SendeNonces c = new SendeNonces();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setNd(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeNonces(SendeNonces c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getNd());
	}

	public void encodeBestaetigeNonce(BestaetigeNonce c) throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGENONCE;

		encode(c.getN());
	}

	public BestaetigeNonce decodeBestaetigeNonce(byte[] in) throws Exception {
		BestaetigeNonce c = new BestaetigeNonce();
		encoding_length++;
		c.setN(((Nonce) decode(in)));
		return c;
	}

	private void compSizeBestaetigeNonce(BestaetigeNonce c) throws Exception {
		encoding_length++;
		compSize(c.getN());
	}

	public void encodeUAuthOK(UAuthOK c) throws Exception {
		encodeDestination[encoding_length++] = Code.UAUTHOK;
	}

	public UAuthOK decodeUAuthOK(byte[] in) throws Exception {
		UAuthOK c = new UAuthOK();
		encoding_length++;
		return c;
	}

	private void compSizeUAuthOK(UAuthOK c) throws Exception {
		encoding_length++;
	}

	public void encodeUPINEingabeKiosk(UPINEingabeKiosk c) throws Exception {
		encodeDestination[encoding_length++] = Code.UPINEINGABEKIOSK;

		encode(c.getPin());
	}

	public UPINEingabeKiosk decodeUPINEingabeKiosk(byte[] in) throws Exception {
		UPINEingabeKiosk c = new UPINEingabeKiosk();
		encoding_length++;
		c.setPin(((Secret) decode(in)));
		return c;
	}

	private void compSizeUPINEingabeKiosk(UPINEingabeKiosk c) throws Exception {
		encoding_length++;
		compSize(c.getPin());
	}

	public void encodeErfrageNoncePIN(ErfrageNoncePIN c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEPIN;
	}

	public ErfrageNoncePIN decodeErfrageNoncePIN(byte[] in) throws Exception {
		ErfrageNoncePIN c = new ErfrageNoncePIN();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNoncePIN(ErfrageNoncePIN c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNoncePIN(SendeNoncePIN c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCEPIN;

		encode(c.getNonce());
	}

	public SendeNoncePIN decodeSendeNoncePIN(byte[] in) throws Exception {
		SendeNoncePIN c = new SendeNoncePIN();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeSendeNoncePIN(SendeNoncePIN c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeSendePIN(SendePIN c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEPIN;

		encode(c.getPindata());
	}

	public SendePIN decodeSendePIN(byte[] in) throws Exception {
		SendePIN c = new SendePIN();
		encoding_length++;
		c.setPindata(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendePIN(SendePIN c) throws Exception {
		encoding_length++;
		compSize(c.getPindata());
	}

	public void encodeBestaetigePIN(BestaetigePIN c) throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGEPIN;
	}

	public BestaetigePIN decodeBestaetigePIN(byte[] in) throws Exception {
		BestaetigePIN c = new BestaetigePIN();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigePIN(BestaetigePIN c) throws Exception {
		encoding_length++;
	}

	public void encodeUBestaetigePIN(UBestaetigePIN c) throws Exception {
		encodeDestination[encoding_length++] = Code.UBESTAETIGEPIN;
	}

	public UBestaetigePIN decodeUBestaetigePIN(byte[] in) throws Exception {
		UBestaetigePIN c = new UBestaetigePIN();
		encoding_length++;
		return c;
	}

	private void compSizeUBestaetigePIN(UBestaetigePIN c) throws Exception {
		encoding_length++;
	}

	public void encodeUSchluesselaustausch(USchluesselaustausch c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.USCHLUESSELAUSTAUSCH;
	}

	public USchluesselaustausch decodeUSchluesselaustausch(byte[] in)
			throws Exception {
		USchluesselaustausch c = new USchluesselaustausch();
		encoding_length++;
		return c;
	}

	private void compSizeUSchluesselaustausch(USchluesselaustausch c)
			throws Exception {
		encoding_length++;
	}

	public void encodeErfrageNonceSchluesselaustausch(
			ErfrageNonceSchluesselaustausch c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCESCHLUESSELAUSTAUSCH;
	}

	public ErfrageNonceSchluesselaustausch decodeErfrageNonceSchluesselaustausch(
			byte[] in) throws Exception {
		ErfrageNonceSchluesselaustausch c = new ErfrageNonceSchluesselaustausch();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceSchluesselaustausch(
			ErfrageNonceSchluesselaustausch c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonceSchluesselaustausch(
			SendeNonceSchluesselaustausch c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCESCHLUESSELAUSTAUSCH;

		encode(c.getNd());
	}

	public SendeNonceSchluesselaustausch decodeSendeNonceSchluesselaustausch(
			byte[] in) throws Exception {
		SendeNonceSchluesselaustausch c = new SendeNonceSchluesselaustausch();
		encoding_length++;
		c.setNd(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeNonceSchluesselaustausch(
			SendeNonceSchluesselaustausch c) throws Exception {
		encoding_length++;
		compSize(c.getNd());
	}

	public void encodeSendeSessionkeyKiosk(SendeSessionkeyKiosk c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDESESSIONKEYKIOSK;

		encode(c.getKd());
	}

	public SendeSessionkeyKiosk decodeSendeSessionkeyKiosk(byte[] in)
			throws Exception {
		SendeSessionkeyKiosk c = new SendeSessionkeyKiosk();
		encoding_length++;
		c.setKd(((EncDataAsymm) decode(in)));
		return c;
	}

	private void compSizeSendeSessionkeyKiosk(SendeSessionkeyKiosk c)
			throws Exception {
		encoding_length++;
		compSize(c.getKd());
	}

	public void encodeSchluesselKioskOK(SchluesselKioskOK c) throws Exception {
		encodeDestination[encoding_length++] = Code.SCHLUESSELKIOSKOK;
	}

	public SchluesselKioskOK decodeSchluesselKioskOK(byte[] in)
			throws Exception {
		SchluesselKioskOK c = new SchluesselKioskOK();
		encoding_length++;
		return c;
	}

	private void compSizeSchluesselKioskOK(SchluesselKioskOK c)
			throws Exception {
		encoding_length++;
	}

	public void encodeUSchluesselKioskOK(USchluesselKioskOK c) throws Exception {
		encodeDestination[encoding_length++] = Code.USCHLUESSELKIOSKOK;
	}

	public USchluesselKioskOK decodeUSchluesselKioskOK(byte[] in)
			throws Exception {
		USchluesselKioskOK c = new USchluesselKioskOK();
		encoding_length++;
		return c;
	}

	private void compSizeUSchluesselKioskOK(USchluesselKioskOK c)
			throws Exception {
		encoding_length++;
	}

	public void encodeULeseStammUndNotfalldaten(ULeseStammUndNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ULESESTAMMUNDNOTFALLDATEN;
	}

	public ULeseStammUndNotfalldaten decodeULeseStammUndNotfalldaten(byte[] in)
			throws Exception {
		ULeseStammUndNotfalldaten c = new ULeseStammUndNotfalldaten();
		encoding_length++;
		return c;
	}

	private void compSizeULeseStammUndNotfalldaten(ULeseStammUndNotfalldaten c)
			throws Exception {
		encoding_length++;
	}

	public void encodeLeseStammUndNotfalldaten(LeseStammUndNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.LESESTAMMUNDNOTFALLDATEN;

		encode(c.getNd());
	}

	public LeseStammUndNotfalldaten decodeLeseStammUndNotfalldaten(byte[] in)
			throws Exception {
		LeseStammUndNotfalldaten c = new LeseStammUndNotfalldaten();
		encoding_length++;
		c.setNd(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeLeseStammUndNotfalldaten(LeseStammUndNotfalldaten c)
			throws Exception {
		encoding_length++;
		compSize(c.getNd());
	}

	public void encodeSendeStammUndNotfalldaten(SendeStammUndNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.SENDESTAMMUNDNOTFALLDATEN;

		encode(c.getSd());
	}

	public SendeStammUndNotfalldaten decodeSendeStammUndNotfalldaten(byte[] in)
			throws Exception {
		SendeStammUndNotfalldaten c = new SendeStammUndNotfalldaten();
		encoding_length++;
		c.setSd(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSendeStammUndNotfalldaten(SendeStammUndNotfalldaten c)
			throws Exception {
		encoding_length++;
		compSize(c.getSd());
	}

	public void encodeUSendeStammUndNotfalldaten(USendeStammUndNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.USENDESTAMMUNDNOTFALLDATEN;

		encodeString((String) c.getNotfalldaten());

		encodeInt((int) c.getAnzahlRezepte());

		encode(c.getVsd());
	}

	public USendeStammUndNotfalldaten decodeUSendeStammUndNotfalldaten(byte[] in)
			throws Exception {
		USendeStammUndNotfalldaten c = new USendeStammUndNotfalldaten();
		encoding_length++;
		c.setNotfalldaten(decodeString(in));
		c.setAnzahlRezepte(decodeInt(in));
		c.setVsd(((Versichertenstammdaten) decode(in)));
		return c;
	}

	private void compSizeUSendeStammUndNotfalldaten(USendeStammUndNotfalldaten c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getNotfalldaten());

		compSizeInt(c.getAnzahlRezepte());
		compSize(c.getVsd());
	}

	public void encodeULeseRezept(ULeseRezept c) throws Exception {
		encodeDestination[encoding_length++] = Code.ULESEREZEPT;

		encodeInt((int) c.getIndex());
	}

	public ULeseRezept decodeULeseRezept(byte[] in) throws Exception {
		ULeseRezept c = new ULeseRezept();
		encoding_length++;
		c.setIndex(decodeInt(in));
		return c;
	}

	private void compSizeULeseRezept(ULeseRezept c) throws Exception {
		encoding_length++;

		compSizeInt(c.getIndex());
	}

	public void encodeLeseRezept(LeseRezept c) throws Exception {
		encodeDestination[encoding_length++] = Code.LESEREZEPT;

		encode(c.getNdi());
	}

	public LeseRezept decodeLeseRezept(byte[] in) throws Exception {
		LeseRezept c = new LeseRezept();
		encoding_length++;
		c.setNdi(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeLeseRezept(LeseRezept c) throws Exception {
		encoding_length++;
		compSize(c.getNdi());
	}

	public void encodeSendeRezeptdaten(SendeRezeptdaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDEREZEPTDATEN;

		encode(c.getNur());
	}

	public SendeRezeptdaten decodeSendeRezeptdaten(byte[] in) throws Exception {
		SendeRezeptdaten c = new SendeRezeptdaten();
		encoding_length++;
		c.setNur(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeSendeRezeptdaten(SendeRezeptdaten c) throws Exception {
		encoding_length++;
		compSize(c.getNur());
	}

	public void encodeUSendeRezept(USendeRezept c) throws Exception {
		encodeDestination[encoding_length++] = Code.USENDEREZEPT;

		encode(c.getRd());
	}

	public USendeRezept decodeUSendeRezept(byte[] in) throws Exception {
		USendeRezept c = new USendeRezept();
		encoding_length++;
		c.setRd(((ERezeptdaten) decode(in)));
		return c;
	}

	private void compSizeUSendeRezept(USendeRezept c) throws Exception {
		encoding_length++;
		compSize(c.getRd());
	}

	public void encodeUNotfalldatenLoeschen(UNotfalldatenLoeschen c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.UNOTFALLDATENLOESCHEN;
	}

	public UNotfalldatenLoeschen decodeUNotfalldatenLoeschen(byte[] in)
			throws Exception {
		UNotfalldatenLoeschen c = new UNotfalldatenLoeschen();
		encoding_length++;
		return c;
	}

	private void compSizeUNotfalldatenLoeschen(UNotfalldatenLoeschen c)
			throws Exception {
		encoding_length++;
	}

	public void encodeErfrageNonceNotfalldatenLoeschen(
			ErfrageNonceNotfalldatenLoeschen c) throws Exception {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCENOTFALLDATENLOESCHEN;
	}

	public ErfrageNonceNotfalldatenLoeschen decodeErfrageNonceNotfalldatenLoeschen(
			byte[] in) throws Exception {
		ErfrageNonceNotfalldatenLoeschen c = new ErfrageNonceNotfalldatenLoeschen();
		encoding_length++;
		return c;
	}

	private void compSizeErfrageNonceNotfalldatenLoeschen(
			ErfrageNonceNotfalldatenLoeschen c) throws Exception {
		encoding_length++;
	}

	public void encodeLoescheNotfalldaten(LoescheNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.LOESCHENOTFALLDATEN;

		encode(c.getNsd());
	}

	public LoescheNotfalldaten decodeLoescheNotfalldaten(byte[] in)
			throws Exception {
		LoescheNotfalldaten c = new LoescheNotfalldaten();
		encoding_length++;
		c.setNsd(((EncDataSymm) decode(in)));
		return c;
	}

	private void compSizeLoescheNotfalldaten(LoescheNotfalldaten c)
			throws Exception {
		encoding_length++;
		compSize(c.getNsd());
	}

	public void encodeBestaetigeLoeschenNotfalldaten(
			BestaetigeLoeschenNotfalldaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.BESTAETIGELOESCHENNOTFALLDATEN;
	}

	public BestaetigeLoeschenNotfalldaten decodeBestaetigeLoeschenNotfalldaten(
			byte[] in) throws Exception {
		BestaetigeLoeschenNotfalldaten c = new BestaetigeLoeschenNotfalldaten();
		encoding_length++;
		return c;
	}

	private void compSizeBestaetigeLoeschenNotfalldaten(
			BestaetigeLoeschenNotfalldaten c) throws Exception {
		encoding_length++;
	}

	public void encodeUBestaetigeLoeschenNotfalldaten(
			UBestaetigeLoeschenNotfalldaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.UBESTAETIGELOESCHENNOTFALLDATEN;
	}

	public UBestaetigeLoeschenNotfalldaten decodeUBestaetigeLoeschenNotfalldaten(
			byte[] in) throws Exception {
		UBestaetigeLoeschenNotfalldaten c = new UBestaetigeLoeschenNotfalldaten();
		encoding_length++;
		return c;
	}

	private void compSizeUBestaetigeLoeschenNotfalldaten(
			UBestaetigeLoeschenNotfalldaten c) throws Exception {
		encoding_length++;
	}

	public void encodeSendeNonce(SendeNonce c) throws Exception {
		encodeDestination[encoding_length++] = Code.SENDENONCE;

		encode(c.getNonce());
	}

	public SendeNonce decodeSendeNonce(byte[] in) throws Exception {
		SendeNonce c = new SendeNonce();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeSendeNonce(SendeNonce c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeUUserOK(UUserOK c) throws Exception {
		encodeDestination[encoding_length++] = Code.UUSEROK;
	}

	public UUserOK decodeUUserOK(byte[] in) throws Exception {
		UUserOK c = new UUserOK();
		encoding_length++;
		return c;
	}

	private void compSizeUUserOK(UUserOK c) throws Exception {
		encoding_length++;
	}

	public void encodeUUserUpdate(UUserUpdate c) throws Exception {
		encodeDestination[encoding_length++] = Code.UUSERUPDATE;
	}

	public UUserUpdate decodeUUserUpdate(byte[] in) throws Exception {
		UUserUpdate c = new UUserUpdate();
		encoding_length++;
		return c;
	}

	private void compSizeUUserUpdate(UUserUpdate c) throws Exception {
		encoding_length++;
	}

	public void encodeGetData(GetData c) throws Exception {
		encodeDestination[encoding_length++] = Code.GETDATA;
	}

	public GetData decodeGetData(byte[] in) throws Exception {
		GetData c = new GetData();
		encoding_length++;
		return c;
	}

	private void compSizeGetData(GetData c) throws Exception {
		encoding_length++;
	}

	public void encodeOK(OK c) throws Exception {
		encodeDestination[encoding_length++] = Code.OK;
	}

	public OK decodeOK(byte[] in) throws Exception {
		OK c = new OK();
		encoding_length++;
		return c;
	}

	private void compSizeOK(OK c) throws Exception {
		encoding_length++;
	}

	public void encodeCardData(CardData c) throws Exception {
		encodeDestination[encoding_length++] = Code.CARDDATA;

		encode(c.getNonce());

		encode(c.getDaten());

		encode(c.getSigdaten());

		encode(c.getZertEGK());
	}

	public CardData decodeCardData(byte[] in) throws Exception {
		CardData c = new CardData();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setDaten(((EncDataAsymm) decode(in)));
		c.setSigdaten(((SignedData) decode(in)));
		c.setZertEGK(((ZertifikatPatient) decode(in)));
		return c;
	}

	private void compSizeCardData(CardData c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getDaten());
		compSize(c.getSigdaten());
		compSize(c.getZertEGK());
	}

	public void encodeUpdate(Update c) throws Exception {
		encodeDestination[encoding_length++] = Code.UPDATE;

		encode(c.getDaten());

		encode(c.getSigdaten());

		encode(c.getSigWithNonce());
	}

	public Update decodeUpdate(byte[] in) throws Exception {
		Update c = new Update();
		encoding_length++;
		c.setDaten(((EncDataAsymm) decode(in)));
		c.setSigdaten(((SignedData) decode(in)));
		c.setSigWithNonce(((SignedData) decode(in)));
		return c;
	}

	private void compSizeUpdate(Update c) throws Exception {
		encoding_length++;
		compSize(c.getDaten());
		compSize(c.getSigdaten());
		compSize(c.getSigWithNonce());
	}

	public void encodeNonce(Nonce c) throws Exception {
		encodeDestination[encoding_length++] = Code.NONCE;

		encodeByteArray((byte[]) c.getNonce());
	}

	public Nonce decodeNonce(byte[] in) throws Exception {
		Nonce c = new Nonce();
		encoding_length++;
		c.setNonce(decodeByteArray(in));
		return c;
	}

	private void compSizeNonce(Nonce c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getNonce());
	}

	public void encodeZertifikatLeistungserbringer(
			ZertifikatLeistungserbringer c) throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATLEISTUNGSERBRINGER;

		encode(c.getSignedCert());

		encode(c.getData());
	}

	public ZertifikatLeistungserbringer decodeZertifikatLeistungserbringer(
			byte[] in) throws Exception {
		ZertifikatLeistungserbringer c = new ZertifikatLeistungserbringer();
		encoding_length++;
		c.setSignedCert(((SignedData) decode(in)));
		c.setData(((LeistungserbringerInfo) decode(in)));
		return c;
	}

	private void compSizeZertifikatLeistungserbringer(
			ZertifikatLeistungserbringer c) throws Exception {
		encoding_length++;
		compSize(c.getSignedCert());
		compSize(c.getData());
	}

	public void encodeZertifikatPatient(ZertifikatPatient c) throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATPATIENT;

		encode(c.getSignedCert());

		encode(c.getData());
	}

	public ZertifikatPatient decodeZertifikatPatient(byte[] in)
			throws Exception {
		ZertifikatPatient c = new ZertifikatPatient();
		encoding_length++;
		c.setSignedCert(((SignedData) decode(in)));
		c.setData(((ZertifikatPatientData) decode(in)));
		return c;
	}

	private void compSizeZertifikatPatient(ZertifikatPatient c)
			throws Exception {
		encoding_length++;
		compSize(c.getSignedCert());
		compSize(c.getData());
	}

	private void encodeEncDataAsymm(EncDataAsymm c) {

		encodeDestination[encoding_length++] = Code.ENCDATAASYMM;
		encodeByteArray(c.encrypted);

		encodeInt(c.plainlength);
		encodeInt(c.enclength);

		encodeByte(c.plainDataType);
	}

	public EncDataAsymm decodeEncDataAsymm(byte[] in) throws Exception {
		if (in[encoding_length++] != Code.ENCDATAASYMM)
			stop();
		EncDataAsymm into = new EncDataAsymm();
		//public byte[] encrypted;
		//public short plainlength;
		//public short enclength;
		//public byte plainDataType;
		into.encrypted = decodeByteArray(in);
		into.plainlength = decodeInt(in);
		into.enclength = decodeInt(in);
		into.plainDataType = decodeByte(in);
		return into;
	}

	private void compSizeEncDataAsymm(EncDataAsymm c) {
		encoding_length++;
		compSizeByteArray(c.encrypted);
		encoding_length += 8; // short + short + byte
	}

	public void encodeNonceData(NonceData c) throws Exception {
		encodeDestination[encoding_length++] = Code.NONCEDATA;

		encode(c.getNonce());
	}

	public NonceData decodeNonceData(byte[] in) throws Exception {
		NonceData c = new NonceData();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeNonceData(NonceData c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeSecret(Secret c) throws Exception {
		encodeDestination[encoding_length++] = Code.SECRET;

		encodeByteArray((byte[]) c.getSecret());
	}

	public Secret decodeSecret(byte[] in) throws Exception {
		Secret c = new Secret();
		encoding_length++;
		c.setSecret(decodeByteArray(in));
		return c;
	}

	private void compSizeSecret(Secret c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getSecret());
	}

	public void encodePINData(PINData c) throws Exception {
		encodeDestination[encoding_length++] = Code.PINDATA;

		encode(c.getPin());

		encode(c.getN());
	}

	public PINData decodePINData(byte[] in) throws Exception {
		PINData c = new PINData();
		encoding_length++;
		c.setPin(((Secret) decode(in)));
		c.setN(((Nonce) decode(in)));
		return c;
	}

	private void compSizePINData(PINData c) throws Exception {
		encoding_length++;
		compSize(c.getPin());
		compSize(c.getN());
	}

	private void encodeEncDataSymm(EncDataSymm c) {

		encodeDestination[encoding_length++] = Code.ENCDATASYMM;
		encodeByteArray(c.encrypted);

		encodeInt(c.plainlength);
		encodeInt(c.enclength);

		encodeByte(c.plainDataType);
	}

	public EncDataSymm decodeEncDataSymm(byte[] in) throws TerminalException {
		if (in[encoding_length++] != Code.ENCDATASYMM)
			stop();
		EncDataSymm into = new EncDataSymm();
		//public byte[] encrypted;
		//public short plainlength;
		//public short enclength;
		//public byte plainDataType;
		into.encrypted = decodeByteArray(in);
		into.plainlength = decodeInt(in);
		into.enclength = decodeInt(in);
		into.plainDataType = decodeByte(in);
		return into;
	}

	private void compSizeEncDataSymm(EncDataSymm c) {
		encoding_length++;
		compSizeByteArray(c.encrypted);
		encoding_length += 8; // short + short + byte
	}

	public void encodeEGKDaten(EGKDaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.EGKDATEN;

		encode(c.getNonce());

		encode(c.getDaten());
	}

	public EGKDaten decodeEGKDaten(byte[] in) throws Exception {
		EGKDaten c = new EGKDaten();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setDaten(((Versichertenstammdaten) decode(in)));
		return c;
	}

	private void compSizeEGKDaten(EGKDaten c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getDaten());
	}

	public void encodeArzneimittel(Arzneimittel c) throws Exception {
		encodeDestination[encoding_length++] = Code.ARZNEIMITTEL;

		encodeString((String) c.getName());

		encodeString((String) c.getEinnahmemenge());

		encodeString((String) c.getArzneimittelrisiko());
	}

	public Arzneimittel decodeArzneimittel(byte[] in) throws Exception {
		Arzneimittel c = new Arzneimittel();
		encoding_length++;
		c.setName(decodeString(in));
		c.setEinnahmemenge(decodeString(in));
		c.setArzneimittelrisiko(decodeString(in));
		return c;
	}

	private void compSizeArzneimittel(Arzneimittel c) throws Exception {
		encoding_length++;

		compSizeString(c.getName());

		compSizeString(c.getEinnahmemenge());

		compSizeString(c.getArzneimittelrisiko());
	}

	public void encodeERezeptdaten(ERezeptdaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.EREZEPTDATEN;

		encode(c.getNonce());

		encodeString((String) c.getAusstellungsdatum());

		encodeString((String) c.getAngabezureinnahme());

		encode(c.getArzneimittel());

		encode(c.getAusgestelltfuer());
	}

	public ERezeptdaten decodeERezeptdaten(byte[] in) throws Exception {
		ERezeptdaten c = new ERezeptdaten();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setAusstellungsdatum(decodeString(in));
		c.setAngabezureinnahme(decodeString(in));
		c.setArzneimittel(((Arzneimittel) decode(in)));
		c.setAusgestelltfuer(((Versichertenstammdaten) decode(in)));
		return c;
	}

	private void compSizeERezeptdaten(ERezeptdaten c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());

		compSizeString(c.getAusstellungsdatum());

		compSizeString(c.getAngabezureinnahme());
		compSize(c.getArzneimittel());
		compSize(c.getAusgestelltfuer());
	}

	public void encodeSignedData(SignedData c) throws Exception {
		encodeDestination[encoding_length++] = Code.SIGNEDDATA;

		encodeByteArray((byte[]) c.getSigned());
	}

	public SignedData decodeSignedData(byte[] in) throws Exception {
		SignedData c = new SignedData();
		encoding_length++;
		c.setSigned(decodeByteArray(in));
		return c;
	}

	private void compSizeSignedData(SignedData c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getSigned());
	}

	public void encodeSecretNonceData(SecretNonceData c) throws Exception {
		encodeDestination[encoding_length++] = Code.SECRETNONCEDATA;

		encode(c.getNonce());
	}

	public SecretNonceData decodeSecretNonceData(byte[] in) throws Exception {
		SecretNonceData c = new SecretNonceData();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		return c;
	}

	private void compSizeSecretNonceData(SecretNonceData c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
	}

	public void encodeKeyData(KeyData c) throws Exception {
		encodeDestination[encoding_length++] = Code.KEYDATA;

		encode(c.getNonce());

		encode(c.getSessionkey());
	}

	public KeyData decodeKeyData(byte[] in) throws Exception {
		KeyData c = new KeyData();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setSessionkey(((SymmKey) decode(in)));
		return c;
	}

	private void compSizeKeyData(KeyData c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getSessionkey());
	}

	public void encodeIndexNonce(IndexNonce c) throws Exception {
		encodeDestination[encoding_length++] = Code.INDEXNONCE;

		encodeInt((int) c.getIndex());

		encode(c.getNonce());

		encode(c.getNonceApothekenPC());
	}

	public IndexNonce decodeIndexNonce(byte[] in) throws Exception {
		IndexNonce c = new IndexNonce();
		encoding_length++;
		c.setIndex(decodeInt(in));
		c.setNonce(((Nonce) decode(in)));
		c.setNonceApothekenPC(((Nonce) decode(in)));
		return c;
	}

	private void compSizeIndexNonce(IndexNonce c) throws Exception {
		encoding_length++;

		compSizeInt(c.getIndex());
		compSize(c.getNonce());
		compSize(c.getNonceApothekenPC());
	}

	public void encodeRezeptdatenUndNonce(RezeptdatenUndNonce c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.REZEPTDATENUNDNONCE;

		encode(c.getNonceApothekenPC());

		encode(c.getRezeptdaten());
	}

	public RezeptdatenUndNonce decodeRezeptdatenUndNonce(byte[] in)
			throws Exception {
		RezeptdatenUndNonce c = new RezeptdatenUndNonce();
		encoding_length++;
		c.setNonceApothekenPC(((Nonce) decode(in)));
		c.setRezeptdaten(((ERezeptdaten) decode(in)));
		return c;
	}

	private void compSizeRezeptdatenUndNonce(RezeptdatenUndNonce c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonceApothekenPC());
		compSize(c.getRezeptdaten());
	}

	public void encodeNotfalldaten(Notfalldaten c) throws Exception {
		encodeDestination[encoding_length++] = Code.NOTFALLDATEN;

		encode(c.getNonce());

		encodeString((String) c.getDaten());
	}

	public Notfalldaten decodeNotfalldaten(byte[] in) throws Exception {
		Notfalldaten c = new Notfalldaten();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setDaten(decodeString(in));
		return c;
	}

	private void compSizeNotfalldaten(Notfalldaten c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());

		compSizeString(c.getDaten());
	}

	public void encodeNotfalldatenNonce(NotfalldatenNonce c) throws Exception {
		encodeDestination[encoding_length++] = Code.NOTFALLDATENNONCE;

		encode(c.getNonce());

		encodeString((String) c.getDaten());
	}

	public NotfalldatenNonce decodeNotfalldatenNonce(byte[] in)
			throws Exception {
		NotfalldatenNonce c = new NotfalldatenNonce();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setDaten(decodeString(in));
		return c;
	}

	private void compSizeNotfalldatenNonce(NotfalldatenNonce c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonce());

		compSizeString(c.getDaten());
	}

	public void encodeZertifikatKiosk(ZertifikatKiosk c) throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATKIOSK;

		encode(c.getData());

		encode(c.getSignedCert());
	}

	public ZertifikatKiosk decodeZertifikatKiosk(byte[] in) throws Exception {
		ZertifikatKiosk c = new ZertifikatKiosk();
		encoding_length++;
		c.setData(((ZertifikatKioskData) decode(in)));
		c.setSignedCert(((SignedData) decode(in)));
		return c;
	}

	private void compSizeZertifikatKiosk(ZertifikatKiosk c) throws Exception {
		encoding_length++;
		compSize(c.getData());
		compSize(c.getSignedCert());
	}

	public void encodeStammUndNotfalldaten(StammUndNotfalldaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.STAMMUNDNOTFALLDATEN;

		encodeString((String) c.getNotfalldaten());

		encode(c.getNonce());

		encodeInt((int) c.getAnzahlRezepte());

		encode(c.getVsd());
	}

	public StammUndNotfalldaten decodeStammUndNotfalldaten(byte[] in)
			throws Exception {
		StammUndNotfalldaten c = new StammUndNotfalldaten();
		encoding_length++;
		c.setNotfalldaten(decodeString(in));
		c.setNonce(((Nonce) decode(in)));
		c.setAnzahlRezepte(decodeInt(in));
		c.setVsd(((Versichertenstammdaten) decode(in)));
		return c;
	}

	private void compSizeStammUndNotfalldaten(StammUndNotfalldaten c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getNotfalldaten());
		compSize(c.getNonce());

		compSizeInt(c.getAnzahlRezepte());
		compSize(c.getVsd());
	}

	public void encodeVersichertenstammdaten(Versichertenstammdaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.VERSICHERTENSTAMMDATEN;

		encodeString((String) c.getName());

		encodeString((String) c.getVersichertennr());

		encodeInt((int) c.getKennummerKrankenkasse());

		encodeString((String) c.getAdresse());
	}

	public Versichertenstammdaten decodeVersichertenstammdaten(byte[] in)
			throws Exception {
		Versichertenstammdaten c = new Versichertenstammdaten();
		encoding_length++;
		c.setName(decodeString(in));
		c.setVersichertennr(decodeString(in));
		c.setKennummerKrankenkasse(decodeInt(in));
		c.setAdresse(decodeString(in));
		return c;
	}

	private void compSizeVersichertenstammdaten(Versichertenstammdaten c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());

		compSizeString(c.getVersichertennr());

		compSizeInt(c.getKennummerKrankenkasse());

		compSizeString(c.getAdresse());
	}

	public void encodeNonceDataAndIndex(NonceDataAndIndex c) throws Exception {
		encodeDestination[encoding_length++] = Code.NONCEDATAANDINDEX;

		encode(c.getNonce());

		encodeInt((int) c.getIndex());
	}

	public NonceDataAndIndex decodeNonceDataAndIndex(byte[] in)
			throws Exception {
		NonceDataAndIndex c = new NonceDataAndIndex();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setIndex(decodeInt(in));
		return c;
	}

	private void compSizeNonceDataAndIndex(NonceDataAndIndex c)
			throws Exception {
		encoding_length++;
		compSize(c.getNonce());

		compSizeInt(c.getIndex());
	}

	public void encodeNonceUndRezept(NonceUndRezept c) throws Exception {
		encodeDestination[encoding_length++] = Code.NONCEUNDREZEPT;

		encode(c.getNonce());

		encode(c.getRd());
	}

	public NonceUndRezept decodeNonceUndRezept(byte[] in) throws Exception {
		NonceUndRezept c = new NonceUndRezept();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setRd(((ERezeptdaten) decode(in)));
		return c;
	}

	private void compSizeNonceUndRezept(NonceUndRezept c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getRd());
	}

	public void encodeNonceStringData(NonceStringData c) throws Exception {
		encodeDestination[encoding_length++] = Code.NONCESTRINGDATA;

		encode(c.getNonce());

		encodeString((String) c.getS());
	}

	public NonceStringData decodeNonceStringData(byte[] in) throws Exception {
		NonceStringData c = new NonceStringData();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setS(decodeString(in));
		return c;
	}

	private void compSizeNonceStringData(NonceStringData c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());

		compSizeString(c.getS());
	}

	public void encodeUpdateAuth(UpdateAuth c) throws Exception {
		encodeDestination[encoding_length++] = Code.UPDATEAUTH;

		encode(c.getNonce());

		encode(c.getData());
	}

	public UpdateAuth decodeUpdateAuth(byte[] in) throws Exception {
		UpdateAuth c = new UpdateAuth();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setData(((Versichertenstammdaten) decode(in)));
		return c;
	}

	private void compSizeUpdateAuth(UpdateAuth c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getData());
	}

	public void encodeLeistungserbringerInfo(LeistungserbringerInfo c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.LEISTUNGSERBRINGERINFO;

		encodeString((String) c.getName());

		encode(c.getPubkey());

		encodeInt((int) c.getId());
	}

	public LeistungserbringerInfo decodeLeistungserbringerInfo(byte[] in)
			throws Exception {
		LeistungserbringerInfo c = new LeistungserbringerInfo();
		encoding_length++;
		c.setName(decodeString(in));
		c.setPubkey(((PublicKey) decode(in)));
		c.setId(decodeInt(in));
		return c;
	}

	private void compSizeLeistungserbringerInfo(LeistungserbringerInfo c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());
		compSize(c.getPubkey());

		compSizeInt(c.getId());
	}

	public void encodeZertifikatPatientData(ZertifikatPatientData c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATPATIENTDATA;

		encodeString((String) c.getName());

		encode(c.getPubkey());

		encodeInt((int) c.getVersichertennr());

		encodeInt((int) c.getKrankenkassenID());

		encodeInt((int) c.getId());
	}

	public ZertifikatPatientData decodeZertifikatPatientData(byte[] in)
			throws Exception {
		ZertifikatPatientData c = new ZertifikatPatientData();
		encoding_length++;
		c.setName(decodeString(in));
		c.setPubkey(((PublicKey) decode(in)));
		c.setVersichertennr(decodeInt(in));
		c.setKrankenkassenID(decodeInt(in));
		c.setId(decodeInt(in));
		return c;
	}

	private void compSizeZertifikatPatientData(ZertifikatPatientData c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());
		compSize(c.getPubkey());

		compSizeInt(c.getVersichertennr());

		compSizeInt(c.getKrankenkassenID());

		compSizeInt(c.getId());
	}

	public void encodeSymmKey(SymmKey c) throws Exception {
		encodeDestination[encoding_length++] = Code.SYMMKEY;

		encodeByteArray((byte[]) c.getKey());
	}

	public SymmKey decodeSymmKey(byte[] in) throws Exception {
		SymmKey c = new SymmKey();
		encoding_length++;
		c.setKey(decodeByteArray(in));
		return c;
	}

	private void compSizeSymmKey(SymmKey c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getKey());
	}

	public void encodeZertifikatKioskData(ZertifikatKioskData c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATKIOSKDATA;

		encodeString((String) c.getName());

		encode(c.getPubkey());

		encodeInt((int) c.getId());
	}

	public ZertifikatKioskData decodeZertifikatKioskData(byte[] in)
			throws Exception {
		ZertifikatKioskData c = new ZertifikatKioskData();
		encoding_length++;
		c.setName(decodeString(in));
		c.setPubkey(((PublicKey) decode(in)));
		c.setId(decodeInt(in));
		return c;
	}

	private void compSizeZertifikatKioskData(ZertifikatKioskData c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());
		compSize(c.getPubkey());

		compSizeInt(c.getId());
	}

	private void encodePublicKey(PublicKey c) throws Exception {
		encodeDestination[encoding_length++] = Code.PUBLICKEY;
		encodeByteArray(c.getModulus());

	}

	public PublicKey decodePublicKey(byte[] in) throws Exception {
		if (in[encoding_length++] != Code.PUBLICKEY)
			stop();
		PublicKey res = new PublicKey();
		res.setKey(decodeByteArray(in));
		return res;
	}

	private void compSizePublicKey(PublicKey c) throws Exception {
		encoding_length++;
		compSizeByteArray(c.getModulus());

	}

	public void encodeZertifikatTrustedThirdParty(ZertifikatTrustedThirdParty c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATTRUSTEDTHIRDPARTY;

		encode(c.getSignedCert());

		encode(c.getData());
	}

	public ZertifikatTrustedThirdParty decodeZertifikatTrustedThirdParty(
			byte[] in) throws Exception {
		ZertifikatTrustedThirdParty c = new ZertifikatTrustedThirdParty();
		encoding_length++;
		c.setSignedCert(((SignedData) decode(in)));
		c.setData(((ZertifikatTTPData) decode(in)));
		return c;
	}

	private void compSizeZertifikatTrustedThirdParty(
			ZertifikatTrustedThirdParty c) throws Exception {
		encoding_length++;
		compSize(c.getSignedCert());
		compSize(c.getData());
	}

	public void encodePrivateKey(PrivateKey c) throws Exception {
		encodeDestination[encoding_length++] = Code.PRIVATEKEY;

		encodeByteArray(c.getModulus());
		encodeByteArray(c.getPrivexponent());

	}

	public PrivateKey decodePrivateKey(byte[] in) throws Exception {
		if (in[encoding_length++] != Code.PRIVATEKEY)
			stop();
		PrivateKey res = new PrivateKey();
		res.setKey(decodeByteArray(in), decodeByteArray(in));
		return res;
	}

	private void compSizePrivateKey(PrivateKey c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getModulus());
		compSizeByteArray(c.getPrivexponent());

	}

	public void encodeZertifikatTTPData(ZertifikatTTPData c) throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATTTPDATA;

		encodeString((String) c.getName());

		encode(c.getPubkey());
	}

	public ZertifikatTTPData decodeZertifikatTTPData(byte[] in)
			throws Exception {
		ZertifikatTTPData c = new ZertifikatTTPData();
		encoding_length++;
		c.setName(decodeString(in));
		c.setPubkey(((PublicKey) decode(in)));
		return c;
	}

	private void compSizeZertifikatTTPData(ZertifikatTTPData c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());
		compSize(c.getPubkey());
	}

	private void stop() throws TerminalException {
		throw new TerminalException("Coding");
	}
}
