package generated.eGKmitServices.javacard.heilberufsausweisApotheker;

import javacard.framework.ISO7816;
import javacard.framework.ISOException;
import javacard.framework.Util;
/**
 * Class for T(L)V encoding of messages and other classes
 * 
 */
public class Coding {

	/**
	 * offset into encodeDestination for writing/reading
	 */
	private short encoding_length;

	/**
	 * array used for encoding/decoding
	 */
	private byte[] encodeDestination;

	/**
	 * Singleton instance
	 */
	private static Coding the_instance;

	/**
	 * Singleton accessor
	 * 
	 * @return Singleton instance
	 */

	public static Coding getInstance() {
		if (the_instance == null)
			the_instance = new Coding();
		return the_instance;
	}
	/**
	 * return offset into encodeDestination
	 * 
	 * @return value of encoding_length
	 */
	public short getEncodingLength() {
		return encoding_length;
	}

	/**
	 * initializes encoding. This method must be called before encoding an object.
	 * The offset (encoding_length) is reset to 0 and encodeDestination is set to dest.
	 * 
	 * @param dest The (empty) destination array for encoding
	 */
	public void encodeInit(byte[] dest) {
		encodeDestination = dest;
		encoding_length = 0;
	}
	/**
	 * initializes decoding. The offset (encoding_length) is reset to 0.
	 */
	public void decodeInit() {
		encoding_length = 0;
	}

	/**
	 * initializes encoding and encodes an object.
	 * 
	 * @param o The object to encode
	 * @param dest destination array for the encoding
	 * @return length of the encoded object
	 */

	private byte decodeByte(byte[] in) {
		if (in[encoding_length++] != Code.BYTE) {

			SimpleComm.raiseError();
		}

		byte res = in[encoding_length++];
		return res;
	}

	private boolean decodeBoolean(byte[] in) {
		if (in[encoding_length++] != Code.BOOLEAN) {

			SimpleComm.raiseError();

		}
		boolean res = (in[encoding_length++] == 1 ? true : false);
		return res;
	}

	public short encode(Codeable o, byte[] dest) {
		encodeInit(dest);

		try {
			encode(o);
		}

		catch (ArrayIndexOutOfBoundsException e) {
			SimpleComm.raiseError();
		}
		return encoding_length;

	}

	/**
	 * computes the encoding length of an object without actually encoding it.
	 * 
	 * @param o The object of which the encoding length should be computed
	 * @return encoding length
	 */

	public short computeSize(Codeable o) {
		encoding_length = 0;
		try {
			compSize(o);
			return encoding_length;
		} catch (Exception e) {
			return -1;
		}
	}

	private void encodeShort(short o) {
		encodeDestination[encoding_length++] = Code.SHORT;
		Util.setShort(encodeDestination, encoding_length, o);
		encoding_length += 2;
	}

	public void encodeByte(byte o) {
		encodeDestination[encoding_length++] = Code.BYTE;
		encodeDestination[encoding_length++] = o;
	}

	private void encodeBoolean(boolean o) {
		encodeDestination[encoding_length++] = Code.BOOLEAN;
		encodeDestination[encoding_length++] = (byte) (o ? 1 : 0);
	}

	private void compSizeShort(short o) {
		encoding_length += 3;
	}

	private void compSizeByte(byte o) {
		encoding_length += 2;
	}

	private void compSizeBoolean(boolean o) {
		encoding_length += 2;
	}

	private short decodeShort(byte[] in) {
		if (in[encoding_length++] != Code.SHORT)
			SimpleComm.raiseError();
		short res = Util.getShort(in, encoding_length);
		encoding_length += 2;
		return res;
	}

	public void encodeByteArray(byte[] o) {

		encodeDestination[encoding_length++] = Code.BYTEARRAY;

		Util.setShort(encodeDestination, encoding_length, (short) (o.length));
		encoding_length += 2;

		Util.arrayCopy(o, (short) 0, encodeDestination, encoding_length,
				(short) o.length);

		encoding_length += o.length;
	}

	private void compSizeByteArray(byte[] o) {
		encoding_length += 3;
		encoding_length += o.length;
	}

	private void decodeByteArrayInto(byte[] in, byte[] into) {
		if (in[encoding_length++] != Code.BYTEARRAY)
			SimpleComm.raiseError();
		short len = Util.getShort(in, encoding_length);
		encoding_length += 2;
		Util.arrayCopy(in, encoding_length, into, (short) 0, len);
		encoding_length += len;
	}

	/**
	 * encodes an object without initializing destination and offset.
	 * 
	 * @param c object to encode
	 */

	public void encode(Codeable c) {

		switch (c.getCode()) {
			case Code.AUTHENTISIEREHBA :
				encodeAuthentisiereHBA((AuthentisiereHBA) c);
				break;
			case Code.AUTHENTISIEREEGK :
				encodeAuthentisiereEGK((AuthentisiereEGK) c);
				break;
			case Code.RETAUTHENTISIEREHBA :
				encodeRetAuthentisiereHBA((RetAuthentisiereHBA) c);
				break;
			case Code.RETAUTHENTISIEREEGK :
				encodeRetAuthentisiereEGK((RetAuthentisiereEGK) c);
				break;
			case Code.SENDECHALLENGEHBA :
				encodeSendeChallengeHBA((SendeChallengeHBA) c);
				break;
			case Code.SENDECHALLENGEEGK :
				encodeSendeChallengeEGK((SendeChallengeEGK) c);
				break;
			case Code.RETSENDECHALLENGEHBA :
				encodeRetSendeChallengeHBA((RetSendeChallengeHBA) c);
				break;
			case Code.RETSENDECHALLENGEEGK :
				encodeRetSendeChallengeEGK((RetSendeChallengeEGK) c);
				break;
			case Code.ERFRAGENONCEEGK :
				encodeErfrageNonceEGK((ErfrageNonceEGK) c);
				break;
			case Code.ERFRAGENONCEHBA :
				encodeErfrageNonceHBA((ErfrageNonceHBA) c);
				break;
			case Code.ERFRAGENONCEEGKPIN :
				encodeErfrageNonceEGKpin((ErfrageNonceEGKpin) c);
				break;
			case Code.SENDENONCEEGKPIN :
				encodeSendeNonceEGKpin((SendeNonceEGKpin) c);
				break;
			case Code.SENDEPINEGK :
				encodeSendePINEGK((SendePINEGK) c);
				break;
			case Code.BESTAETIGEPINEGK :
				encodeBestaetigePINEGK((BestaetigePINEGK) c);
				break;
			case Code.ERFRAGENONCEHBAPIN :
				encodeErfrageNonceHBApin((ErfrageNonceHBApin) c);
				break;
			case Code.SENDENONCEHBAPIN :
				encodeSendeNonceHBApin((SendeNonceHBApin) c);
				break;
			case Code.SENDEPINHBA :
				encodeSendePINHBA((SendePINHBA) c);
				break;
			case Code.BESTAETIGEPINHBA :
				encodeBestaetigePINHBA((BestaetigePINHBA) c);
				break;
			case Code.SENDENONCEREZEPT :
				encodeSendeNonceRezept((SendeNonceRezept) c);
				break;
			case Code.SCHICKEREZEPTDATEN :
				encodeSchickeRezeptdaten((SchickeRezeptdaten) c);
				break;
			case Code.SENDEREZEPTDATENSIGNATUR :
				encodeSendeRezeptdatensignatur((SendeRezeptdatensignatur) c);
				break;
			case Code.SENDEREZEPT :
				encodeSendeRezept((SendeRezept) c);
				break;
			case Code.ERFRAGENONCEREZEPT :
				encodeErfrageNonceRezept((ErfrageNonceRezept) c);
				break;
			case Code.BESTAETIGEREZEPTAUSSTELLEN :
				encodeBestaetigeRezeptAusstellen((BestaetigeRezeptAusstellen) c);
				break;
			case Code.ENDAUTH :
				encodeEndAuth((EndAuth) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTHBA :
				encodeSendeNonceVerschluesseltHBA((SendeNonceVerschluesseltHBA) c);
				break;
			case Code.SENDESCHLUESSEL :
				encodeSendeSchluessel((SendeSchluessel) c);
				break;
			case Code.ERFRAGENONCEEGKWEITER :
				encodeErfrageNonceEGKWeiter((ErfrageNonceEGKWeiter) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTEGK :
				encodeSendeNonceVerschluesseltEGK((SendeNonceVerschluesseltEGK) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTEGKWEITER :
				encodeSendeNonceVerschluesseltEGKWeiter((SendeNonceVerschluesseltEGKWeiter) c);
				break;
			case Code.SENDESESSIONKEY :
				encodeSendeSessionkey((SendeSessionkey) c);
				break;
			case Code.BESTAETIGESCHLUESSELAUSTAUSCH :
				encodeBestaetigeSchluesselaustausch((BestaetigeSchluesselaustausch) c);
				break;
			case Code.SENDECHALLENGEHBAWEITER :
				encodeSendeChallengeHBAWeiter((SendeChallengeHBAWeiter) c);
				break;
			case Code.SENDECHALLENGEEGKWEITER :
				encodeSendeChallengeEGKWeiter((SendeChallengeEGKWeiter) c);
				break;
			case Code.RETAUTHENTISIEREEGKWEITER :
				encodeRetAuthentisiereEGKWeiter((RetAuthentisiereEGKWeiter) c);
				break;
			case Code.RETSENDECHALLENGEEGKWEITER :
				encodeRetSendeChallengeEGKWeiter((RetSendeChallengeEGKWeiter) c);
				break;
			case Code.SENDEZERTIFIKATHBA :
				encodeSendeZertifikatHBA((SendeZertifikatHBA) c);
				break;
			case Code.SENDESESSIONKEYWEITER :
				encodeSendeSessionkeyWeiter((SendeSessionkeyWeiter) c);
				break;
			case Code.ERFRAGENONCEFUEREINLOESEN :
				encodeErfrageNonceFuerEinloesen((ErfrageNonceFuerEinloesen) c);
				break;
			case Code.SENDENONCEFUEREINLOESEN :
				encodeSendeNonceFuerEinloesen((SendeNonceFuerEinloesen) c);
				break;
			case Code.EINLOESEN :
				encodeEinloesen((Einloesen) c);
				break;
			case Code.SENDEREZEPTZUMEINLOESEN :
				encodeSendeRezeptZumEinloesen((SendeRezeptZumEinloesen) c);
				break;
			case Code.ERFRAGENONCENOTFALLDATENERSTELLEN :
				encodeErfrageNonceNotfalldatenErstellen((ErfrageNonceNotfalldatenErstellen) c);
				break;
			case Code.SENDENONCENOTFALLDATENERSTELLEN :
				encodeSendeNonceNotfalldatenErstellen((SendeNonceNotfalldatenErstellen) c);
				break;
			case Code.SENDENOTFALLDATEN :
				encodeSendeNotfalldaten((SendeNotfalldaten) c);
				break;
			case Code.SENDENOTFALLDATENSIGNATUR :
				encodeSendeNotfalldatensignatur((SendeNotfalldatensignatur) c);
				break;
			case Code.SENDENOTFALLDATENUNDSIG :
				encodeSendeNotfalldatenUndSig((SendeNotfalldatenUndSig) c);
				break;
			case Code.BESTAETIGENOTFALLDATENERSTELLEN :
				encodeBestaetigeNotfalldatenErstellen((BestaetigeNotfalldatenErstellen) c);
				break;
			case Code.PINARZTEINGEGEBEN :
				encodePINArztEingegeben((PINArztEingegeben) c);
				break;
			case Code.PINOK :
				encodePINOK((PINOK) c);
				break;
			case Code.ERFRAGENOTFALLDATEN :
				encodeErfrageNotfalldaten((ErfrageNotfalldaten) c);
				break;
			case Code.SCHICKENOTFALLDATEN :
				encodeSchickeNotfalldaten((SchickeNotfalldaten) c);
				break;
			case Code.SENDEZERTKIOSK :
				encodeSendeZertKiosk((SendeZertKiosk) c);
				break;
			case Code.SENDEZERTEGK :
				encodeSendeZertEGK((SendeZertEGK) c);
				break;
			case Code.SENDENONCES :
				encodeSendeNonces((SendeNonces) c);
				break;
			case Code.BESTAETIGENONCE :
				encodeBestaetigeNonce((BestaetigeNonce) c);
				break;
			case Code.ERFRAGENONCEPIN :
				encodeErfrageNoncePIN((ErfrageNoncePIN) c);
				break;
			case Code.SENDENONCEPIN :
				encodeSendeNoncePIN((SendeNoncePIN) c);
				break;
			case Code.SENDEPIN :
				encodeSendePIN((SendePIN) c);
				break;
			case Code.BESTAETIGEPIN :
				encodeBestaetigePIN((BestaetigePIN) c);
				break;
			case Code.ERFRAGENONCESCHLUESSELAUSTAUSCH :
				encodeErfrageNonceSchluesselaustausch((ErfrageNonceSchluesselaustausch) c);
				break;
			case Code.SENDENONCESCHLUESSELAUSTAUSCH :
				encodeSendeNonceSchluesselaustausch((SendeNonceSchluesselaustausch) c);
				break;
			case Code.SENDESESSIONKEYKIOSK :
				encodeSendeSessionkeyKiosk((SendeSessionkeyKiosk) c);
				break;
			case Code.SCHLUESSELKIOSKOK :
				encodeSchluesselKioskOK((SchluesselKioskOK) c);
				break;
			case Code.LESESTAMMUNDNOTFALLDATEN :
				encodeLeseStammUndNotfalldaten((LeseStammUndNotfalldaten) c);
				break;
			case Code.SENDESTAMMUNDNOTFALLDATEN :
				encodeSendeStammUndNotfalldaten((SendeStammUndNotfalldaten) c);
				break;
			case Code.LESEREZEPT :
				encodeLeseRezept((LeseRezept) c);
				break;
			case Code.SENDEREZEPTDATEN :
				encodeSendeRezeptdaten((SendeRezeptdaten) c);
				break;
			case Code.ERFRAGENONCENOTFALLDATENLOESCHEN :
				encodeErfrageNonceNotfalldatenLoeschen((ErfrageNonceNotfalldatenLoeschen) c);
				break;
			case Code.LOESCHENOTFALLDATEN :
				encodeLoescheNotfalldaten((LoescheNotfalldaten) c);
				break;
			case Code.BESTAETIGELOESCHENNOTFALLDATEN :
				encodeBestaetigeLoeschenNotfalldaten((BestaetigeLoeschenNotfalldaten) c);
				break;
			case Code.SENDENONCE :
				encodeSendeNonce((SendeNonce) c);
				break;
			case Code.GETDATA :
				encodeGetData((GetData) c);
				break;
			case Code.OK :
				encodeOK((OK) c);
				break;
			case Code.CARDDATA :
				encodeCardData((CardData) c);
				break;
			case Code.UPDATE :
				encodeUpdate((Update) c);
				break;
			case Code.NONCE :
				encodeNonce((Nonce) c);
				break;
			case Code.ZERTIFIKATLEISTUNGSERBRINGER :
				encodeZertifikatLeistungserbringer((ZertifikatLeistungserbringer) c);
				break;
			case Code.ZERTIFIKATPATIENT :
				encodeZertifikatPatient((ZertifikatPatient) c);
				break;
			case Code.ENCDATAASYMM :
				encodeEncDataAsymm((EncDataAsymm) c);
				break;
			case Code.NONCEDATA :
				encodeNonceData((NonceData) c);
				break;
			case Code.PINDATA :
				encodePINData((PINData) c);
				break;
			case Code.ENCDATASYMM :
				encodeEncDataSymm((EncDataSymm) c);
				break;
			case Code.EGKDATEN :
				encodeEGKDaten((EGKDaten) c);
				break;
			case Code.EREZEPTDATEN :
				encodeERezeptdaten((ERezeptdaten) c);
				break;
			case Code.SIGNEDDATA :
				encodeSignedData((SignedData) c);
				break;
			case Code.SECRETNONCEDATA :
				encodeSecretNonceData((SecretNonceData) c);
				break;
			case Code.KEYDATA :
				encodeKeyData((KeyData) c);
				break;
			case Code.INDEXNONCE :
				encodeIndexNonce((IndexNonce) c);
				break;
			case Code.REZEPTDATENUNDNONCE :
				encodeRezeptdatenUndNonce((RezeptdatenUndNonce) c);
				break;
			case Code.NOTFALLDATEN :
				encodeNotfalldaten((Notfalldaten) c);
				break;
			case Code.NOTFALLDATENNONCE :
				encodeNotfalldatenNonce((NotfalldatenNonce) c);
				break;
			case Code.ZERTIFIKATKIOSK :
				encodeZertifikatKiosk((ZertifikatKiosk) c);
				break;
			case Code.STAMMUNDNOTFALLDATEN :
				encodeStammUndNotfalldaten((StammUndNotfalldaten) c);
				break;
			case Code.NONCEDATAANDINDEX :
				encodeNonceDataAndIndex((NonceDataAndIndex) c);
				break;
			case Code.NONCEUNDREZEPT :
				encodeNonceUndRezept((NonceUndRezept) c);
				break;
			case Code.NONCESTRINGDATA :
				encodeNonceStringData((NonceStringData) c);
				break;
			case Code.VERSICHERTENSTAMMDATEN :
				encodeVersichertenstammdaten((Versichertenstammdaten) c);
				break;
			case Code.UPDATEAUTH :
				encodeUpdateAuth((UpdateAuth) c);
				break;
			case Code.LEISTUNGSERBRINGERINFO :
				encodeLeistungserbringerInfo((LeistungserbringerInfo) c);
				break;
			case Code.ZERTIFIKATPATIENTDATA :
				encodeZertifikatPatientData((ZertifikatPatientData) c);
				break;
			case Code.SECRET :
				encodeSecret((Secret) c);
				break;
			case Code.ARZNEIMITTEL :
				encodeArzneimittel((Arzneimittel) c);
				break;
			case Code.SYMMKEY :
				encodeSymmKey((SymmKey) c);
				break;
			case Code.ZERTIFIKATKIOSKDATA :
				encodeZertifikatKioskData((ZertifikatKioskData) c);
				break;
			case Code.PUBLICKEY :
				encodePublicKey((PublicKey) c);
				break;
			case Code.ZERTIFIKATTRUSTEDTHIRDPARTY :
				encodeZertifikatTrustedThirdParty((ZertifikatTrustedThirdParty) c);
				break;
			case Code.PRIVATEKEY :
				encodePrivateKey((PrivateKey) c);
				break;
			case Code.ZERTIFIKATTTPDATA :
				encodeZertifikatTTPData((ZertifikatTTPData) c);
				break;
			// never reached
			default :

				SimpleComm.raiseError();

		}
	}

	public Message decodeMessage(byte[] in, short offset, short expectedLength) {
		encoding_length = offset;
		Message m = null;
		try {
			switch (in[encoding_length]) {
				case Code.AUTHENTISIEREHBA :
					m = Store.newAuthentisiereHBA();
					decodeAuthentisiereHBAInto(in, (AuthentisiereHBA) m);
					break;
				case Code.RETAUTHENTISIEREHBA :
					m = Store.newRetAuthentisiereHBA();
					decodeRetAuthentisiereHBAInto(in, (RetAuthentisiereHBA) m);
					break;
				case Code.SENDECHALLENGEEGK :
					m = Store.newSendeChallengeEGK();
					decodeSendeChallengeEGKInto(in, (SendeChallengeEGK) m);
					break;
				case Code.RETSENDECHALLENGEHBA :
					m = Store.newRetSendeChallengeHBA();
					decodeRetSendeChallengeHBAInto(in, (RetSendeChallengeHBA) m);
					break;
				case Code.ERFRAGENONCEEGK :
					m = Store.newErfrageNonceEGK();
					decodeErfrageNonceEGKInto(in, (ErfrageNonceEGK) m);
					break;
				case Code.ERFRAGENONCEHBA :
					m = Store.newErfrageNonceHBA();
					decodeErfrageNonceHBAInto(in, (ErfrageNonceHBA) m);
					break;
				case Code.ERFRAGENONCEHBAPIN :
					m = Store.newErfrageNonceHBApin();
					decodeErfrageNonceHBApinInto(in, (ErfrageNonceHBApin) m);
					break;
				case Code.SENDENONCEHBAPIN :
					m = Store.newSendeNonceHBApin();
					decodeSendeNonceHBApinInto(in, (SendeNonceHBApin) m);
					break;
				case Code.SENDEPINHBA :
					m = Store.newSendePINHBA();
					decodeSendePINHBAInto(in, (SendePINHBA) m);
					break;
				case Code.BESTAETIGEPINHBA :
					m = Store.newBestaetigePINHBA();
					decodeBestaetigePINHBAInto(in, (BestaetigePINHBA) m);
					break;
				case Code.ENDAUTH :
					m = Store.newEndAuth();
					decodeEndAuthInto(in, (EndAuth) m);
					break;
				case Code.SENDENONCEVERSCHLUESSELTHBA :
					m = Store.newSendeNonceVerschluesseltHBA();
					decodeSendeNonceVerschluesseltHBAInto(in,
							(SendeNonceVerschluesseltHBA) m);
					break;
				case Code.SENDESCHLUESSEL :
					m = Store.newSendeSchluessel();
					decodeSendeSchluesselInto(in, (SendeSchluessel) m);
					break;
				case Code.SENDENONCEVERSCHLUESSELTEGKWEITER :
					m = Store.newSendeNonceVerschluesseltEGKWeiter();
					decodeSendeNonceVerschluesseltEGKWeiterInto(in,
							(SendeNonceVerschluesseltEGKWeiter) m);
					break;
				case Code.SENDESESSIONKEY :
					m = Store.newSendeSessionkey();
					decodeSendeSessionkeyInto(in, (SendeSessionkey) m);
					break;
				case Code.SENDECHALLENGEHBAWEITER :
					m = Store.newSendeChallengeHBAWeiter();
					decodeSendeChallengeHBAWeiterInto(in,
							(SendeChallengeHBAWeiter) m);
					break;
				case Code.RETAUTHENTISIEREEGKWEITER :
					m = Store.newRetAuthentisiereEGKWeiter();
					decodeRetAuthentisiereEGKWeiterInto(in,
							(RetAuthentisiereEGKWeiter) m);
					break;
				case Code.RETSENDECHALLENGEEGKWEITER :
					m = Store.newRetSendeChallengeEGKWeiter();
					decodeRetSendeChallengeEGKWeiterInto(in,
							(RetSendeChallengeEGKWeiter) m);
					break;
			}
		} catch (ArrayIndexOutOfBoundsException e) {
			SimpleComm.raiseError();
		}
		if (m == null || encoding_length != (short) (expectedLength + offset))
			SimpleComm.raiseError();
		return m;
	}

	public void compSize(Codeable c) {
		switch (c.getCode()) {
			case Code.AUTHENTISIEREHBA :

				compSizeAuthentisiereHBA((AuthentisiereHBA) c);
				break;
			case Code.AUTHENTISIEREEGK :

				compSizeAuthentisiereEGK((AuthentisiereEGK) c);
				break;
			case Code.RETAUTHENTISIEREHBA :

				compSizeRetAuthentisiereHBA((RetAuthentisiereHBA) c);
				break;
			case Code.RETAUTHENTISIEREEGK :

				compSizeRetAuthentisiereEGK((RetAuthentisiereEGK) c);
				break;
			case Code.SENDECHALLENGEHBA :

				compSizeSendeChallengeHBA((SendeChallengeHBA) c);
				break;
			case Code.SENDECHALLENGEEGK :

				compSizeSendeChallengeEGK((SendeChallengeEGK) c);
				break;
			case Code.RETSENDECHALLENGEHBA :

				compSizeRetSendeChallengeHBA((RetSendeChallengeHBA) c);
				break;
			case Code.RETSENDECHALLENGEEGK :

				compSizeRetSendeChallengeEGK((RetSendeChallengeEGK) c);
				break;
			case Code.ERFRAGENONCEEGK :

				compSizeErfrageNonceEGK((ErfrageNonceEGK) c);
				break;
			case Code.ERFRAGENONCEHBA :

				compSizeErfrageNonceHBA((ErfrageNonceHBA) c);
				break;
			case Code.ERFRAGENONCEEGKPIN :

				compSizeErfrageNonceEGKpin((ErfrageNonceEGKpin) c);
				break;
			case Code.SENDENONCEEGKPIN :

				compSizeSendeNonceEGKpin((SendeNonceEGKpin) c);
				break;
			case Code.SENDEPINEGK :

				compSizeSendePINEGK((SendePINEGK) c);
				break;
			case Code.BESTAETIGEPINEGK :

				compSizeBestaetigePINEGK((BestaetigePINEGK) c);
				break;
			case Code.ERFRAGENONCEHBAPIN :

				compSizeErfrageNonceHBApin((ErfrageNonceHBApin) c);
				break;
			case Code.SENDENONCEHBAPIN :

				compSizeSendeNonceHBApin((SendeNonceHBApin) c);
				break;
			case Code.SENDEPINHBA :

				compSizeSendePINHBA((SendePINHBA) c);
				break;
			case Code.BESTAETIGEPINHBA :

				compSizeBestaetigePINHBA((BestaetigePINHBA) c);
				break;
			case Code.SENDENONCEREZEPT :

				compSizeSendeNonceRezept((SendeNonceRezept) c);
				break;
			case Code.SCHICKEREZEPTDATEN :

				compSizeSchickeRezeptdaten((SchickeRezeptdaten) c);
				break;
			case Code.SENDEREZEPTDATENSIGNATUR :

				compSizeSendeRezeptdatensignatur((SendeRezeptdatensignatur) c);
				break;
			case Code.SENDEREZEPT :

				compSizeSendeRezept((SendeRezept) c);
				break;
			case Code.ERFRAGENONCEREZEPT :

				compSizeErfrageNonceRezept((ErfrageNonceRezept) c);
				break;
			case Code.BESTAETIGEREZEPTAUSSTELLEN :

				compSizeBestaetigeRezeptAusstellen((BestaetigeRezeptAusstellen) c);
				break;
			case Code.ENDAUTH :

				compSizeEndAuth((EndAuth) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTHBA :

				compSizeSendeNonceVerschluesseltHBA((SendeNonceVerschluesseltHBA) c);
				break;
			case Code.SENDESCHLUESSEL :

				compSizeSendeSchluessel((SendeSchluessel) c);
				break;
			case Code.ERFRAGENONCEEGKWEITER :

				compSizeErfrageNonceEGKWeiter((ErfrageNonceEGKWeiter) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTEGK :

				compSizeSendeNonceVerschluesseltEGK((SendeNonceVerschluesseltEGK) c);
				break;
			case Code.SENDENONCEVERSCHLUESSELTEGKWEITER :

				compSizeSendeNonceVerschluesseltEGKWeiter((SendeNonceVerschluesseltEGKWeiter) c);
				break;
			case Code.SENDESESSIONKEY :

				compSizeSendeSessionkey((SendeSessionkey) c);
				break;
			case Code.BESTAETIGESCHLUESSELAUSTAUSCH :

				compSizeBestaetigeSchluesselaustausch((BestaetigeSchluesselaustausch) c);
				break;
			case Code.SENDECHALLENGEHBAWEITER :

				compSizeSendeChallengeHBAWeiter((SendeChallengeHBAWeiter) c);
				break;
			case Code.SENDECHALLENGEEGKWEITER :

				compSizeSendeChallengeEGKWeiter((SendeChallengeEGKWeiter) c);
				break;
			case Code.RETAUTHENTISIEREEGKWEITER :

				compSizeRetAuthentisiereEGKWeiter((RetAuthentisiereEGKWeiter) c);
				break;
			case Code.RETSENDECHALLENGEEGKWEITER :

				compSizeRetSendeChallengeEGKWeiter((RetSendeChallengeEGKWeiter) c);
				break;
			case Code.SENDEZERTIFIKATHBA :

				compSizeSendeZertifikatHBA((SendeZertifikatHBA) c);
				break;
			case Code.SENDESESSIONKEYWEITER :

				compSizeSendeSessionkeyWeiter((SendeSessionkeyWeiter) c);
				break;
			case Code.ERFRAGENONCEFUEREINLOESEN :

				compSizeErfrageNonceFuerEinloesen((ErfrageNonceFuerEinloesen) c);
				break;
			case Code.SENDENONCEFUEREINLOESEN :

				compSizeSendeNonceFuerEinloesen((SendeNonceFuerEinloesen) c);
				break;
			case Code.EINLOESEN :

				compSizeEinloesen((Einloesen) c);
				break;
			case Code.SENDEREZEPTZUMEINLOESEN :

				compSizeSendeRezeptZumEinloesen((SendeRezeptZumEinloesen) c);
				break;
			case Code.ERFRAGENONCENOTFALLDATENERSTELLEN :

				compSizeErfrageNonceNotfalldatenErstellen((ErfrageNonceNotfalldatenErstellen) c);
				break;
			case Code.SENDENONCENOTFALLDATENERSTELLEN :

				compSizeSendeNonceNotfalldatenErstellen((SendeNonceNotfalldatenErstellen) c);
				break;
			case Code.SENDENOTFALLDATEN :

				compSizeSendeNotfalldaten((SendeNotfalldaten) c);
				break;
			case Code.SENDENOTFALLDATENSIGNATUR :

				compSizeSendeNotfalldatensignatur((SendeNotfalldatensignatur) c);
				break;
			case Code.SENDENOTFALLDATENUNDSIG :

				compSizeSendeNotfalldatenUndSig((SendeNotfalldatenUndSig) c);
				break;
			case Code.BESTAETIGENOTFALLDATENERSTELLEN :

				compSizeBestaetigeNotfalldatenErstellen((BestaetigeNotfalldatenErstellen) c);
				break;
			case Code.PINARZTEINGEGEBEN :

				compSizePINArztEingegeben((PINArztEingegeben) c);
				break;
			case Code.PINOK :

				compSizePINOK((PINOK) c);
				break;
			case Code.ERFRAGENOTFALLDATEN :

				compSizeErfrageNotfalldaten((ErfrageNotfalldaten) c);
				break;
			case Code.SCHICKENOTFALLDATEN :

				compSizeSchickeNotfalldaten((SchickeNotfalldaten) c);
				break;
			case Code.SENDEZERTKIOSK :

				compSizeSendeZertKiosk((SendeZertKiosk) c);
				break;
			case Code.SENDEZERTEGK :

				compSizeSendeZertEGK((SendeZertEGK) c);
				break;
			case Code.SENDENONCES :

				compSizeSendeNonces((SendeNonces) c);
				break;
			case Code.BESTAETIGENONCE :

				compSizeBestaetigeNonce((BestaetigeNonce) c);
				break;
			case Code.ERFRAGENONCEPIN :

				compSizeErfrageNoncePIN((ErfrageNoncePIN) c);
				break;
			case Code.SENDENONCEPIN :

				compSizeSendeNoncePIN((SendeNoncePIN) c);
				break;
			case Code.SENDEPIN :

				compSizeSendePIN((SendePIN) c);
				break;
			case Code.BESTAETIGEPIN :

				compSizeBestaetigePIN((BestaetigePIN) c);
				break;
			case Code.ERFRAGENONCESCHLUESSELAUSTAUSCH :

				compSizeErfrageNonceSchluesselaustausch((ErfrageNonceSchluesselaustausch) c);
				break;
			case Code.SENDENONCESCHLUESSELAUSTAUSCH :

				compSizeSendeNonceSchluesselaustausch((SendeNonceSchluesselaustausch) c);
				break;
			case Code.SENDESESSIONKEYKIOSK :

				compSizeSendeSessionkeyKiosk((SendeSessionkeyKiosk) c);
				break;
			case Code.SCHLUESSELKIOSKOK :

				compSizeSchluesselKioskOK((SchluesselKioskOK) c);
				break;
			case Code.LESESTAMMUNDNOTFALLDATEN :

				compSizeLeseStammUndNotfalldaten((LeseStammUndNotfalldaten) c);
				break;
			case Code.SENDESTAMMUNDNOTFALLDATEN :

				compSizeSendeStammUndNotfalldaten((SendeStammUndNotfalldaten) c);
				break;
			case Code.LESEREZEPT :

				compSizeLeseRezept((LeseRezept) c);
				break;
			case Code.SENDEREZEPTDATEN :

				compSizeSendeRezeptdaten((SendeRezeptdaten) c);
				break;
			case Code.ERFRAGENONCENOTFALLDATENLOESCHEN :

				compSizeErfrageNonceNotfalldatenLoeschen((ErfrageNonceNotfalldatenLoeschen) c);
				break;
			case Code.LOESCHENOTFALLDATEN :

				compSizeLoescheNotfalldaten((LoescheNotfalldaten) c);
				break;
			case Code.BESTAETIGELOESCHENNOTFALLDATEN :

				compSizeBestaetigeLoeschenNotfalldaten((BestaetigeLoeschenNotfalldaten) c);
				break;
			case Code.SENDENONCE :

				compSizeSendeNonce((SendeNonce) c);
				break;
			case Code.GETDATA :

				compSizeGetData((GetData) c);
				break;
			case Code.OK :

				compSizeOK((OK) c);
				break;
			case Code.CARDDATA :

				compSizeCardData((CardData) c);
				break;
			case Code.UPDATE :

				compSizeUpdate((Update) c);
				break;
			case Code.NONCE :

				compSizeNonce((Nonce) c);
				break;
			case Code.ZERTIFIKATLEISTUNGSERBRINGER :

				compSizeZertifikatLeistungserbringer((ZertifikatLeistungserbringer) c);
				break;
			case Code.ZERTIFIKATPATIENT :

				compSizeZertifikatPatient((ZertifikatPatient) c);
				break;
			case Code.ENCDATAASYMM :

				compSizeEncDataAsymm((EncDataAsymm) c);
				break;
			case Code.NONCEDATA :

				compSizeNonceData((NonceData) c);
				break;
			case Code.PINDATA :

				compSizePINData((PINData) c);
				break;
			case Code.ENCDATASYMM :

				compSizeEncDataSymm((EncDataSymm) c);
				break;
			case Code.EGKDATEN :

				compSizeEGKDaten((EGKDaten) c);
				break;
			case Code.EREZEPTDATEN :

				compSizeERezeptdaten((ERezeptdaten) c);
				break;
			case Code.SIGNEDDATA :

				compSizeSignedData((SignedData) c);
				break;
			case Code.SECRETNONCEDATA :

				compSizeSecretNonceData((SecretNonceData) c);
				break;
			case Code.KEYDATA :

				compSizeKeyData((KeyData) c);
				break;
			case Code.INDEXNONCE :

				compSizeIndexNonce((IndexNonce) c);
				break;
			case Code.REZEPTDATENUNDNONCE :

				compSizeRezeptdatenUndNonce((RezeptdatenUndNonce) c);
				break;
			case Code.NOTFALLDATEN :

				compSizeNotfalldaten((Notfalldaten) c);
				break;
			case Code.NOTFALLDATENNONCE :

				compSizeNotfalldatenNonce((NotfalldatenNonce) c);
				break;
			case Code.ZERTIFIKATKIOSK :

				compSizeZertifikatKiosk((ZertifikatKiosk) c);
				break;
			case Code.STAMMUNDNOTFALLDATEN :

				compSizeStammUndNotfalldaten((StammUndNotfalldaten) c);
				break;
			case Code.NONCEDATAANDINDEX :

				compSizeNonceDataAndIndex((NonceDataAndIndex) c);
				break;
			case Code.NONCEUNDREZEPT :

				compSizeNonceUndRezept((NonceUndRezept) c);
				break;
			case Code.NONCESTRINGDATA :

				compSizeNonceStringData((NonceStringData) c);
				break;
			case Code.VERSICHERTENSTAMMDATEN :

				compSizeVersichertenstammdaten((Versichertenstammdaten) c);
				break;
			case Code.UPDATEAUTH :

				compSizeUpdateAuth((UpdateAuth) c);
				break;
			case Code.LEISTUNGSERBRINGERINFO :

				compSizeLeistungserbringerInfo((LeistungserbringerInfo) c);
				break;
			case Code.ZERTIFIKATPATIENTDATA :

				compSizeZertifikatPatientData((ZertifikatPatientData) c);
				break;
			case Code.SECRET :

				compSizeSecret((Secret) c);
				break;
			case Code.ARZNEIMITTEL :

				compSizeArzneimittel((Arzneimittel) c);
				break;
			case Code.SYMMKEY :

				compSizeSymmKey((SymmKey) c);
				break;
			case Code.ZERTIFIKATKIOSKDATA :

				compSizeZertifikatKioskData((ZertifikatKioskData) c);
				break;
			case Code.PUBLICKEY :

				compSizePublicKey((PublicKey) c);
				break;
			case Code.ZERTIFIKATTRUSTEDTHIRDPARTY :

				compSizeZertifikatTrustedThirdParty((ZertifikatTrustedThirdParty) c);
				break;
			case Code.PRIVATEKEY :

				compSizePrivateKey((PrivateKey) c);
				break;
			case Code.ZERTIFIKATTTPDATA :

				compSizeZertifikatTTPData((ZertifikatTTPData) c);
				break;
			// never reached
			default :

				SimpleComm.raiseError();
				break;

		}
	}

	public PlainData decodePlainData(byte[] in, byte expectedType) {
		encoding_length = 0;
		PlainData p = null;
		try {
			if (in[encoding_length] != expectedType) {

				SimpleComm.raiseError();
			}
			switch (in[encoding_length]) {

				case Code.NONCEDATA :
					p = Store.newNonceData();
					decodeNonceDataInto(in, (NonceData) p);
					break;
				case Code.PINDATA :
					p = Store.newPINData();
					decodePINDataInto(in, (PINData) p);
					break;
				case Code.EGKDATEN :
					p = Store.newEGKDaten();
					decodeEGKDatenInto(in, (EGKDaten) p);
					break;
				case Code.EREZEPTDATEN :
					p = Store.newERezeptdaten();
					decodeERezeptdatenInto(in, (ERezeptdaten) p);
					break;
				case Code.SECRETNONCEDATA :
					p = Store.newSecretNonceData();
					decodeSecretNonceDataInto(in, (SecretNonceData) p);
					break;
				case Code.KEYDATA :
					p = Store.newKeyData();
					decodeKeyDataInto(in, (KeyData) p);
					break;
				case Code.INDEXNONCE :
					p = Store.newIndexNonce();
					decodeIndexNonceInto(in, (IndexNonce) p);
					break;
				case Code.REZEPTDATENUNDNONCE :
					p = Store.newRezeptdatenUndNonce();
					decodeRezeptdatenUndNonceInto(in, (RezeptdatenUndNonce) p);
					break;
				case Code.NOTFALLDATEN :
					p = Store.newNotfalldaten();
					decodeNotfalldatenInto(in, (Notfalldaten) p);
					break;
				case Code.NOTFALLDATENNONCE :
					p = Store.newNotfalldatenNonce();
					decodeNotfalldatenNonceInto(in, (NotfalldatenNonce) p);
					break;
				case Code.STAMMUNDNOTFALLDATEN :
					p = Store.newStammUndNotfalldaten();
					decodeStammUndNotfalldatenInto(in, (StammUndNotfalldaten) p);
					break;
				case Code.NONCEDATAANDINDEX :
					p = Store.newNonceDataAndIndex();
					decodeNonceDataAndIndexInto(in, (NonceDataAndIndex) p);
					break;
				case Code.NONCEUNDREZEPT :
					p = Store.newNonceUndRezept();
					decodeNonceUndRezeptInto(in, (NonceUndRezept) p);
					break;
				case Code.NONCESTRINGDATA :
					p = Store.newNonceStringData();
					decodeNonceStringDataInto(in, (NonceStringData) p);
					break;
				case Code.VERSICHERTENSTAMMDATEN :
					p = Store.newVersichertenstammdaten();
					decodeVersichertenstammdatenInto(in,
							(Versichertenstammdaten) p);
					break;

			}
		} catch (ArrayIndexOutOfBoundsException e) {

			SimpleComm.raiseError();
		}

		if (p == null) {

			SimpleComm.raiseError();
		}
		return p;
	}

	private void encodeAuthentisiereHBA(AuthentisiereHBA c) {
		encodeDestination[encoding_length++] = Code.AUTHENTISIEREHBA;
	}

	private void decodeAuthentisiereHBAInto(byte[] in, AuthentisiereHBA into) {
		if (in[encoding_length++] != Code.AUTHENTISIEREHBA)
			SimpleComm.raiseError();
	}

	private void compSizeAuthentisiereHBA(AuthentisiereHBA c) {
		encoding_length++;
	}

	private void encodeAuthentisiereEGK(AuthentisiereEGK c) {
		encodeDestination[encoding_length++] = Code.AUTHENTISIEREEGK;
		encode(c.nonce);
	}

	private void decodeAuthentisiereEGKInto(byte[] in, AuthentisiereEGK into) {
		if (in[encoding_length++] != Code.AUTHENTISIEREEGK)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeAuthentisiereEGK(AuthentisiereEGK c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodeRetAuthentisiereHBA(RetAuthentisiereHBA c) {
		encodeDestination[encoding_length++] = Code.RETAUTHENTISIEREHBA;
		encode(c.zertArzt);
	}

	private void decodeRetAuthentisiereHBAInto(byte[] in,
			RetAuthentisiereHBA into) {
		if (in[encoding_length++] != Code.RETAUTHENTISIEREHBA)
			SimpleComm.raiseError();
		decodeZertifikatLeistungserbringerInto(in,
				(ZertifikatLeistungserbringer) (into.zertArzt));
	}

	private void compSizeRetAuthentisiereHBA(RetAuthentisiereHBA c) {
		encoding_length++;
		compSize(c.zertArzt);
	}

	private void encodeRetAuthentisiereEGK(RetAuthentisiereEGK c) {
		encodeDestination[encoding_length++] = Code.RETAUTHENTISIEREEGK;
		encode(c.zertPatient);
	}

	private void decodeRetAuthentisiereEGKInto(byte[] in,
			RetAuthentisiereEGK into) {
		if (in[encoding_length++] != Code.RETAUTHENTISIEREEGK)
			SimpleComm.raiseError();
		decodeZertifikatPatientInto(in, (ZertifikatPatient) (into.zertPatient));
	}

	private void compSizeRetAuthentisiereEGK(RetAuthentisiereEGK c) {
		encoding_length++;
		compSize(c.zertPatient);
	}

	private void encodeSendeChallengeHBA(SendeChallengeHBA c) {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEHBA;
		encode(c.encryptedNonce);
	}

	private void decodeSendeChallengeHBAInto(byte[] in, SendeChallengeHBA into) {
		if (in[encoding_length++] != Code.SENDECHALLENGEHBA)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.encryptedNonce));
	}

	private void compSizeSendeChallengeHBA(SendeChallengeHBA c) {
		encoding_length++;
		compSize(c.encryptedNonce);
	}

	private void encodeSendeChallengeEGK(SendeChallengeEGK c) {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEEGK;
		encode(c.encryptedNonce);
	}

	private void decodeSendeChallengeEGKInto(byte[] in, SendeChallengeEGK into) {
		if (in[encoding_length++] != Code.SENDECHALLENGEEGK)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.encryptedNonce));
	}

	private void compSizeSendeChallengeEGK(SendeChallengeEGK c) {
		encoding_length++;
		compSize(c.encryptedNonce);
	}

	private void encodeRetSendeChallengeHBA(RetSendeChallengeHBA c) {
		encodeDestination[encoding_length++] = Code.RETSENDECHALLENGEHBA;
		encode(c.nonceCard);
		encode(c.nonceTerm);
	}

	private void decodeRetSendeChallengeHBAInto(byte[] in,
			RetSendeChallengeHBA into) {
		if (in[encoding_length++] != Code.RETSENDECHALLENGEHBA)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonceCard));
		decodeNonceInto(in, (Nonce) (into.nonceTerm));
	}

	private void compSizeRetSendeChallengeHBA(RetSendeChallengeHBA c) {
		encoding_length++;
		compSize(c.nonceCard);
		compSize(c.nonceTerm);
	}

	private void encodeRetSendeChallengeEGK(RetSendeChallengeEGK c) {
		encodeDestination[encoding_length++] = Code.RETSENDECHALLENGEEGK;
		encode(c.nonceCard);
		encode(c.nonceTerm);
	}

	private void decodeRetSendeChallengeEGKInto(byte[] in,
			RetSendeChallengeEGK into) {
		if (in[encoding_length++] != Code.RETSENDECHALLENGEEGK)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonceCard));
		decodeNonceInto(in, (Nonce) (into.nonceTerm));
	}

	private void compSizeRetSendeChallengeEGK(RetSendeChallengeEGK c) {
		encoding_length++;
		compSize(c.nonceCard);
		compSize(c.nonceTerm);
	}

	private void encodeErfrageNonceEGK(ErfrageNonceEGK c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEEGK;
	}

	private void decodeErfrageNonceEGKInto(byte[] in, ErfrageNonceEGK into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEEGK)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceEGK(ErfrageNonceEGK c) {
		encoding_length++;
	}

	private void encodeErfrageNonceHBA(ErfrageNonceHBA c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEHBA;
	}

	private void decodeErfrageNonceHBAInto(byte[] in, ErfrageNonceHBA into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEHBA)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceHBA(ErfrageNonceHBA c) {
		encoding_length++;
	}

	private void encodeErfrageNonceEGKpin(ErfrageNonceEGKpin c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEEGKPIN;
	}

	private void decodeErfrageNonceEGKpinInto(byte[] in, ErfrageNonceEGKpin into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEEGKPIN)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceEGKpin(ErfrageNonceEGKpin c) {
		encoding_length++;
	}

	private void encodeSendeNonceEGKpin(SendeNonceEGKpin c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEEGKPIN;
		encode(c.n);
	}

	private void decodeSendeNonceEGKpinInto(byte[] in, SendeNonceEGKpin into) {
		if (in[encoding_length++] != Code.SENDENONCEEGKPIN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.n));
	}

	private void compSizeSendeNonceEGKpin(SendeNonceEGKpin c) {
		encoding_length++;
		compSize(c.n);
	}

	private void encodeSendePINEGK(SendePINEGK c) {
		encodeDestination[encoding_length++] = Code.SENDEPINEGK;
		encode(c.ed);
	}

	private void decodeSendePINEGKInto(byte[] in, SendePINEGK into) {
		if (in[encoding_length++] != Code.SENDEPINEGK)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.ed));
	}

	private void compSizeSendePINEGK(SendePINEGK c) {
		encoding_length++;
		compSize(c.ed);
	}

	private void encodeBestaetigePINEGK(BestaetigePINEGK c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGEPINEGK;
	}

	private void decodeBestaetigePINEGKInto(byte[] in, BestaetigePINEGK into) {
		if (in[encoding_length++] != Code.BESTAETIGEPINEGK)
			SimpleComm.raiseError();
	}

	private void compSizeBestaetigePINEGK(BestaetigePINEGK c) {
		encoding_length++;
	}

	private void encodeErfrageNonceHBApin(ErfrageNonceHBApin c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEHBAPIN;
	}

	private void decodeErfrageNonceHBApinInto(byte[] in, ErfrageNonceHBApin into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEHBAPIN)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceHBApin(ErfrageNonceHBApin c) {
		encoding_length++;
	}

	private void encodeSendeNonceHBApin(SendeNonceHBApin c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEHBAPIN;
		encode(c.n);
	}

	private void decodeSendeNonceHBApinInto(byte[] in, SendeNonceHBApin into) {
		if (in[encoding_length++] != Code.SENDENONCEHBAPIN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.n));
	}

	private void compSizeSendeNonceHBApin(SendeNonceHBApin c) {
		encoding_length++;
		compSize(c.n);
	}

	private void encodeSendePINHBA(SendePINHBA c) {
		encodeDestination[encoding_length++] = Code.SENDEPINHBA;
		encode(c.ed);
	}

	private void decodeSendePINHBAInto(byte[] in, SendePINHBA into) {
		if (in[encoding_length++] != Code.SENDEPINHBA)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.ed));
	}

	private void compSizeSendePINHBA(SendePINHBA c) {
		encoding_length++;
		compSize(c.ed);
	}

	private void encodeBestaetigePINHBA(BestaetigePINHBA c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGEPINHBA;
	}

	private void decodeBestaetigePINHBAInto(byte[] in, BestaetigePINHBA into) {
		if (in[encoding_length++] != Code.BESTAETIGEPINHBA)
			SimpleComm.raiseError();
	}

	private void compSizeBestaetigePINHBA(BestaetigePINHBA c) {
		encoding_length++;
	}

	private void encodeSendeNonceRezept(SendeNonceRezept c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEREZEPT;
		encode(c.ed);
	}

	private void decodeSendeNonceRezeptInto(byte[] in, SendeNonceRezept into) {
		if (in[encoding_length++] != Code.SENDENONCEREZEPT)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.ed));
	}

	private void compSizeSendeNonceRezept(SendeNonceRezept c) {
		encoding_length++;
		compSize(c.ed);
	}

	private void encodeSchickeRezeptdaten(SchickeRezeptdaten c) {
		encodeDestination[encoding_length++] = Code.SCHICKEREZEPTDATEN;
		encode(c.ed);
	}

	private void decodeSchickeRezeptdatenInto(byte[] in, SchickeRezeptdaten into) {
		if (in[encoding_length++] != Code.SCHICKEREZEPTDATEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.ed));
	}

	private void compSizeSchickeRezeptdaten(SchickeRezeptdaten c) {
		encoding_length++;
		compSize(c.ed);
	}

	private void encodeSendeRezeptdatensignatur(SendeRezeptdatensignatur c) {
		encodeDestination[encoding_length++] = Code.SENDEREZEPTDATENSIGNATUR;
		encode(c.sig);
	}

	private void decodeSendeRezeptdatensignaturInto(byte[] in,
			SendeRezeptdatensignatur into) {
		if (in[encoding_length++] != Code.SENDEREZEPTDATENSIGNATUR)
			SimpleComm.raiseError();
		decodeSignedDataInto(in, (SignedData) (into.sig));
	}

	private void compSizeSendeRezeptdatensignatur(SendeRezeptdatensignatur c) {
		encoding_length++;
		compSize(c.sig);
	}

	private void encodeSendeRezept(SendeRezept c) {
		encodeDestination[encoding_length++] = Code.SENDEREZEPT;
		encode(c.ed);
		encode(c.sig);
	}

	private void decodeSendeRezeptInto(byte[] in, SendeRezept into) {
		if (in[encoding_length++] != Code.SENDEREZEPT)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.ed));
		decodeSignedDataInto(in, (SignedData) (into.sig));
	}

	private void compSizeSendeRezept(SendeRezept c) {
		encoding_length++;
		compSize(c.ed);
		compSize(c.sig);
	}

	private void encodeErfrageNonceRezept(ErfrageNonceRezept c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEREZEPT;
	}

	private void decodeErfrageNonceRezeptInto(byte[] in, ErfrageNonceRezept into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEREZEPT)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceRezept(ErfrageNonceRezept c) {
		encoding_length++;
	}

	private void encodeBestaetigeRezeptAusstellen(BestaetigeRezeptAusstellen c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGEREZEPTAUSSTELLEN;
	}

	private void decodeBestaetigeRezeptAusstellenInto(byte[] in,
			BestaetigeRezeptAusstellen into) {
		if (in[encoding_length++] != Code.BESTAETIGEREZEPTAUSSTELLEN)
			SimpleComm.raiseError();
	}

	private void compSizeBestaetigeRezeptAusstellen(BestaetigeRezeptAusstellen c) {
		encoding_length++;
	}

	private void encodeEndAuth(EndAuth c) {
		encodeDestination[encoding_length++] = Code.ENDAUTH;
	}

	private void decodeEndAuthInto(byte[] in, EndAuth into) {
		if (in[encoding_length++] != Code.ENDAUTH)
			SimpleComm.raiseError();
	}

	private void compSizeEndAuth(EndAuth c) {
		encoding_length++;
	}

	private void encodeSendeNonceVerschluesseltHBA(SendeNonceVerschluesseltHBA c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEVERSCHLUESSELTHBA;
		encode(c.enc);
	}

	private void decodeSendeNonceVerschluesseltHBAInto(byte[] in,
			SendeNonceVerschluesseltHBA into) {
		if (in[encoding_length++] != Code.SENDENONCEVERSCHLUESSELTHBA)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.enc));
	}

	private void compSizeSendeNonceVerschluesseltHBA(
			SendeNonceVerschluesseltHBA c) {
		encoding_length++;
		compSize(c.enc);
	}

	private void encodeSendeSchluessel(SendeSchluessel c) {
		encodeDestination[encoding_length++] = Code.SENDESCHLUESSEL;
		encode(c.enc);
	}

	private void decodeSendeSchluesselInto(byte[] in, SendeSchluessel into) {
		if (in[encoding_length++] != Code.SENDESCHLUESSEL)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.enc));
	}

	private void compSizeSendeSchluessel(SendeSchluessel c) {
		encoding_length++;
		compSize(c.enc);
	}

	private void encodeErfrageNonceEGKWeiter(ErfrageNonceEGKWeiter c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEEGKWEITER;
	}

	private void decodeErfrageNonceEGKWeiterInto(byte[] in,
			ErfrageNonceEGKWeiter into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEEGKWEITER)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceEGKWeiter(ErfrageNonceEGKWeiter c) {
		encoding_length++;
	}

	private void encodeSendeNonceVerschluesseltEGK(SendeNonceVerschluesseltEGK c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEVERSCHLUESSELTEGK;
		encode(c.enc);
	}

	private void decodeSendeNonceVerschluesseltEGKInto(byte[] in,
			SendeNonceVerschluesseltEGK into) {
		if (in[encoding_length++] != Code.SENDENONCEVERSCHLUESSELTEGK)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.enc));
	}

	private void compSizeSendeNonceVerschluesseltEGK(
			SendeNonceVerschluesseltEGK c) {
		encoding_length++;
		compSize(c.enc);
	}

	private void encodeSendeNonceVerschluesseltEGKWeiter(
			SendeNonceVerschluesseltEGKWeiter c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEVERSCHLUESSELTEGKWEITER;
		encode(c.enc);
	}

	private void decodeSendeNonceVerschluesseltEGKWeiterInto(byte[] in,
			SendeNonceVerschluesseltEGKWeiter into) {
		if (in[encoding_length++] != Code.SENDENONCEVERSCHLUESSELTEGKWEITER)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.enc));
	}

	private void compSizeSendeNonceVerschluesseltEGKWeiter(
			SendeNonceVerschluesseltEGKWeiter c) {
		encoding_length++;
		compSize(c.enc);
	}

	private void encodeSendeSessionkey(SendeSessionkey c) {
		encodeDestination[encoding_length++] = Code.SENDESESSIONKEY;
		encode(c.enc);
	}

	private void decodeSendeSessionkeyInto(byte[] in, SendeSessionkey into) {
		if (in[encoding_length++] != Code.SENDESESSIONKEY)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.enc));
	}

	private void compSizeSendeSessionkey(SendeSessionkey c) {
		encoding_length++;
		compSize(c.enc);
	}

	private void encodeBestaetigeSchluesselaustausch(
			BestaetigeSchluesselaustausch c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGESCHLUESSELAUSTAUSCH;
	}

	private void decodeBestaetigeSchluesselaustauschInto(byte[] in,
			BestaetigeSchluesselaustausch into) {
		if (in[encoding_length++] != Code.BESTAETIGESCHLUESSELAUSTAUSCH)
			SimpleComm.raiseError();
	}

	private void compSizeBestaetigeSchluesselaustausch(
			BestaetigeSchluesselaustausch c) {
		encoding_length++;
	}

	private void encodeSendeChallengeHBAWeiter(SendeChallengeHBAWeiter c) {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEHBAWEITER;
		encode(c.encCard);
		encode(c.enc);
	}

	private void decodeSendeChallengeHBAWeiterInto(byte[] in,
			SendeChallengeHBAWeiter into) {
		if (in[encoding_length++] != Code.SENDECHALLENGEHBAWEITER)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.encCard));
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.enc));
	}

	private void compSizeSendeChallengeHBAWeiter(SendeChallengeHBAWeiter c) {
		encoding_length++;
		compSize(c.encCard);
		compSize(c.enc);
	}

	private void encodeSendeChallengeEGKWeiter(SendeChallengeEGKWeiter c) {
		encodeDestination[encoding_length++] = Code.SENDECHALLENGEEGKWEITER;
		encode(c.encCard);
		encode(c.enc);
	}

	private void decodeSendeChallengeEGKWeiterInto(byte[] in,
			SendeChallengeEGKWeiter into) {
		if (in[encoding_length++] != Code.SENDECHALLENGEEGKWEITER)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.encCard));
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.enc));
	}

	private void compSizeSendeChallengeEGKWeiter(SendeChallengeEGKWeiter c) {
		encoding_length++;
		compSize(c.encCard);
		compSize(c.enc);
	}

	private void encodeRetAuthentisiereEGKWeiter(RetAuthentisiereEGKWeiter c) {
		encodeDestination[encoding_length++] = Code.RETAUTHENTISIEREEGKWEITER;
		encode(c.zertPatient);
	}

	private void decodeRetAuthentisiereEGKWeiterInto(byte[] in,
			RetAuthentisiereEGKWeiter into) {
		if (in[encoding_length++] != Code.RETAUTHENTISIEREEGKWEITER)
			SimpleComm.raiseError();
		decodeZertifikatPatientInto(in, (ZertifikatPatient) (into.zertPatient));
	}

	private void compSizeRetAuthentisiereEGKWeiter(RetAuthentisiereEGKWeiter c) {
		encoding_length++;
		compSize(c.zertPatient);
	}

	private void encodeRetSendeChallengeEGKWeiter(RetSendeChallengeEGKWeiter c) {
		encodeDestination[encoding_length++] = Code.RETSENDECHALLENGEEGKWEITER;
		encode(c.nonce);
	}

	private void decodeRetSendeChallengeEGKWeiterInto(byte[] in,
			RetSendeChallengeEGKWeiter into) {
		if (in[encoding_length++] != Code.RETSENDECHALLENGEEGKWEITER)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeRetSendeChallengeEGKWeiter(RetSendeChallengeEGKWeiter c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodeSendeZertifikatHBA(SendeZertifikatHBA c) {
		encodeDestination[encoding_length++] = Code.SENDEZERTIFIKATHBA;
		encode(c.zertLeistungserbringer);
	}

	private void decodeSendeZertifikatHBAInto(byte[] in, SendeZertifikatHBA into) {
		if (in[encoding_length++] != Code.SENDEZERTIFIKATHBA)
			SimpleComm.raiseError();
		decodeZertifikatLeistungserbringerInto(in,
				(ZertifikatLeistungserbringer) (into.zertLeistungserbringer));
	}

	private void compSizeSendeZertifikatHBA(SendeZertifikatHBA c) {
		encoding_length++;
		compSize(c.zertLeistungserbringer);
	}

	private void encodeSendeSessionkeyWeiter(SendeSessionkeyWeiter c) {
		encodeDestination[encoding_length++] = Code.SENDESESSIONKEYWEITER;
		encode(c.enc);
	}

	private void decodeSendeSessionkeyWeiterInto(byte[] in,
			SendeSessionkeyWeiter into) {
		if (in[encoding_length++] != Code.SENDESESSIONKEYWEITER)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.enc));
	}

	private void compSizeSendeSessionkeyWeiter(SendeSessionkeyWeiter c) {
		encoding_length++;
		compSize(c.enc);
	}

	private void encodeErfrageNonceFuerEinloesen(ErfrageNonceFuerEinloesen c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEFUEREINLOESEN;
	}

	private void decodeErfrageNonceFuerEinloesenInto(byte[] in,
			ErfrageNonceFuerEinloesen into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEFUEREINLOESEN)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceFuerEinloesen(ErfrageNonceFuerEinloesen c) {
		encoding_length++;
	}

	private void encodeSendeNonceFuerEinloesen(SendeNonceFuerEinloesen c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEFUEREINLOESEN;
		encode(c.nonce);
	}

	private void decodeSendeNonceFuerEinloesenInto(byte[] in,
			SendeNonceFuerEinloesen into) {
		if (in[encoding_length++] != Code.SENDENONCEFUEREINLOESEN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeSendeNonceFuerEinloesen(SendeNonceFuerEinloesen c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodeEinloesen(Einloesen c) {
		encodeDestination[encoding_length++] = Code.EINLOESEN;
		encode(c.in);
	}

	private void decodeEinloesenInto(byte[] in, Einloesen into) {
		if (in[encoding_length++] != Code.EINLOESEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.in));
	}

	private void compSizeEinloesen(Einloesen c) {
		encoding_length++;
		compSize(c.in);
	}

	private void encodeSendeRezeptZumEinloesen(SendeRezeptZumEinloesen c) {
		encodeDestination[encoding_length++] = Code.SENDEREZEPTZUMEINLOESEN;
		encode(c.rezeptdatenUNonce);
		encode(c.sig);
		encode(c.zertArzt);
	}

	private void decodeSendeRezeptZumEinloesenInto(byte[] in,
			SendeRezeptZumEinloesen into) {
		if (in[encoding_length++] != Code.SENDEREZEPTZUMEINLOESEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.rezeptdatenUNonce));
		decodeSignedDataInto(in, (SignedData) (into.sig));
		decodeZertifikatLeistungserbringerInto(in,
				(ZertifikatLeistungserbringer) (into.zertArzt));
	}

	private void compSizeSendeRezeptZumEinloesen(SendeRezeptZumEinloesen c) {
		encoding_length++;
		compSize(c.rezeptdatenUNonce);
		compSize(c.sig);
		compSize(c.zertArzt);
	}

	private void encodeErfrageNonceNotfalldatenErstellen(
			ErfrageNonceNotfalldatenErstellen c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCENOTFALLDATENERSTELLEN;
	}

	private void decodeErfrageNonceNotfalldatenErstellenInto(byte[] in,
			ErfrageNonceNotfalldatenErstellen into) {
		if (in[encoding_length++] != Code.ERFRAGENONCENOTFALLDATENERSTELLEN)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceNotfalldatenErstellen(
			ErfrageNonceNotfalldatenErstellen c) {
		encoding_length++;
	}

	private void encodeSendeNonceNotfalldatenErstellen(
			SendeNonceNotfalldatenErstellen c) {
		encodeDestination[encoding_length++] = Code.SENDENONCENOTFALLDATENERSTELLEN;
		encode(c.noncedata);
	}

	private void decodeSendeNonceNotfalldatenErstellenInto(byte[] in,
			SendeNonceNotfalldatenErstellen into) {
		if (in[encoding_length++] != Code.SENDENONCENOTFALLDATENERSTELLEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.noncedata));
	}

	private void compSizeSendeNonceNotfalldatenErstellen(
			SendeNonceNotfalldatenErstellen c) {
		encoding_length++;
		compSize(c.noncedata);
	}

	private void encodeSendeNotfalldaten(SendeNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.SENDENOTFALLDATEN;
		encode(c.notfalldaten);
	}

	private void decodeSendeNotfalldatenInto(byte[] in, SendeNotfalldaten into) {
		if (in[encoding_length++] != Code.SENDENOTFALLDATEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.notfalldaten));
	}

	private void compSizeSendeNotfalldaten(SendeNotfalldaten c) {
		encoding_length++;
		compSize(c.notfalldaten);
	}

	private void encodeSendeNotfalldatensignatur(SendeNotfalldatensignatur c) {
		encodeDestination[encoding_length++] = Code.SENDENOTFALLDATENSIGNATUR;
		encode(c.sig);
	}

	private void decodeSendeNotfalldatensignaturInto(byte[] in,
			SendeNotfalldatensignatur into) {
		if (in[encoding_length++] != Code.SENDENOTFALLDATENSIGNATUR)
			SimpleComm.raiseError();
		decodeSignedDataInto(in, (SignedData) (into.sig));
	}

	private void compSizeSendeNotfalldatensignatur(SendeNotfalldatensignatur c) {
		encoding_length++;
		compSize(c.sig);
	}

	private void encodeSendeNotfalldatenUndSig(SendeNotfalldatenUndSig c) {
		encodeDestination[encoding_length++] = Code.SENDENOTFALLDATENUNDSIG;
		encode(c.nfd);
		encode(c.sig);
	}

	private void decodeSendeNotfalldatenUndSigInto(byte[] in,
			SendeNotfalldatenUndSig into) {
		if (in[encoding_length++] != Code.SENDENOTFALLDATENUNDSIG)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.nfd));
		decodeSignedDataInto(in, (SignedData) (into.sig));
	}

	private void compSizeSendeNotfalldatenUndSig(SendeNotfalldatenUndSig c) {
		encoding_length++;
		compSize(c.nfd);
		compSize(c.sig);
	}

	private void encodeBestaetigeNotfalldatenErstellen(
			BestaetigeNotfalldatenErstellen c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGENOTFALLDATENERSTELLEN;
	}

	private void decodeBestaetigeNotfalldatenErstellenInto(byte[] in,
			BestaetigeNotfalldatenErstellen into) {
		if (in[encoding_length++] != Code.BESTAETIGENOTFALLDATENERSTELLEN)
			SimpleComm.raiseError();
	}

	private void compSizeBestaetigeNotfalldatenErstellen(
			BestaetigeNotfalldatenErstellen c) {
		encoding_length++;
	}

	private void encodePINArztEingegeben(PINArztEingegeben c) {
		encodeDestination[encoding_length++] = Code.PINARZTEINGEGEBEN;
		encode(c.nonce);
	}

	private void decodePINArztEingegebenInto(byte[] in, PINArztEingegeben into) {
		if (in[encoding_length++] != Code.PINARZTEINGEGEBEN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizePINArztEingegeben(PINArztEingegeben c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodePINOK(PINOK c) {
		encodeDestination[encoding_length++] = Code.PINOK;
		encode(c.noncedata);
	}

	private void decodePINOKInto(byte[] in, PINOK into) {
		if (in[encoding_length++] != Code.PINOK)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.noncedata));
	}

	private void compSizePINOK(PINOK c) {
		encoding_length++;
		compSize(c.noncedata);
	}

	private void encodeErfrageNotfalldaten(ErfrageNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENOTFALLDATEN;
		encode(c.nonce);
	}

	private void decodeErfrageNotfalldatenInto(byte[] in,
			ErfrageNotfalldaten into) {
		if (in[encoding_length++] != Code.ERFRAGENOTFALLDATEN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeErfrageNotfalldaten(ErfrageNotfalldaten c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodeSchickeNotfalldaten(SchickeNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.SCHICKENOTFALLDATEN;
		encode(c.nfd);
	}

	private void decodeSchickeNotfalldatenInto(byte[] in,
			SchickeNotfalldaten into) {
		if (in[encoding_length++] != Code.SCHICKENOTFALLDATEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.nfd));
	}

	private void compSizeSchickeNotfalldaten(SchickeNotfalldaten c) {
		encoding_length++;
		compSize(c.nfd);
	}

	private void encodeSendeZertKiosk(SendeZertKiosk c) {
		encodeDestination[encoding_length++] = Code.SENDEZERTKIOSK;
		encode(c.zertKiosk);
	}

	private void decodeSendeZertKioskInto(byte[] in, SendeZertKiosk into) {
		if (in[encoding_length++] != Code.SENDEZERTKIOSK)
			SimpleComm.raiseError();
		decodeZertifikatKioskInto(in, (ZertifikatKiosk) (into.zertKiosk));
	}

	private void compSizeSendeZertKiosk(SendeZertKiosk c) {
		encoding_length++;
		compSize(c.zertKiosk);
	}

	private void encodeSendeZertEGK(SendeZertEGK c) {
		encodeDestination[encoding_length++] = Code.SENDEZERTEGK;
		encode(c.nd);
		encode(c.z);
	}

	private void decodeSendeZertEGKInto(byte[] in, SendeZertEGK into) {
		if (in[encoding_length++] != Code.SENDEZERTEGK)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.nd));
		decodeZertifikatPatientInto(in, (ZertifikatPatient) (into.z));
	}

	private void compSizeSendeZertEGK(SendeZertEGK c) {
		encoding_length++;
		compSize(c.nd);
		compSize(c.z);
	}

	private void encodeSendeNonces(SendeNonces c) {
		encodeDestination[encoding_length++] = Code.SENDENONCES;
		encode(c.nonce);
		encode(c.nd);
	}

	private void decodeSendeNoncesInto(byte[] in, SendeNonces into) {
		if (in[encoding_length++] != Code.SENDENONCES)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.nd));
	}

	private void compSizeSendeNonces(SendeNonces c) {
		encoding_length++;
		compSize(c.nonce);
		compSize(c.nd);
	}

	private void encodeBestaetigeNonce(BestaetigeNonce c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGENONCE;
		encode(c.n);
	}

	private void decodeBestaetigeNonceInto(byte[] in, BestaetigeNonce into) {
		if (in[encoding_length++] != Code.BESTAETIGENONCE)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.n));
	}

	private void compSizeBestaetigeNonce(BestaetigeNonce c) {
		encoding_length++;
		compSize(c.n);
	}

	private void encodeErfrageNoncePIN(ErfrageNoncePIN c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCEPIN;
	}

	private void decodeErfrageNoncePINInto(byte[] in, ErfrageNoncePIN into) {
		if (in[encoding_length++] != Code.ERFRAGENONCEPIN)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNoncePIN(ErfrageNoncePIN c) {
		encoding_length++;
	}

	private void encodeSendeNoncePIN(SendeNoncePIN c) {
		encodeDestination[encoding_length++] = Code.SENDENONCEPIN;
		encode(c.nonce);
	}

	private void decodeSendeNoncePINInto(byte[] in, SendeNoncePIN into) {
		if (in[encoding_length++] != Code.SENDENONCEPIN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeSendeNoncePIN(SendeNoncePIN c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodeSendePIN(SendePIN c) {
		encodeDestination[encoding_length++] = Code.SENDEPIN;
		encode(c.pindata);
	}

	private void decodeSendePINInto(byte[] in, SendePIN into) {
		if (in[encoding_length++] != Code.SENDEPIN)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.pindata));
	}

	private void compSizeSendePIN(SendePIN c) {
		encoding_length++;
		compSize(c.pindata);
	}

	private void encodeBestaetigePIN(BestaetigePIN c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGEPIN;
	}

	private void decodeBestaetigePINInto(byte[] in, BestaetigePIN into) {
		if (in[encoding_length++] != Code.BESTAETIGEPIN)
			SimpleComm.raiseError();
	}

	private void compSizeBestaetigePIN(BestaetigePIN c) {
		encoding_length++;
	}

	private void encodeErfrageNonceSchluesselaustausch(
			ErfrageNonceSchluesselaustausch c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCESCHLUESSELAUSTAUSCH;
	}

	private void decodeErfrageNonceSchluesselaustauschInto(byte[] in,
			ErfrageNonceSchluesselaustausch into) {
		if (in[encoding_length++] != Code.ERFRAGENONCESCHLUESSELAUSTAUSCH)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceSchluesselaustausch(
			ErfrageNonceSchluesselaustausch c) {
		encoding_length++;
	}

	private void encodeSendeNonceSchluesselaustausch(
			SendeNonceSchluesselaustausch c) {
		encodeDestination[encoding_length++] = Code.SENDENONCESCHLUESSELAUSTAUSCH;
		encode(c.nd);
	}

	private void decodeSendeNonceSchluesselaustauschInto(byte[] in,
			SendeNonceSchluesselaustausch into) {
		if (in[encoding_length++] != Code.SENDENONCESCHLUESSELAUSTAUSCH)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.nd));
	}

	private void compSizeSendeNonceSchluesselaustausch(
			SendeNonceSchluesselaustausch c) {
		encoding_length++;
		compSize(c.nd);
	}

	private void encodeSendeSessionkeyKiosk(SendeSessionkeyKiosk c) {
		encodeDestination[encoding_length++] = Code.SENDESESSIONKEYKIOSK;
		encode(c.kd);
	}

	private void decodeSendeSessionkeyKioskInto(byte[] in,
			SendeSessionkeyKiosk into) {
		if (in[encoding_length++] != Code.SENDESESSIONKEYKIOSK)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.kd));
	}

	private void compSizeSendeSessionkeyKiosk(SendeSessionkeyKiosk c) {
		encoding_length++;
		compSize(c.kd);
	}

	private void encodeSchluesselKioskOK(SchluesselKioskOK c) {
		encodeDestination[encoding_length++] = Code.SCHLUESSELKIOSKOK;
	}

	private void decodeSchluesselKioskOKInto(byte[] in, SchluesselKioskOK into) {
		if (in[encoding_length++] != Code.SCHLUESSELKIOSKOK)
			SimpleComm.raiseError();
	}

	private void compSizeSchluesselKioskOK(SchluesselKioskOK c) {
		encoding_length++;
	}

	private void encodeLeseStammUndNotfalldaten(LeseStammUndNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.LESESTAMMUNDNOTFALLDATEN;
		encode(c.nd);
	}

	private void decodeLeseStammUndNotfalldatenInto(byte[] in,
			LeseStammUndNotfalldaten into) {
		if (in[encoding_length++] != Code.LESESTAMMUNDNOTFALLDATEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.nd));
	}

	private void compSizeLeseStammUndNotfalldaten(LeseStammUndNotfalldaten c) {
		encoding_length++;
		compSize(c.nd);
	}

	private void encodeSendeStammUndNotfalldaten(SendeStammUndNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.SENDESTAMMUNDNOTFALLDATEN;
		encode(c.sd);
	}

	private void decodeSendeStammUndNotfalldatenInto(byte[] in,
			SendeStammUndNotfalldaten into) {
		if (in[encoding_length++] != Code.SENDESTAMMUNDNOTFALLDATEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.sd));
	}

	private void compSizeSendeStammUndNotfalldaten(SendeStammUndNotfalldaten c) {
		encoding_length++;
		compSize(c.sd);
	}

	private void encodeLeseRezept(LeseRezept c) {
		encodeDestination[encoding_length++] = Code.LESEREZEPT;
		encode(c.ndi);
	}

	private void decodeLeseRezeptInto(byte[] in, LeseRezept into) {
		if (in[encoding_length++] != Code.LESEREZEPT)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.ndi));
	}

	private void compSizeLeseRezept(LeseRezept c) {
		encoding_length++;
		compSize(c.ndi);
	}

	private void encodeSendeRezeptdaten(SendeRezeptdaten c) {
		encodeDestination[encoding_length++] = Code.SENDEREZEPTDATEN;
		encode(c.nur);
	}

	private void decodeSendeRezeptdatenInto(byte[] in, SendeRezeptdaten into) {
		if (in[encoding_length++] != Code.SENDEREZEPTDATEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.nur));
	}

	private void compSizeSendeRezeptdaten(SendeRezeptdaten c) {
		encoding_length++;
		compSize(c.nur);
	}

	private void encodeErfrageNonceNotfalldatenLoeschen(
			ErfrageNonceNotfalldatenLoeschen c) {
		encodeDestination[encoding_length++] = Code.ERFRAGENONCENOTFALLDATENLOESCHEN;
	}

	private void decodeErfrageNonceNotfalldatenLoeschenInto(byte[] in,
			ErfrageNonceNotfalldatenLoeschen into) {
		if (in[encoding_length++] != Code.ERFRAGENONCENOTFALLDATENLOESCHEN)
			SimpleComm.raiseError();
	}

	private void compSizeErfrageNonceNotfalldatenLoeschen(
			ErfrageNonceNotfalldatenLoeschen c) {
		encoding_length++;
	}

	private void encodeLoescheNotfalldaten(LoescheNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.LOESCHENOTFALLDATEN;
		encode(c.nsd);
	}

	private void decodeLoescheNotfalldatenInto(byte[] in,
			LoescheNotfalldaten into) {
		if (in[encoding_length++] != Code.LOESCHENOTFALLDATEN)
			SimpleComm.raiseError();
		decodeEncDataSymmInto(in, (EncDataSymm) (into.nsd));
	}

	private void compSizeLoescheNotfalldaten(LoescheNotfalldaten c) {
		encoding_length++;
		compSize(c.nsd);
	}

	private void encodeBestaetigeLoeschenNotfalldaten(
			BestaetigeLoeschenNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.BESTAETIGELOESCHENNOTFALLDATEN;
	}

	private void decodeBestaetigeLoeschenNotfalldatenInto(byte[] in,
			BestaetigeLoeschenNotfalldaten into) {
		if (in[encoding_length++] != Code.BESTAETIGELOESCHENNOTFALLDATEN)
			SimpleComm.raiseError();
	}

	private void compSizeBestaetigeLoeschenNotfalldaten(
			BestaetigeLoeschenNotfalldaten c) {
		encoding_length++;
	}

	private void encodeSendeNonce(SendeNonce c) {
		encodeDestination[encoding_length++] = Code.SENDENONCE;
		encode(c.nonce);
	}

	private void decodeSendeNonceInto(byte[] in, SendeNonce into) {
		if (in[encoding_length++] != Code.SENDENONCE)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeSendeNonce(SendeNonce c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodeGetData(GetData c) {
		encodeDestination[encoding_length++] = Code.GETDATA;
	}

	private void decodeGetDataInto(byte[] in, GetData into) {
		if (in[encoding_length++] != Code.GETDATA)
			SimpleComm.raiseError();
	}

	private void compSizeGetData(GetData c) {
		encoding_length++;
	}

	private void encodeOK(OK c) {
		encodeDestination[encoding_length++] = Code.OK;
	}

	private void decodeOKInto(byte[] in, OK into) {
		if (in[encoding_length++] != Code.OK)
			SimpleComm.raiseError();
	}

	private void compSizeOK(OK c) {
		encoding_length++;
	}

	private void encodeCardData(CardData c) {
		encodeDestination[encoding_length++] = Code.CARDDATA;
		encode(c.nonce);
		encode(c.daten);
		encode(c.sigdaten);
		encode(c.zertEGK);
	}

	private void decodeCardDataInto(byte[] in, CardData into) {
		if (in[encoding_length++] != Code.CARDDATA)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.daten));
		decodeSignedDataInto(in, (SignedData) (into.sigdaten));
		decodeZertifikatPatientInto(in, (ZertifikatPatient) (into.zertEGK));
	}

	private void compSizeCardData(CardData c) {
		encoding_length++;
		compSize(c.nonce);
		compSize(c.daten);
		compSize(c.sigdaten);
		compSize(c.zertEGK);
	}

	private void encodeUpdate(Update c) {
		encodeDestination[encoding_length++] = Code.UPDATE;
		encode(c.daten);
		encode(c.sigdaten);
		encode(c.sigWithNonce);
	}

	private void decodeUpdateInto(byte[] in, Update into) {
		if (in[encoding_length++] != Code.UPDATE)
			SimpleComm.raiseError();
		decodeEncDataAsymmInto(in, (EncDataAsymm) (into.daten));
		decodeSignedDataInto(in, (SignedData) (into.sigdaten));
		decodeSignedDataInto(in, (SignedData) (into.sigWithNonce));
	}

	private void compSizeUpdate(Update c) {
		encoding_length++;
		compSize(c.daten);
		compSize(c.sigdaten);
		compSize(c.sigWithNonce);
	}

	private void encodeNonce(Nonce c) {
		encodeDestination[encoding_length++] = Code.NONCE;
		encodeByteArray((byte[]) c.nonce);
	}

	private void decodeNonceInto(byte[] in, Nonce into) {
		if (in[encoding_length++] != Code.NONCE)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.nonce);
	}

	private void compSizeNonce(Nonce c) {
		encoding_length++;
		compSizeByteArray(c.nonce);
	}

	private void encodeZertifikatLeistungserbringer(
			ZertifikatLeistungserbringer c) {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATLEISTUNGSERBRINGER;
		encode(c.signedCert);
		encode(c.data);
	}

	private void decodeZertifikatLeistungserbringerInto(byte[] in,
			ZertifikatLeistungserbringer into) {
		if (in[encoding_length++] != Code.ZERTIFIKATLEISTUNGSERBRINGER)
			SimpleComm.raiseError();
		decodeSignedDataInto(in, (SignedData) (into.signedCert));
		decodeLeistungserbringerInfoInto(in,
				(LeistungserbringerInfo) (into.data));
	}

	private void compSizeZertifikatLeistungserbringer(
			ZertifikatLeistungserbringer c) {
		encoding_length++;
		compSize(c.signedCert);
		compSize(c.data);
	}

	private void encodeZertifikatPatient(ZertifikatPatient c) {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATPATIENT;
		encode(c.signedCert);
		encode(c.data);
	}

	private void decodeZertifikatPatientInto(byte[] in, ZertifikatPatient into) {
		if (in[encoding_length++] != Code.ZERTIFIKATPATIENT)
			SimpleComm.raiseError();
		decodeSignedDataInto(in, (SignedData) (into.signedCert));
		decodeZertifikatPatientDataInto(in, (ZertifikatPatientData) (into.data));
	}

	private void compSizeZertifikatPatient(ZertifikatPatient c) {
		encoding_length++;
		compSize(c.signedCert);
		compSize(c.data);
	}

	private void encodeEncDataAsymm(EncDataAsymm c) {

		encodeDestination[encoding_length++] = Code.ENCDATAASYMM;
		encodeByteArray(c.encrypted);

		encodeShort(c.plainlength);
		encodeShort(c.enclength);

		encodeByte(c.plainDataType);
	}

	private void decodeEncDataAsymmInto(byte[] in, EncDataAsymm into) {
		if (in[encoding_length++] != Code.ENCDATAASYMM)
			SimpleComm.raiseError();
		//public byte[] encrypted;
		//public short plainlength;
		//public short enclength;
		//public byte plainDataType;
		decodeByteArrayInto(in, into.encrypted);
		into.plainlength = decodeShort(in);
		into.enclength = decodeShort(in);
		byte b = decodeByte(in);
		into.plainDataType = b;
		if (b != Code.NONCEDATA && b != Code.PINDATA && b != Code.EGKDATEN
				&& b != Code.EREZEPTDATEN && b != Code.SECRETNONCEDATA
				&& b != Code.KEYDATA && b != Code.INDEXNONCE
				&& b != Code.REZEPTDATENUNDNONCE && b != Code.NOTFALLDATEN
				&& b != Code.NOTFALLDATENNONCE
				&& b != Code.STAMMUNDNOTFALLDATEN
				&& b != Code.NONCEDATAANDINDEX && b != Code.NONCEUNDREZEPT
				&& b != Code.NONCESTRINGDATA
				&& b != Code.VERSICHERTENSTAMMDATEN)
			SimpleComm.raiseError();
	}

	private void compSizeEncDataAsymm(EncDataAsymm c) {
		encoding_length++;
		compSizeByteArray(c.encrypted);
		encoding_length += 8; // short + short + byte
	}

	private void encodeNonceData(NonceData c) {
		encodeDestination[encoding_length++] = Code.NONCEDATA;
		encode(c.nonce);
	}

	private void decodeNonceDataInto(byte[] in, NonceData into) {
		if (in[encoding_length++] != Code.NONCEDATA)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeNonceData(NonceData c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodePINData(PINData c) {
		encodeDestination[encoding_length++] = Code.PINDATA;
		encode(c.pin);
		encode(c.n);
	}

	private void decodePINDataInto(byte[] in, PINData into) {
		if (in[encoding_length++] != Code.PINDATA)
			SimpleComm.raiseError();
		decodeSecretInto(in, (Secret) (into.pin));
		decodeNonceInto(in, (Nonce) (into.n));
	}

	private void compSizePINData(PINData c) {
		encoding_length++;
		compSize(c.pin);
		compSize(c.n);
	}

	private void encodeEncDataSymm(EncDataSymm c) {

		encodeDestination[encoding_length++] = Code.ENCDATASYMM;
		encodeByteArray(c.encrypted);

		encodeShort(c.plainlength);
		encodeShort(c.enclength);

		encodeByte(c.plainDataType);
	}

	private void decodeEncDataSymmInto(byte[] in, EncDataSymm into) {
		if (in[encoding_length++] != Code.ENCDATASYMM)
			SimpleComm.raiseError();
		//public byte[] encrypted;
		//public short plainlength;
		//public short enclength;
		//public byte plainDataType;
		decodeByteArrayInto(in, into.encrypted);
		into.plainlength = decodeShort(in);
		into.enclength = decodeShort(in);
		byte b = decodeByte(in);
		into.plainDataType = b;
		if (b != Code.NONCEDATA && b != Code.PINDATA && b != Code.EGKDATEN
				&& b != Code.EREZEPTDATEN && b != Code.SECRETNONCEDATA
				&& b != Code.KEYDATA && b != Code.INDEXNONCE
				&& b != Code.REZEPTDATENUNDNONCE && b != Code.NOTFALLDATEN
				&& b != Code.NOTFALLDATENNONCE
				&& b != Code.STAMMUNDNOTFALLDATEN
				&& b != Code.NONCEDATAANDINDEX && b != Code.NONCEUNDREZEPT
				&& b != Code.NONCESTRINGDATA
				&& b != Code.VERSICHERTENSTAMMDATEN)
			SimpleComm.raiseError();
	}

	private void compSizeEncDataSymm(EncDataSymm c) {
		encoding_length++;
		compSizeByteArray(c.encrypted);
		encoding_length += 8; // short + short + byte
	}

	private void encodeEGKDaten(EGKDaten c) {
		encodeDestination[encoding_length++] = Code.EGKDATEN;
		encode(c.nonce);
		encode(c.daten);
	}

	private void decodeEGKDatenInto(byte[] in, EGKDaten into) {
		if (in[encoding_length++] != Code.EGKDATEN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeVersichertenstammdatenInto(in,
				(Versichertenstammdaten) (into.daten));
	}

	private void compSizeEGKDaten(EGKDaten c) {
		encoding_length++;
		compSize(c.nonce);
		compSize(c.daten);
	}

	private void encodeERezeptdaten(ERezeptdaten c) {
		encodeDestination[encoding_length++] = Code.EREZEPTDATEN;
		encode(c.nonce);
		encodeByteArray((byte[]) c.ausstellungsdatum);
		encodeByteArray((byte[]) c.angabezureinnahme);
		encode(c.arzneimittel);
		encode(c.ausgestelltfuer);
	}

	private void decodeERezeptdatenInto(byte[] in, ERezeptdaten into) {
		if (in[encoding_length++] != Code.EREZEPTDATEN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeByteArrayInto(in, into.ausstellungsdatum);
		decodeByteArrayInto(in, into.angabezureinnahme);
		decodeArzneimittelInto(in, (Arzneimittel) (into.arzneimittel));
		decodeVersichertenstammdatenInto(in,
				(Versichertenstammdaten) (into.ausgestelltfuer));
	}

	private void compSizeERezeptdaten(ERezeptdaten c) {
		encoding_length++;
		compSize(c.nonce);
		compSizeByteArray(c.ausstellungsdatum);
		compSizeByteArray(c.angabezureinnahme);
		compSize(c.arzneimittel);
		compSize(c.ausgestelltfuer);
	}

	private void encodeSignedData(SignedData c) {
		encodeDestination[encoding_length++] = Code.SIGNEDDATA;
		encodeByteArray((byte[]) c.signed);
	}

	private void decodeSignedDataInto(byte[] in, SignedData into) {
		if (in[encoding_length++] != Code.SIGNEDDATA)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.signed);
	}

	private void compSizeSignedData(SignedData c) {
		encoding_length++;
		compSizeByteArray(c.signed);
	}

	private void encodeSecretNonceData(SecretNonceData c) {
		encodeDestination[encoding_length++] = Code.SECRETNONCEDATA;
		encode(c.nonce);
	}

	private void decodeSecretNonceDataInto(byte[] in, SecretNonceData into) {
		if (in[encoding_length++] != Code.SECRETNONCEDATA)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
	}

	private void compSizeSecretNonceData(SecretNonceData c) {
		encoding_length++;
		compSize(c.nonce);
	}

	private void encodeKeyData(KeyData c) {
		encodeDestination[encoding_length++] = Code.KEYDATA;
		encode(c.nonce);
		encode(c.sessionkey);
	}

	private void decodeKeyDataInto(byte[] in, KeyData into) {
		if (in[encoding_length++] != Code.KEYDATA)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeSymmKeyInto(in, (SymmKey) (into.sessionkey));
	}

	private void compSizeKeyData(KeyData c) {
		encoding_length++;
		compSize(c.nonce);
		compSize(c.sessionkey);
	}

	private void encodeIndexNonce(IndexNonce c) {
		encodeDestination[encoding_length++] = Code.INDEXNONCE;
		encodeShort((short) c.index);
		encode(c.nonce);
		encode(c.nonceApothekenPC);
	}

	private void decodeIndexNonceInto(byte[] in, IndexNonce into) {
		if (in[encoding_length++] != Code.INDEXNONCE)
			SimpleComm.raiseError();
		into.index = decodeShort(in);
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeNonceInto(in, (Nonce) (into.nonceApothekenPC));
	}

	private void compSizeIndexNonce(IndexNonce c) {
		encoding_length++;
		compSizeShort(c.index);
		compSize(c.nonce);
		compSize(c.nonceApothekenPC);
	}

	private void encodeRezeptdatenUndNonce(RezeptdatenUndNonce c) {
		encodeDestination[encoding_length++] = Code.REZEPTDATENUNDNONCE;
		encode(c.nonceApothekenPC);
		encode(c.rezeptdaten);
	}

	private void decodeRezeptdatenUndNonceInto(byte[] in,
			RezeptdatenUndNonce into) {
		if (in[encoding_length++] != Code.REZEPTDATENUNDNONCE)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonceApothekenPC));
		decodeERezeptdatenInto(in, (ERezeptdaten) (into.rezeptdaten));
	}

	private void compSizeRezeptdatenUndNonce(RezeptdatenUndNonce c) {
		encoding_length++;
		compSize(c.nonceApothekenPC);
		compSize(c.rezeptdaten);
	}

	private void encodeNotfalldaten(Notfalldaten c) {
		encodeDestination[encoding_length++] = Code.NOTFALLDATEN;
		encode(c.nonce);
		encodeByteArray((byte[]) c.daten);
	}

	private void decodeNotfalldatenInto(byte[] in, Notfalldaten into) {
		if (in[encoding_length++] != Code.NOTFALLDATEN)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeByteArrayInto(in, into.daten);
	}

	private void compSizeNotfalldaten(Notfalldaten c) {
		encoding_length++;
		compSize(c.nonce);
		compSizeByteArray(c.daten);
	}

	private void encodeNotfalldatenNonce(NotfalldatenNonce c) {
		encodeDestination[encoding_length++] = Code.NOTFALLDATENNONCE;
		encode(c.nonce);
		encodeByteArray((byte[]) c.daten);
	}

	private void decodeNotfalldatenNonceInto(byte[] in, NotfalldatenNonce into) {
		if (in[encoding_length++] != Code.NOTFALLDATENNONCE)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeByteArrayInto(in, into.daten);
	}

	private void compSizeNotfalldatenNonce(NotfalldatenNonce c) {
		encoding_length++;
		compSize(c.nonce);
		compSizeByteArray(c.daten);
	}

	private void encodeZertifikatKiosk(ZertifikatKiosk c) {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATKIOSK;
		encode(c.data);
		encode(c.signedCert);
	}

	private void decodeZertifikatKioskInto(byte[] in, ZertifikatKiosk into) {
		if (in[encoding_length++] != Code.ZERTIFIKATKIOSK)
			SimpleComm.raiseError();
		decodeZertifikatKioskDataInto(in, (ZertifikatKioskData) (into.data));
		decodeSignedDataInto(in, (SignedData) (into.signedCert));
	}

	private void compSizeZertifikatKiosk(ZertifikatKiosk c) {
		encoding_length++;
		compSize(c.data);
		compSize(c.signedCert);
	}

	private void encodeStammUndNotfalldaten(StammUndNotfalldaten c) {
		encodeDestination[encoding_length++] = Code.STAMMUNDNOTFALLDATEN;
		encodeByteArray((byte[]) c.notfalldaten);
		encode(c.nonce);
		encodeShort((short) c.anzahlRezepte);
		encode(c.vsd);
	}

	private void decodeStammUndNotfalldatenInto(byte[] in,
			StammUndNotfalldaten into) {
		if (in[encoding_length++] != Code.STAMMUNDNOTFALLDATEN)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.notfalldaten);
		decodeNonceInto(in, (Nonce) (into.nonce));
		into.anzahlRezepte = decodeShort(in);
		decodeVersichertenstammdatenInto(in,
				(Versichertenstammdaten) (into.vsd));
	}

	private void compSizeStammUndNotfalldaten(StammUndNotfalldaten c) {
		encoding_length++;
		compSizeByteArray(c.notfalldaten);
		compSize(c.nonce);
		compSizeShort(c.anzahlRezepte);
		compSize(c.vsd);
	}

	private void encodeNonceDataAndIndex(NonceDataAndIndex c) {
		encodeDestination[encoding_length++] = Code.NONCEDATAANDINDEX;
		encode(c.nonce);
		encodeShort((short) c.index);
	}

	private void decodeNonceDataAndIndexInto(byte[] in, NonceDataAndIndex into) {
		if (in[encoding_length++] != Code.NONCEDATAANDINDEX)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		into.index = decodeShort(in);
	}

	private void compSizeNonceDataAndIndex(NonceDataAndIndex c) {
		encoding_length++;
		compSize(c.nonce);
		compSizeShort(c.index);
	}

	private void encodeNonceUndRezept(NonceUndRezept c) {
		encodeDestination[encoding_length++] = Code.NONCEUNDREZEPT;
		encode(c.nonce);
		encode(c.rd);
	}

	private void decodeNonceUndRezeptInto(byte[] in, NonceUndRezept into) {
		if (in[encoding_length++] != Code.NONCEUNDREZEPT)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeERezeptdatenInto(in, (ERezeptdaten) (into.rd));
	}

	private void compSizeNonceUndRezept(NonceUndRezept c) {
		encoding_length++;
		compSize(c.nonce);
		compSize(c.rd);
	}

	private void encodeNonceStringData(NonceStringData c) {
		encodeDestination[encoding_length++] = Code.NONCESTRINGDATA;
		encode(c.nonce);
		encodeByteArray((byte[]) c.s);
	}

	private void decodeNonceStringDataInto(byte[] in, NonceStringData into) {
		if (in[encoding_length++] != Code.NONCESTRINGDATA)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeByteArrayInto(in, into.s);
	}

	private void compSizeNonceStringData(NonceStringData c) {
		encoding_length++;
		compSize(c.nonce);
		compSizeByteArray(c.s);
	}

	private void encodeVersichertenstammdaten(Versichertenstammdaten c) {
		encodeDestination[encoding_length++] = Code.VERSICHERTENSTAMMDATEN;
		encodeByteArray((byte[]) c.name);
		encodeByteArray((byte[]) c.versichertennr);
		encodeShort((short) c.KennummerKrankenkasse);
		encodeByteArray((byte[]) c.adresse);
	}

	private void decodeVersichertenstammdatenInto(byte[] in,
			Versichertenstammdaten into) {
		if (in[encoding_length++] != Code.VERSICHERTENSTAMMDATEN)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.name);
		decodeByteArrayInto(in, into.versichertennr);
		into.KennummerKrankenkasse = decodeShort(in);
		decodeByteArrayInto(in, into.adresse);
	}

	private void compSizeVersichertenstammdaten(Versichertenstammdaten c) {
		encoding_length++;
		compSizeByteArray(c.name);
		compSizeByteArray(c.versichertennr);
		compSizeShort(c.KennummerKrankenkasse);
		compSizeByteArray(c.adresse);
	}

	private void encodeUpdateAuth(UpdateAuth c) {
		encodeDestination[encoding_length++] = Code.UPDATEAUTH;
		encode(c.nonce);
		encode(c.data);
	}

	private void decodeUpdateAuthInto(byte[] in, UpdateAuth into) {
		if (in[encoding_length++] != Code.UPDATEAUTH)
			SimpleComm.raiseError();
		decodeNonceInto(in, (Nonce) (into.nonce));
		decodeVersichertenstammdatenInto(in,
				(Versichertenstammdaten) (into.data));
	}

	private void compSizeUpdateAuth(UpdateAuth c) {
		encoding_length++;
		compSize(c.nonce);
		compSize(c.data);
	}

	private void encodeLeistungserbringerInfo(LeistungserbringerInfo c) {
		encodeDestination[encoding_length++] = Code.LEISTUNGSERBRINGERINFO;
		encodeByteArray((byte[]) c.name);
		encode(c.pubkey);
		encodeShort((short) c.id);
	}

	private void decodeLeistungserbringerInfoInto(byte[] in,
			LeistungserbringerInfo into) {
		if (in[encoding_length++] != Code.LEISTUNGSERBRINGERINFO)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.name);
		decodePublicKeyInto(in, (PublicKey) (into.pubkey));
		into.id = decodeShort(in);
	}

	private void compSizeLeistungserbringerInfo(LeistungserbringerInfo c) {
		encoding_length++;
		compSizeByteArray(c.name);
		compSize(c.pubkey);
		compSizeShort(c.id);
	}

	private void encodeZertifikatPatientData(ZertifikatPatientData c) {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATPATIENTDATA;
		encodeByteArray((byte[]) c.name);
		encode(c.pubkey);
		encodeShort((short) c.versichertennr);
		encodeShort((short) c.krankenkassenID);
		encodeShort((short) c.id);
	}

	private void decodeZertifikatPatientDataInto(byte[] in,
			ZertifikatPatientData into) {
		if (in[encoding_length++] != Code.ZERTIFIKATPATIENTDATA)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.name);
		decodePublicKeyInto(in, (PublicKey) (into.pubkey));
		into.versichertennr = decodeShort(in);
		into.krankenkassenID = decodeShort(in);
		into.id = decodeShort(in);
	}

	private void compSizeZertifikatPatientData(ZertifikatPatientData c) {
		encoding_length++;
		compSizeByteArray(c.name);
		compSize(c.pubkey);
		compSizeShort(c.versichertennr);
		compSizeShort(c.krankenkassenID);
		compSizeShort(c.id);
	}

	private void encodeSecret(Secret c) {
		encodeDestination[encoding_length++] = Code.SECRET;
		encodeByteArray((byte[]) c.secret);
	}

	private void decodeSecretInto(byte[] in, Secret into) {
		if (in[encoding_length++] != Code.SECRET)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.secret);
	}

	private void compSizeSecret(Secret c) {
		encoding_length++;
		compSizeByteArray(c.secret);
	}

	private void encodeArzneimittel(Arzneimittel c) {
		encodeDestination[encoding_length++] = Code.ARZNEIMITTEL;
		encodeByteArray((byte[]) c.name);
		encodeByteArray((byte[]) c.einnahmemenge);
		encodeByteArray((byte[]) c.arzneimittelrisiko);
	}

	private void decodeArzneimittelInto(byte[] in, Arzneimittel into) {
		if (in[encoding_length++] != Code.ARZNEIMITTEL)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.name);
		decodeByteArrayInto(in, into.einnahmemenge);
		decodeByteArrayInto(in, into.arzneimittelrisiko);
	}

	private void compSizeArzneimittel(Arzneimittel c) {
		encoding_length++;
		compSizeByteArray(c.name);
		compSizeByteArray(c.einnahmemenge);
		compSizeByteArray(c.arzneimittelrisiko);
	}

	private void encodeSymmKey(SymmKey c) {
		encodeDestination[encoding_length++] = Code.SYMMKEY;
		// public byte[] key; - ok
		// private DESKey apikey; - ignored
		encodeByteArray(c.key);
	}

	private void decodeSymmKeyInto(byte[] in, SymmKey into) {
		if (in[encoding_length++] != Code.SYMMKEY)
			SimpleComm.raiseError();
		// public byte[] key; - ok
		// private DESKey apikey; - handled by updateKey() below
		decodeByteArrayInto(in, into.key);
		into.updateKey();
	}

	private void compSizeSymmKey(SymmKey c) {
		encoding_length++;
		// public byte[] key; - ok
		// private DESKey apikey; - ignored
		compSizeByteArray(c.key);
	}

	private void encodeZertifikatKioskData(ZertifikatKioskData c) {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATKIOSKDATA;
		encodeByteArray((byte[]) c.name);
		encode(c.pubkey);
		encodeShort((short) c.id);
	}

	private void decodeZertifikatKioskDataInto(byte[] in,
			ZertifikatKioskData into) {
		if (in[encoding_length++] != Code.ZERTIFIKATKIOSKDATA)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.name);
		decodePublicKeyInto(in, (PublicKey) (into.pubkey));
		into.id = decodeShort(in);
	}

	private void compSizeZertifikatKioskData(ZertifikatKioskData c) {
		encoding_length++;
		compSizeByteArray(c.name);
		compSize(c.pubkey);
		compSizeShort(c.id);
	}

	private void encodePublicKey(PublicKey c) {
		encodeDestination[encoding_length++] = Code.PUBLICKEY;

		encodeByteArray(c.modulus);
	}

	private void decodePublicKeyInto(byte[] in, PublicKey into) {
		if (in[encoding_length++] != Code.PUBLICKEY)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.modulus);
		into.updateKey();
	}

	private void compSizePublicKey(PublicKey c) {
		encoding_length++;

		compSizeByteArray(c.modulus);

	}

	private void encodeZertifikatTrustedThirdParty(ZertifikatTrustedThirdParty c) {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATTRUSTEDTHIRDPARTY;
		encode(c.signedCert);
		encode(c.data);
	}

	private void decodeZertifikatTrustedThirdPartyInto(byte[] in,
			ZertifikatTrustedThirdParty into) {
		if (in[encoding_length++] != Code.ZERTIFIKATTRUSTEDTHIRDPARTY)
			SimpleComm.raiseError();
		decodeSignedDataInto(in, (SignedData) (into.signedCert));
		decodeZertifikatTTPDataInto(in, (ZertifikatTTPData) (into.data));
	}

	private void compSizeZertifikatTrustedThirdParty(
			ZertifikatTrustedThirdParty c) {
		encoding_length++;
		compSize(c.signedCert);
		compSize(c.data);
	}

	public void encodePrivateKey(PrivateKey c) {
		encodeDestination[encoding_length++] = Code.PRIVATEKEY;

		encodeByteArray(c.modulus);
		encodeByteArray(c.privexponent);

	}

	private void decodePrivateKeyInto(byte[] in, PrivateKey into) {
		if (in[encoding_length++] != Code.PRIVATEKEY)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.modulus);
		decodeByteArrayInto(in, into.privexponent);
		into.updateKey();
	}

	private void compSizePrivateKey(PrivateKey c) {
		encoding_length++;

		compSizeByteArray(c.modulus);
		compSizeByteArray(c.privexponent);

	}

	private void encodeZertifikatTTPData(ZertifikatTTPData c) {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATTTPDATA;
		encodeByteArray((byte[]) c.name);
		encode(c.pubkey);
	}

	private void decodeZertifikatTTPDataInto(byte[] in, ZertifikatTTPData into) {
		if (in[encoding_length++] != Code.ZERTIFIKATTTPDATA)
			SimpleComm.raiseError();
		decodeByteArrayInto(in, into.name);
		decodePublicKeyInto(in, (PublicKey) (into.pubkey));
	}

	private void compSizeZertifikatTTPData(ZertifikatTTPData c) {
		encoding_length++;
		compSizeByteArray(c.name);
		compSize(c.pubkey);
	}

	public void decodeHeilberufsausweisApotheker(byte[] in, short offset,
			HeilberufsausweisApotheker thisHeilberufsausweisApotheker) {
		encoding_length = offset;
		if (in[encoding_length++] != Code.HEILBERUFSAUSWEISAPOTHEKER)
			SimpleComm.raiseError();

		if (in[encoding_length] != Code.IGNORE) {
			decodeSecretInto(in, thisHeilberufsausweisApotheker.pin);
		} else
			encoding_length++;

		if (in[encoding_length] != Code.IGNORE) {
			decodePrivateKeyInto(in, thisHeilberufsausweisApotheker.privkeyHBA);
		} else
			encoding_length++;

		if (in[encoding_length] != Code.IGNORE) {
			decodeZertifikatLeistungserbringerInto(in,
					thisHeilberufsausweisApotheker.zert);
		} else
			encoding_length++;

		if (in[encoding_length] != Code.IGNORE) {
			decodeZertifikatTrustedThirdPartyInto(in,
					thisHeilberufsausweisApotheker.zertTTP);
		} else
			encoding_length++;

		if (in[encoding_length] != Code.IGNORE) {
			thisHeilberufsausweisApotheker.zustand = decodeByte(in);
		} else
			encoding_length++;
	}

}
