package generated.eGKmitServices.server.krankenkasse;

import swt.util.ByteArray;
import java.nio.charset.Charset;

/**
 * Class for T(L)V encoding of messages and other classes
 * 
 */
public class Coding {

	/**
	 * offset into encodeDestination for writing/reading
	 */
	private short encoding_length;

	/**
	 * array used for encoding/decoding
	 */
	private byte[] encodeDestination;

	/**
	 * Singleton instance
	 */
	private static Coding the_instance;

	private static Charset charset = Charset.forName("UTF-8");

	/**
	 * Singleton accessor
	 * 
	 * @return Singleton instance
	 */

	public static Coding getInstance() {
		if (the_instance == null)
			the_instance = new Coding();
		return the_instance;
	}
	/**
	 * return offset into encodeDestination
	 * 
	 * @return value of encoding_length
	 */
	public short getEncodingLength() {
		return encoding_length;
	}

	/**
	 * initializes encoding. This method must be called before encoding an object.
	 * The offset (encoding_length) is reset to 0 and encodeDestination is set to dest.
	 * 
	 * @param dest The (empty) destination array for encoding
	 */
	public void encodeInit(byte[] dest) {
		encodeDestination = dest;
		encoding_length = 0;
	}
	/**
	 * initializes decoding. The offset (encoding_length) is reset to 0.
	 */
	public void decodeInit() {
		encoding_length = 0;
	}

	/**
	 * initializes encoding and encodes an object.
	 * 
	 * @param o The object to encode
	 * @param dest destination array for the encoding
	 * @return length of the encoded object
	 */

	/**
	 * decodes an object beginning at given offset
	 * 
	 * @param in array containing encoded object
	 * @param offset where the encoded object begins
	 * @return the decoded object
	 */

	private byte decodeByte(byte[] in) throws ServiceException {
		if (in[encoding_length++] != Code.BYTE) {

			stop();
		}

		byte res = in[encoding_length++];
		return res;
	}

	private boolean decodeBoolean(byte[] in) throws ServiceException {
		if (in[encoding_length++] != Code.BOOLEAN) {
			stop();

		}
		boolean res = (in[encoding_length++] == 1 ? true : false);
		return res;
	}

	public Codeable decode(byte[] in, short offset) throws Exception {
		encoding_length = offset;
		return decode(in);
	}
	public byte[] stringToByte(String s) {
		byte[] b = s.getBytes(charset);
		byte[] ret = new byte[LengthConstants.LENGTHOFSTRING];
		int len = (b.length < LengthConstants.LENGTHOFSTRING)
				? b.length
				: LengthConstants.LENGTHOFSTRING;
		ByteArray.copy(b, 0, ret, 0, len);
		return ret;
	}

	public boolean isValidCardString(String s) {
		return (s.getBytes(charset).length <= LengthConstants.LENGTHOFSTRING);
	}

	public void encodeString(String i) {
		byte[] o = stringToByte(i);
		encodeDestination[encoding_length++] = Code.BYTEARRAY;
		ByteArray.setShort(encodeDestination, encoding_length,
				(short) (o.length));
		encoding_length += 2;
		ByteArray.copy(o, 0, encodeDestination, encoding_length, o.length);
		encoding_length += o.length;
	}

	private void compSizeString(String i) {
		byte[] o = stringToByte(i);
		encoding_length += 3;
		encoding_length += o.length;
	}

	public short encode(Codeable o, byte[] dest) throws Exception {
		encodeInit(dest);

		encode(o);
		return computeSize(o);

	}

	/**
	 * computes the encoding length of an object without actually encoding it.
	 * 
	 * @param o The object of which the encoding length should be computed
	 * @return encoding length
	 */

	public short computeSize(Codeable o) {
		encoding_length = 0;
		try {
			compSize(o);
			return encoding_length;
		} catch (Exception e) {
			return -1;
		}
	}

	public void encodeByte(byte o) {
		encodeDestination[encoding_length++] = Code.BYTE;
		encodeDestination[encoding_length++] = o;
	}

	private void encodeBoolean(boolean o) {
		encodeDestination[encoding_length++] = Code.BOOLEAN;
		encodeDestination[encoding_length++] = (byte) (o ? 1 : 0);
	}

	private void compSizeByte(byte o) {
		encoding_length += 2;
	}

	public void encodeInt(int i) {
		short o = (short) i;
		if (i != (int) o)
			System.err.println("int too big for cast into short.");
		encodeDestination[encoding_length++] = Code.SHORT;
		ByteArray.setShort(encodeDestination, encoding_length, o);
		encoding_length += 2;
	}

	private void compSizeInt(int i) {
		short o = (short) i;
		encoding_length += 3;
	}
	public int decodeInt(byte[] in) {
		short res = ByteArray.getShort(in, ++encoding_length);
		encoding_length += 2;
		return (int) res;
	}

	public byte[] decodeByteArray(byte[] in) {
		short len = ByteArray.getShort(in, ++encoding_length);
		byte[] dest = new byte[len];
		encoding_length += 2;
		ByteArray.copy(in, encoding_length, dest, 0, len);
		encoding_length += len;
		return dest;
	}

	public String byteToString(byte[] b) {
		int len = b.length;
		while (len > 0 && b[len - 1] == 0)
			len--;
		return new String(ByteArray.subarray(b, 0, len), charset);
	}

	public String decodeString(byte[] in) {
		short len = ByteArray.getShort(in, ++encoding_length);
		byte[] dest = new byte[len];
		encoding_length += 2;
		ByteArray.copy(in, encoding_length, dest, 0, len);
		encoding_length += len;
		String str = byteToString(dest);
		return str;
	}

	private void compSizeBoolean(boolean o) {
		encoding_length += 2;
	}

	public void encodeByteArray(byte[] o) {

		//byte[] o = stringToByte(i);

		encodeDestination[encoding_length++] = Code.BYTEARRAY;

		ByteArray.setShort(encodeDestination, encoding_length,
				(short) (o.length));
		encoding_length += 2;

		ByteArray.copy(o, 0, encodeDestination, encoding_length, o.length);

		encoding_length += o.length;
	}

	private void compSizeByteArray(byte[] o) {
		encoding_length += 3;
		encoding_length += o.length;
	}

	/**
	 * encodes an object without initializing destination and offset.
	 * 
	 * @param c object to encode
	 */

	public void encode(Codeable c) throws Exception {

		switch (c.getCode()) {
			case Code.CARDDATA :
				encodeCardData((CardData) c);
				break;
			case Code.UPDATE :
				encodeUpdate((Update) c);
				break;
			case Code.NONCE :
				encodeNonce((Nonce) c);
				break;
			case Code.ZERTIFIKATPATIENT :
				encodeZertifikatPatient((ZertifikatPatient) c);
				break;
			case Code.SIGNEDDATA :
				encodeSignedData((SignedData) c);
				break;
			case Code.ENCDATAASYMM :
				encodeEncDataAsymm((EncDataAsymm) c);
				break;
			case Code.VERSICHERTENSTAMMDATEN :
				encodeVersichertenstammdaten((Versichertenstammdaten) c);
				break;
			case Code.UPDATEAUTH :
				encodeUpdateAuth((UpdateAuth) c);
				break;
			case Code.ZERTIFIKATPATIENTDATA :
				encodeZertifikatPatientData((ZertifikatPatientData) c);
				break;
			case Code.PUBLICKEY :
				encodePublicKey((PublicKey) c);
				break;
			case Code.PRIVATEKEY :
				encodePrivateKey((PrivateKey) c);
				break;
			case Code.ZERTIFIKATTTPDATA :
				encodeZertifikatTTPData((ZertifikatTTPData) c);
				break;
			// never reached
			default :

				stop();

		}
	}

	public Codeable decode(byte[] in) throws Exception {
		switch (in[encoding_length]) {
			case Code.CARDDATA :
				return decodeCardData(in);
			case Code.UPDATE :
				return decodeUpdate(in);
			case Code.NONCE :
				return decodeNonce(in);
			case Code.ZERTIFIKATPATIENT :
				return decodeZertifikatPatient(in);
			case Code.SIGNEDDATA :
				return decodeSignedData(in);
			case Code.ENCDATAASYMM :
				return decodeEncDataAsymm(in);
			case Code.VERSICHERTENSTAMMDATEN :
				return decodeVersichertenstammdaten(in);
			case Code.UPDATEAUTH :
				return decodeUpdateAuth(in);
			case Code.ZERTIFIKATPATIENTDATA :
				return decodeZertifikatPatientData(in);
			case Code.PUBLICKEY :
				return decodePublicKey(in);
			case Code.PRIVATEKEY :
				return decodePrivateKey(in);
			case Code.ZERTIFIKATTTPDATA :
				return decodeZertifikatTTPData(in);
				// never reached
			default :
				stop();
		}
		return null;
	}

	public void compSize(Codeable c) throws Exception {
		switch (c.getCode()) {
			case Code.CARDDATA :

				compSizeCardData((CardData) c);

				break;
			case Code.UPDATE :

				compSizeUpdate((Update) c);

				break;
			case Code.NONCE :

				compSizeNonce((Nonce) c);

				break;
			case Code.ZERTIFIKATPATIENT :

				compSizeZertifikatPatient((ZertifikatPatient) c);

				break;
			case Code.SIGNEDDATA :

				compSizeSignedData((SignedData) c);

				break;
			case Code.ENCDATAASYMM :

				compSizeEncDataAsymm((EncDataAsymm) c);

				break;
			case Code.VERSICHERTENSTAMMDATEN :

				compSizeVersichertenstammdaten((Versichertenstammdaten) c);

				break;
			case Code.UPDATEAUTH :

				compSizeUpdateAuth((UpdateAuth) c);

				break;
			case Code.ZERTIFIKATPATIENTDATA :

				compSizeZertifikatPatientData((ZertifikatPatientData) c);

				break;
			case Code.PUBLICKEY :

				compSizePublicKey((PublicKey) c);

				break;
			case Code.PRIVATEKEY :

				compSizePrivateKey((PrivateKey) c);

				break;
			case Code.ZERTIFIKATTTPDATA :

				compSizeZertifikatTTPData((ZertifikatTTPData) c);

				break;
			// never reached
			default :

				stop();

		}
	}

	public PlainData decodePlainData(byte[] in, byte expectedType)
			throws Exception {
		encoding_length = 0;
		PlainData p = null;
		try {
			if (in[encoding_length] != expectedType) {
				stop();

			}
			switch (in[encoding_length]) {

				case Code.VERSICHERTENSTAMMDATEN :
					p = decodeVersichertenstammdaten(in);
					break;

			}
		} catch (ArrayIndexOutOfBoundsException e) {
			stop();

		}

		if (p == null) {
			stop();

		}
		return p;
	}

	public void encodeCardData(CardData c) throws Exception {
		encodeDestination[encoding_length++] = Code.CARDDATA;

		encode(c.getNonce());

		encode(c.getDaten());

		encode(c.getSigdaten());

		encode(c.getZertEGK());
	}

	public CardData decodeCardData(byte[] in) throws Exception {
		CardData c = new CardData();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setDaten(((EncDataAsymm) decode(in)));
		c.setSigdaten(((SignedData) decode(in)));
		c.setZertEGK(((ZertifikatPatient) decode(in)));
		return c;
	}

	private void compSizeCardData(CardData c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getDaten());
		compSize(c.getSigdaten());
		compSize(c.getZertEGK());
	}

	public void encodeUpdate(Update c) throws Exception {
		encodeDestination[encoding_length++] = Code.UPDATE;

		encode(c.getDaten());

		encode(c.getSigdaten());

		encode(c.getSigWithNonce());
	}

	public Update decodeUpdate(byte[] in) throws Exception {
		Update c = new Update();
		encoding_length++;
		c.setDaten(((EncDataAsymm) decode(in)));
		c.setSigdaten(((SignedData) decode(in)));
		c.setSigWithNonce(((SignedData) decode(in)));
		return c;
	}

	private void compSizeUpdate(Update c) throws Exception {
		encoding_length++;
		compSize(c.getDaten());
		compSize(c.getSigdaten());
		compSize(c.getSigWithNonce());
	}

	public void encodeNonce(Nonce c) throws Exception {
		encodeDestination[encoding_length++] = Code.NONCE;

		encodeByteArray((byte[]) c.getNonce());
	}

	public Nonce decodeNonce(byte[] in) throws Exception {
		Nonce c = new Nonce();
		encoding_length++;
		c.setNonce(decodeByteArray(in));
		return c;
	}

	private void compSizeNonce(Nonce c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getNonce());
	}

	public void encodeZertifikatPatient(ZertifikatPatient c) throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATPATIENT;

		encode(c.getSignedCert());

		encode(c.getData());
	}

	public ZertifikatPatient decodeZertifikatPatient(byte[] in)
			throws Exception {
		ZertifikatPatient c = new ZertifikatPatient();
		encoding_length++;
		c.setSignedCert(((SignedData) decode(in)));
		c.setData(((ZertifikatPatientData) decode(in)));
		return c;
	}

	private void compSizeZertifikatPatient(ZertifikatPatient c)
			throws Exception {
		encoding_length++;
		compSize(c.getSignedCert());
		compSize(c.getData());
	}

	public void encodeSignedData(SignedData c) throws Exception {
		encodeDestination[encoding_length++] = Code.SIGNEDDATA;

		encodeByteArray((byte[]) c.getSigned());
	}

	public SignedData decodeSignedData(byte[] in) throws Exception {
		SignedData c = new SignedData();
		encoding_length++;
		c.setSigned(decodeByteArray(in));
		return c;
	}

	private void compSizeSignedData(SignedData c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getSigned());
	}

	private void encodeEncDataAsymm(EncDataAsymm c) {

		encodeDestination[encoding_length++] = Code.ENCDATAASYMM;
		encodeByteArray(c.encrypted);

		encodeInt(c.plainlength);
		encodeInt(c.enclength);

		encodeByte(c.plainDataType);
	}

	public EncDataAsymm decodeEncDataAsymm(byte[] in) throws Exception {
		if (in[encoding_length++] != Code.ENCDATAASYMM)
			stop();
		EncDataAsymm into = new EncDataAsymm();
		//public byte[] encrypted;
		//public short plainlength;
		//public short enclength;
		//public byte plainDataType;
		into.encrypted = decodeByteArray(in);
		into.plainlength = decodeInt(in);
		into.enclength = decodeInt(in);
		into.plainDataType = decodeByte(in);
		return into;
	}

	private void compSizeEncDataAsymm(EncDataAsymm c) {
		encoding_length++;
		compSizeByteArray(c.encrypted);
		encoding_length += 8; // short + short + byte
	}

	public void encodeVersichertenstammdaten(Versichertenstammdaten c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.VERSICHERTENSTAMMDATEN;

		encodeString((String) c.getName());

		encodeString((String) c.getVersichertennr());

		encodeInt((int) c.getKennummerKrankenkasse());

		encodeString((String) c.getAdresse());
	}

	public Versichertenstammdaten decodeVersichertenstammdaten(byte[] in)
			throws Exception {
		Versichertenstammdaten c = new Versichertenstammdaten();
		encoding_length++;
		c.setName(decodeString(in));
		c.setVersichertennr(decodeString(in));
		c.setKennummerKrankenkasse(decodeInt(in));
		c.setAdresse(decodeString(in));
		return c;
	}

	private void compSizeVersichertenstammdaten(Versichertenstammdaten c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());

		compSizeString(c.getVersichertennr());

		compSizeInt(c.getKennummerKrankenkasse());

		compSizeString(c.getAdresse());
	}

	public void encodeUpdateAuth(UpdateAuth c) throws Exception {
		encodeDestination[encoding_length++] = Code.UPDATEAUTH;

		encode(c.getNonce());

		encode(c.getData());
	}

	public UpdateAuth decodeUpdateAuth(byte[] in) throws Exception {
		UpdateAuth c = new UpdateAuth();
		encoding_length++;
		c.setNonce(((Nonce) decode(in)));
		c.setData(((Versichertenstammdaten) decode(in)));
		return c;
	}

	private void compSizeUpdateAuth(UpdateAuth c) throws Exception {
		encoding_length++;
		compSize(c.getNonce());
		compSize(c.getData());
	}

	public void encodeZertifikatPatientData(ZertifikatPatientData c)
			throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATPATIENTDATA;

		encodeString((String) c.getName());

		encode(c.getPubkey());

		encodeInt((int) c.getVersichertennr());

		encodeInt((int) c.getKrankenkassenID());

		encodeInt((int) c.getId());
	}

	public ZertifikatPatientData decodeZertifikatPatientData(byte[] in)
			throws Exception {
		ZertifikatPatientData c = new ZertifikatPatientData();
		encoding_length++;
		c.setName(decodeString(in));
		c.setPubkey(((PublicKey) decode(in)));
		c.setVersichertennr(decodeInt(in));
		c.setKrankenkassenID(decodeInt(in));
		c.setId(decodeInt(in));
		return c;
	}

	private void compSizeZertifikatPatientData(ZertifikatPatientData c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());
		compSize(c.getPubkey());

		compSizeInt(c.getVersichertennr());

		compSizeInt(c.getKrankenkassenID());

		compSizeInt(c.getId());
	}

	private void encodePublicKey(PublicKey c) throws Exception {
		encodeDestination[encoding_length++] = Code.PUBLICKEY;
		encodeByteArray(c.getModulus());

	}

	public PublicKey decodePublicKey(byte[] in) throws Exception {
		if (in[encoding_length++] != Code.PUBLICKEY)
			stop();
		PublicKey res = new PublicKey();
		res.setKey(decodeByteArray(in));
		return res;
	}

	private void compSizePublicKey(PublicKey c) throws Exception {
		encoding_length++;
		compSizeByteArray(c.getModulus());

	}

	public void encodePrivateKey(PrivateKey c) throws Exception {
		encodeDestination[encoding_length++] = Code.PRIVATEKEY;

		encodeByteArray(c.getModulus());
		encodeByteArray(c.getPrivexponent());

	}

	public PrivateKey decodePrivateKey(byte[] in) throws Exception {
		if (in[encoding_length++] != Code.PRIVATEKEY)
			stop();
		PrivateKey res = new PrivateKey();
		res.setKey(decodeByteArray(in), decodeByteArray(in));
		return res;
	}

	private void compSizePrivateKey(PrivateKey c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getModulus());
		compSizeByteArray(c.getPrivexponent());

	}

	public void encodeZertifikatTTPData(ZertifikatTTPData c) throws Exception {
		encodeDestination[encoding_length++] = Code.ZERTIFIKATTTPDATA;

		encodeString((String) c.getName());

		encode(c.getPubkey());
	}

	public ZertifikatTTPData decodeZertifikatTTPData(byte[] in)
			throws Exception {
		ZertifikatTTPData c = new ZertifikatTTPData();
		encoding_length++;
		c.setName(decodeString(in));
		c.setPubkey(((PublicKey) decode(in)));
		return c;
	}

	private void compSizeZertifikatTTPData(ZertifikatTTPData c)
			throws Exception {
		encoding_length++;

		compSizeString(c.getName());
		compSize(c.getPubkey());
	}

	private void stop() throws ServiceException {
		throw new ServiceException("Coding");
	}
}
